if (!Ext.ClassManager.isCreated("Portfolio.dynapp.model.Affiliation")) {

	Ext.define("Portfolio.dynapp.model.Affiliation", {

		extend: "Ext.data.Model",

		alternateClassName: [
			"Ext.dynapp.model.Affiliation"
		],

		fields: [
			{ type: "string", name: "name" },
			{ type: "string", name: "url" }
		]
	});
}
