if (!Ext.ClassManager.isCreated("Ext.ux.navigation.model.RolePermissionTree")) {
	Ext.define("Ext.ux.navigation.model.RolePermissionTree", {
		extend: "Ext.data.Model",
		fields: [
			{ name: "id", type: "auto" },
			{ name: "text", type: "auto" },
			{ name: "url", type: "string" },
			{ name: "leaf", type: "bool" }
		],
		idProperty: "id",

		proxy: {
			paramsAsJson: true,
			type: "ajax",
			url: "/ip2-services/ip2transaction.aspx.json",
			reader: {
				type: "json",
				headers: {
					"Accept": "application/json"
				},
				rootProperty: "children",
				totalProperty: "totalRows"
			},
			actionMethods: {
				read: "POST"
			}
		}
	});
}
