
/**
 * @class Ext.ux.ip2.dmd.columnsExplorer.Tree
 * Class to create.....
 * @extend Ext.tree.Panel
 * @alias dmdColumnsTree
 * 
 */
Ext.define('Ext.ux.dmd.columnsexplorer.Tree', {
	
	extend: 'Ext.tree.Panel',
	alias: ['widget.dmdColumnsTree'],
	

	columnsData: null,	// <-- This will be set to an empty Object in initComponent

	getColumns: function() {
		return this.columnsData;
	},

	// Load columns to the view
	loadColumns: function(data) {

		var tree = this,
			root;

		// Function that creates the root node from data
		function createRootNode() {

		}

		return tree;
	},

	// Clear the columns from the view
	clearColumns: function() {

		var tree = this;

		tree.setColumns(null);

		return tree;
	},

	insertColumn: function(data) {

		var tree = this;

	},



	initComponent: function() {

		this.callParent(arguments);

		var tree = this;


	}


});