Ext.define("Portfolio.registry.App", {

	extend: "Ext.panel.Panel",
	layout: "fit",

	alias: "widget.registry-app",

	options: {},

	dataStore: null,

	alternateClassName: [
		"Ext.registry.app"
	],

	requires: [
		"Portfolio.registry.view.Main",
		"Portfolio.registry.util.Service"
	],

	initComponent: function() {

		var i_self = this;
		i_self.callParent();

		var o_options = i_self.options;

		var i_panel = Ext.widget("registry-main", {
			options: i_self.options,
			itemId: "registry-main",
			dataStore: i_self.dataStore || ""
		});

		i_self.add(i_panel);

		return i_self;
	}
});
