Ext.define('Ext.ux.themed.ActionColumn', {

	requires: [
		'Ext.grid.column.Action'
	],

	extend: 'Ext.grid.column.Action',
	
	alias: ['widget.themedactioncolumn'],
	
	constructor: function (o_config) {

		var i_cmp = this,
			s_icon = o_config.iconCls || o_config.icon || '',
			s_iconCls = 'icon icon-' + s_icon.toLowerCase();

		o_config.iconCls = s_iconCls;
		o_config.items = o_config.items || i_cmp.items || [{}];
		o_config.icon = null;
		o_config.width = o_config.width || 25;

		i_cmp.callParent([o_config]);

		return i_cmp;
	},

	initComponent: function () {

		var i_cmp = this;

		i_cmp.callParent();

		return i_cmp;
	}
});
