Ext.define('Ext.ux.stackorganizer.column.TreeIcon', {

	extend: 'Ext.tree.Column',

	alias: ['widget.stackicontreecolumn'],

	mixins: {
		'iconRenderer': 'Ext.ux.stackorganizer.column.IconRenderer'
	},

	initComponent: function () {

		var i_cmp = this;

		i_cmp.callParent(arguments);

		i_cmp.initIconRenderer();

		return i_cmp;
	}
});
