Ext.define("Ext.ux.form.field.ISOTimestampDisplayfield", {
	extend: "Ext.form.field.Display",
	alias: "widget.isotimestampdisplayfield",

	config: {
		isoField: null
	},

	initComponent: function () {

		var i_self = this;

		var o_config = i_self.initialConfig;
		var i_field = Ext.widget("isotimestampfield", o_config);
		i_field.isFormField = false;
		i_self.setIsoField(i_field);
		i_self.callParent();
	},

	setValue: function(value) {

		var i_self = this;
		var i_isofield = i_self.getIsoField();

		i_isofield.setValue(value);

		i_self.callParent([i_isofield.getRawValue()]);
	},

	getValue: function() {

		var i_self = this;
		var i_isofield = i_self.getIsoField();

		return i_isofield.getValue();
	},

	getSubmitValue: function() {

		var i_self = this;
		var i_isofield = i_self.getIsoField();

		return i_isofield.getValue();
	}
});
