/*	Icons for Markdown */
.markdown-h1 { background: url(icons/heading_1.png); }
.markdown-h2 { background: url(icons/heading_2.png); }
.markdown-h3 { background: url(icons/heading_3.png); }
.markdown-h4 { background: url(icons/heading_4.png); }
.markdown-h5 { background: url(icons/heading_5.png); }
.markdown-h6 { background: url(icons/heading_6.png); }
.markdown-bold { background: url(icons/bold.png); }
.markdown-italic { background: url(icons/italic.png); }
.markdown-ulist { background: url(icons/ul.png); }
.markdown-olist { background: url(icons/ol.png); }
.markdown-link { background: url(icons/link.png); }
.markdown-undo { background: url(icons/arrow_undo.png); }
.markdown-redo { background: url(icons/arrow_redo.png); }
.markdown-hr { background: url(icons/next-column.png); }
.markdown-code { background: url(icons/code.png); }
.markdown-image { background: url(icons/image.png); }
.markdown-quote { background: url(icons/quote.png); }
.markdown-sup { background: url(icons/text_superscript.png); }