.ip2-designer-form {
	background-image: url(/system/shared/collection/icons/16x16/plain/window.png) !important;
}
.ip2-designer-portlet {
	background-image: url(/system/shared/collection/icons/16x16/plain/window_colors.png) !important;
}
.ip2-designer-portlet-add {
	background-image: url(/system/shared/collection/icons/16x16/plain/portlet_add.png) !important;
}
.ip2-designer-column {
	background-image: url(/system/shared/collection/icons/16x16/plain/column.png) !important;
}
.ip2-designer-column-add {
	background-image: url(/system/shared/collection/icons/16x16/plain/column_add.png) !important;
}
.ip2-designer-field {
	background-image: url(/system/shared/collection/icons/16x16/plain/data.png) !important;
}
.ip2-designer-relation-field {
	background-image: url(/system/shared/collection/icons/16x16/plain/connection.png) !important;
}
.ip2-designer-entity {
	background-image: url(/system/shared/collection/icons/16x16/plain/table.png) !important;
}
.ip2-designer-parentRelation {
	background-image: url(/system/shared/collection/icons/16x16/plain/link.png) !important;
}
.ip2-designer-affiliations {
	background-image: url(/system/shared/collection/icons/16x16/plain/component.png) !important;
}
.ip2-designer-widgets {
	background-image: url(/system/shared/collection/icons/16x16/plain/gear.png) !important;
}

.ip2-icon-tab {
	background-position: 0 -2px;
}

.ip2-icon-file {
	background-image:url(/system/shared/collection/icons/16x16/plain/document.png) !important;
}
.ip2-icon-new-file {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_add.png) !important;
}
.ip2-icon-unsaved-file {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_new.png) !important;
}
.ip2-icon-folder {
	background-image:url(/system/shared/collection/icons/16x16/plain/folder.png) !important;
}
.ip2-icon-new-folder {
	background-image:url(/system/shared/collection/icons/16x16/plain/folder_add.png) !important;
}
.ip2-icon-unsaved-folder {
	background-image:url(/system/shared/collection/icons/16x16/plain/folder_new.png) !important;
}

.ip2-icon-up {
	background-image:url(/system/shared/collection/icons/16x16/plain/arrow_up_green.png) !important;
}

.ip2-icon-folder-href {
	background-image:url(/system/shared/collection/icons/16x16/plain/folder_window.png) !important;
}
.ip2-icon-folder-page {
	background-image:url(/system/shared/collection/icons/16x16/plain/folder_document.png) !important;
}
.ip2-icon-view {
	background-image:url(/system/shared/collection/icons/16x16/plain/magnifying_glass.png) !important;
}
.ip2-icon-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/add.png) !important;
}
.ip2-icon-edit {
	background-image:url(/system/shared/collection/icons/16x16/plain/edit.png) !important;
}
.ip2-icon-copy {
	background-image:url(/system/shared/collection/icons/16x16/plain/copy.png) !important;
}
.ip2-icon-documents {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_orientation_portrait.png) !important;
}
.ip2-icon-delete {
	background-image:url(/system/shared/collection/icons/16x16/plain/error.png) !important;
}
.ip2-icon-remove {
	background-image:url(/system/shared/collection/icons/16x16/plain/delete.png) !important;
}
.ip2-icon-deselect {
	background-image:url(/system/shared/collection/icons/16x16/plain/row.png) !important;
}
.ip2-icon-property {
	background-image:url(/system/shared/collection/icons/16x16/plain/gear.png) !important;
}
.ip2-icon-gear {
	background-image:url(/system/shared/collection/icons/16x16/plain/gear.png) !important;
}
.ip2-icon-gear-edit {
	background-image:url(/system/shared/collection/icons/16x16/plain/gear_edit.png) !important;
}
.ip2-icon-refresh {
	background-image:url(/system/shared/collection/icons/16x16/plain/refresh.png) !important;
}
.ip2-icon-undo {
	background-image:url(/system/shared/collection/icons/16x16/plain/undo.png) !important;
}
.ip2-icon-checks {
	background-image:url(/system/shared/collection/icons/16x16/plain/checks.png) !important;
}


.ip2-icon-row {
	background-image:url(/system/shared/collection/icons/16x16/plain/row.png) !important;
}
.ip2-icon-row-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_add.png) !important;
}
.ip2-icon-row-add-after {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_add_after.png) !important;
}
.ip2-icon-row-add-before {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_add_before.png) !important;
}
.ip2-icon-row-delete {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_delete.png) !important;
}
.ip2-icon-row-edit {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_edit.png) !important;
}
.ip2-icon-row-preferences {
	background-image:url(/system/shared/collection/icons/16x16/plain/row_preferences.png) !important;
}

.ip2-icon-expand {
	background-image:url(/system/shared/collection/icons/16x16/plain/elements_tree.png) !important;
}
.ip2-icon-collapse {
	background-image:url(/system/shared/collection/icons/16x16/plain/text.png) !important;
}
.ip2-icon-save {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue.png) !important;
}
.ip2-icon-close {
	background-image:url(/system/shared/collection/icons/16x16/plain/door2.png) !important;
}
.ip2-icon-restart {
	background-image:url(/system/shared/collection/icons/16x16/plain/nav_refresh_green.png) !important;
}
.ip2-icon-search {
	background-image:url(/system/shared/collection/icons/16x16/plain/find.png) !important;
}
.ip2-icon-list {
	background-image:url(/system/shared/collection/icons/16x16/plain/text_align_justified.png) !important;
}
.ip2-icon-add-relation {
	background-image:url(/system/shared/collection/icons/16x16/plain/link_add.png) !important;
}
.ip2-icon-add-entity {
	background-image:url(/system/shared/collection/icons/16x16/plain/text.png) !important;
}
.ip2-icon-add-link {
	background-image:url(/system/shared/collection/icons/16x16/plain/link.png) !important;
}
.ip2-icon-add-dmd {
	background-image:url(/system/shared/collection/icons/16x16/plain/cube_molecule.png) !important;
}
.ip2-icon-import-entity {
	background-image:url(/system/shared/collection/icons/16x16/plain/import1.png) !important;
}
.ip2-icon-new-domain-model {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_empty.png) !important;
}
.ip2-icon-nav-redo-blue {
	background-image:url(/system/shared/collection/icons/16x16/plain/nav_redo_blue.png) !important;
}
.ip2-icon-nav-undo-blue {
	background-image:url(/system/shared/collection/icons/16x16/plain/nav_undo_blue.png) !important;
}
.ip2-icon-open-domain-model {
	background-image:url(/system/shared/collection/icons/16x16/plain/server_into.png) !important;
}
.ip2-icon-save-domain-model {
	background-image:url(/system/shared/collection/icons/16x16/plain/save_as.png) !important;
}
.ip2-icon-close-domain-model {
	background-image:url(/system/shared/collection/icons/16x16/plain/close.png) !important;
}
.ip2-icon-domain-model-properties {
	background-image:url(/system/shared/collection/icons/16x16/plain/window_gear.png) !important;
}
.ip2-icon-domain-model-newEntity {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_gear.png) !important;
}
.ip2-icon-import-table {
	background-image:url(/system/shared/collection/icons/16x16/plain/server_from_client.png) !important;
}
.ip2-icon-basedatatypes {
	background-image:url(/system/shared/collection/icons/16x16/plain/dictionary.png) !important;
}

.ip2-icon-branch-find {
	background-image:url(/system/shared/collection/icons/16x16/plain/branch_find.png) !important;
}
.ip2-icon-branch-view {
	background-image:url(/system/shared/collection/icons/16x16/plain/branch_view.png) !important;
}

.ip2-icon-bullet-delete {
	 background-image:url(/system/shared/collection/icons/16x16/plain/bullet_delete.png) !important;
}
.ip2-icon-bullet-picture {
	 background-image:url(/system/shared/collection/icons/16x16/plain/bullet_picture.png) !important;
}


.ip2-icon-bullet-ball-green {
	 background-image:url(/system/shared/collection/icons/16x16/plain/bullet_ball_green.png) !important;
}
.ip2-icon-bullet-ball-red {
	 background-image:url(/system/shared/collection/icons/16x16/plain/bullet_ball_red.png) !important;
}

.ip2-icon-translate {
	background-image:url(/system/images/flags/16x16/flag_generic.png) !important;
}
.ip2-icon-menuOrganizer {
	background-image:url(/system/shared/collection/icons/16x16/plain/text_tree.png) !important;
}
.ip2-icon-sourceEditor {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_edit.png) !important;
}

.ip2-icon-undo {
	background-image:url(/system/shared/collection/icons/16x16/plain/undo.png) !important;
}
.ip2-icon-redo {
	background-image:url(/system/shared/collection/icons/16x16/plain/redo.png) !important;
}

.ip2-icon-window-colors {
		background-image:url(/system/shared/collection/icons/16x16/plain/window_colors.png) !important;
}

.ip2-icon-window-application {
		background-image:url(/system/shared/collection/icons/16x16/plain/window_application.png) !important;
}
.ip2-icon-window-application-edit {
		background-image:url(/system/shared/collection/icons/16x16/plain/window_application_edit.png) !important;
}

.ip2-icon-exit {
	background-image:url(/system/shared/collection/icons/16x16/plain/exit.png) !important;
}



.ip2-icon-icecube {
	background-image: url(/system/shared/collection/icons/16x16/plain/icecube.png) !important;
}

.ip2-icon-data {
	background-image: url(/system/shared/collection/icons/16x16/plain/data.png) !important;
}
.ip2-icon-data-add {
	background-image: url(/system/shared/collection/icons/16x16/plain/add.png) !important;
}
.ip2-icon-data-connection {
	background-image: url(/system/shared/collection/icons/16x16/plain/connection.png) !important;
}
.ip2-icon-data-certificate {
	background-image: url(/system/shared/collection/icons/16x16/plain/certificate.png) !important;
}
.ip2-icon-data-copy {
	background-image: url(/system/shared/collection/icons/16x16/plain/copy.png) !important;
}
.ip2-icon-data-delete {
	background-image: url(/system/shared/collection/icons/16x16/plain/delete.png) !important;
}
.ip2-icon-data-disk {
	background-image: url(/system/shared/collection/icons/16x16/plain/disk.png) !important;
}
.ip2-icon-data-down {
	background-image: url(/system/shared/collection/icons/16x16/plain/down.png) !important;
}
.ip2-icon-data-edit {
	background-image: url(/system/shared/collection/icons/16x16/plain/edit.png) !important;
}
.ip2-icon-data-error {
	background-image: url(/system/shared/collection/icons/16x16/plain/error.png) !important;
}
.ip2-icon-data-find {
	background-image: url(/system/shared/collection/icons/16x16/plain/find.png) !important;
}
.ip2-icon-data-forbidden {
	background-image: url(/system/shared/collection/icons/16x16/plain/forbidden.png) !important;
}
.ip2-icon-data-gear {
	background-image: url(/system/shared/collection/icons/16x16/plain/gear.png) !important;
}
.ip2-icon-data-information {
	background-image: url(/system/shared/collection/icons/16x16/plain/information.png) !important;
}
.ip2-icon-data-into {
	background-image: url(/system/shared/collection/icons/16x16/plain/into.png) !important;
}
.ip2-icon-data-lock {
	background-image: url(/system/shared/collection/icons/16x16/plain/lock.png) !important;
}
.ip2-icon-data-new {
	background-image: url(/system/shared/collection/icons/16x16/plain/new.png) !important;
}
.ip2-icon-data-next {
	background-image: url(/system/shared/collection/icons/16x16/plain/next.png) !important;
}
.ip2-icon-data-out {
	background-image: url(/system/shared/collection/icons/16x16/plain/out.png) !important;
}
.ip2-icon-data-ok {
	background-image: url(/system/shared/collection/icons/16x16/plain/ok.png) !important;
}
.ip2-icon-data-preferences {
	background-image: url(/system/shared/collection/icons/16x16/plain/preferences.png) !important;
}
.ip2-icon-data-previous {
	background-image: url(/system/shared/collection/icons/16x16/plain/previous.png) !important;
}
.ip2-icon-data-refresh {
	background-image: url(/system/shared/collection/icons/16x16/plain/arrow_circle2.png) !important;
}
.ip2-icon-data-replace {
	background-image: url(/system/shared/collection/icons/16x16/plain/arrow_circle2.png) !important;
}
.ip2-icon-data-scroll {
	background-image: url(/system/shared/collection/icons/16x16/plain/scroll_bar_vertical.png) !important;
}
.ip2-icon-data-table {
	background-image: url(/system/shared/collection/icons/16x16/plain/data_table.png) !important;
}
.ip2-icon-data-time {
	background-image: url(/system/shared/collection/icons/16x16/plain/calendar_clock.png) !important;
}
.ip2-icon-data-unknown {
	background-image: url(/system/shared/collection/icons/16x16/plain/symbol_questionmark.png) !important;
}
.ip2-icon-data-up {
	background-image: url(/system/shared/collection/icons/16x16/plain/up.png) !important;
}
.ip2-icon-data-view {
	background-image: url(/system/shared/collection/icons/16x16/plain/view.png) !important;
}
.ip2-icon-data-warning {
	background-image: url(/system/shared/collection/icons/16x16/plain/warning.png) !important;
}





.ip2-icon-cube-molecule {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule.png) !important;
}
.ip2-icon-cube-molecule-add {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_add.png) !important;
}
.ip2-icon-cube-molecule-connection {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_connection.png) !important;
}
.ip2-icon-cube-molecule-certificate {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_certificate.png) !important;
}
.ip2-icon-cube-molecule-copy {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_copy.png) !important;
}
.ip2-icon-cube-molecule-delete {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_delete.png) !important;
}
.ip2-icon-cube-molecule-disk {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_disk.png) !important;
}
.ip2-icon-cube-molecule-down {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_down.png) !important;
}
.ip2-icon-cube-molecule-edit {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_edit.png) !important;
}
.ip2-icon-cube-molecule-error {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_error.png) !important;
}
.ip2-icon-cube-molecule-find {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_find.png) !important;
}
.ip2-icon-cube-molecule-forbidden {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_forbidden.png) !important;
}
.ip2-icon-cube-molecule-gear {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_gear.png) !important;
}
.ip2-icon-cube-molecule-information {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_information.png) !important;
}
.ip2-icon-cube-molecule-into {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_into.png) !important;
}
.ip2-icon-cube-molecule-lock {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_lock.png) !important;
}
.ip2-icon-cube-molecule-new {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_new.png) !important;
}
.ip2-icon-cube-molecule-next {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_next.png) !important;
}
.ip2-icon-cube-molecule-out {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_out.png) !important;
}
.ip2-icon-cube-molecule-ok {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_ok.png) !important;
}
.ip2-icon-cube-molecule-preferences {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_preferences.png) !important;
}
.ip2-icon-cube-molecule-previous {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_previous.png) !important;
}
.ip2-icon-cube-molecule-refresh {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_refresh.png) !important;
}
.ip2-icon-cube-molecule-replace {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_replace.png) !important;
}
.ip2-icon-cube-molecule-scroll {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_scroll.png) !important;
}
.ip2-icon-cube-molecule-table {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_table.png) !important;
}
.ip2-icon-cube-molecule-time {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_time.png) !important;
}
.ip2-icon-cube-molecule-unknown {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_unknown.png) !important;
}
.ip2-icon-cube-molecule-up {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_up.png) !important;
}
.ip2-icon-cube-molecule-view {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_view.png) !important;
}
.ip2-icon-cube-molecule-warning {
	background-image: url(/system/shared/collection/icons/16x16/plain/cube_molecule_warning.png) !important;
}

.ip2-icon-link {
	background-image:url(/system/shared/collection/icons/16x16/plain/link.png) !important;
}
.ip2-icon-link-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/link_add.png) !important;
}
.ip2-icon-link-new {
	background-image:url(/system/shared/collection/icons/16x16/plain/link_new.png) !important;
}


.ip2-icon-table {
	background-image:url(/system/shared/collection/icons/16x16/plain/table.png) !important;
}
.ip2-icon-table-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/table_add.png) !important;
}
.ip2-icon-table-new {
	background-image:url(/system/shared/collection/icons/16x16/plain/table_new.png) !important;
}
.ip2-icon-table-selection-column {
	background-image:url(/system/shared/collection/icons/16x16/plain/table_selection_column.png) !important;
}
.ip2-icon-table-view {
	background-image:url(/system/shared/collection/icons/16x16/plain/table_view.png) !important;
}


.ip2-icon-flag-da, .ip2-icon-flag-denmark {
	background-image: url(/system/shared/collection/icons/16x16/plain/flag_denmark.png) !important;
}
.ip2-icon-flag-en, .ip2-icon-flag-great-britain {
	background-image: url(/system/shared/collection/icons/16x16/plain/flag_great_britain.png) !important;
}
.ip2-icon-flag-de, .ip2-icon-flag-germany {
	background-image: url(/system/shared/collection/icons/16x16/plain/flag_germany.png) !important;
}
.ip2-icon-flag-sv, .ip2-icon-flag-sweden {
	background-image: url(/system/shared/collection/icons/16x16/plain/flag_sweden.png) !important;
}
.ip2-icon-flag-no, .ip2-icon-flag-norway {
	background-image: url(/system/shared/collection/icons/16x16/plain/flag_norway.png) !important;
}






.ip2-icon-disk-blue {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue.png) !important;
}
.ip2-icon-disk-blue-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_add.png) !important;
}
.ip2-icon-disk-blue-delete {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_delete.png) !important;
}
.ip2-icon-disk-blue-edit {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_edit.png) !important;
}
.ip2-icon-disk-blue-error {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_error.png) !important;
}
.ip2-icon-disk-blue-information {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_information.png) !important;
}
.ip2-icon-disk-blue-ok {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_ok.png) !important;
}
.ip2-icon-disk-blue-warning {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_warning.png) !important;
}
.ip2-icon-disk-blue-delete {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_delete.png) !important;
}
.ip2-icon-disk-blue-window {
	background-image:url(/system/shared/collection/icons/16x16/plain/disk_blue_window.png) !important;
}

.ip2-icon-find-text {
	background-image:url(/system/shared/collection/icons/16x16/plain/find_text.png) !important;
}

.ip2-icon-replace {
	background-image:url(/system/shared/collection/icons/16x16/plain/replace.png) !important;
}
.ip2-icon-text-code-colored {
	background-image:url(/system/shared/collection/icons/16x16/plain/text_code_colored.png) !important;
}

.ip2-icon-font {
	background-image:url(/system/shared/collection/icons/16x16/plain/font.png) !important;
}

.ip2-icon-1 {
	background-image:url(/system/shared/collection/icons/16x16/plain/1.png) !important;
}
.ip2-icon-2 {
	background-image:url(/system/shared/collection/icons/16x16/plain/2.png) !important;
}
.ip2-icon-3 {
	background-image:url(/system/shared/collection/icons/16x16/plain/3.png) !important;
}
.ip2-icon-4 {
	background-image:url(/system/shared/collection/icons/16x16/plain/4.png) !important;
}
.ip2-icon-5 {
	background-image:url(/system/shared/collection/icons/16x16/plain/5.png) !important;
}

.x-form-itemselector-top {
		background-image: url(/system/shared/collection/icons/16x16/plain/top.png);
}
.x-form-itemselector-up {
		background-image: url(/system/shared/collection/icons/16x16/plain/up.png);
}
.x-form-itemselector-add {
		background-image: url(/system/shared/collection/icons/16x16/plain/right.png);
}
.x-form-itemselector-remove {
		background-image: url(/system/shared/collection/icons/16x16/plain/left.png);
}
.x-form-itemselector-down {
		background-image: url(/system/shared/collection/icons/16x16/plain/down.png);
}
.x-form-itemselector-bottom {
		background-image: url(/system/shared/collection/icons/16x16/plain/bottom.png);
}
.x-form-accept-trigger {
	background-image: url(syste/shared/ext4/resources/themes/images/grey/form/trigger-accept.png);
}
