if (!Ext.ClassManager.isCreated("Portfolio.designer.model.ip2DomainModelHistory")) {
	Ext.define("Portfolio.designer.model.ip2DomainModelHistory", {
		extend: "Ext.data.Model",
		proxy: {
			type: "ajax",
			url: "/ip2-services/ip2navigator.aspx?" +   Ext.Object.toQueryString({
				func: "database",
				dbName: "navhis",
				action: "list",
				histyp: "dmd",
				distinctValue: true
			})
		}
	});
};
