if (!Ext.ClassManager.isCreated("Ext.ux.icon.view.model.DataView")) {
	Ext.define("Ext.ux.icon.view.model.DataView", {
		extend: "Ext.data.Model",
		fields: [
			{ name: "keyword", type: "auto" },
			{ name: "icon", type: "auto" }
		],
		idProperty: "icon",
		proxy: {
			url: "/ip2-services/ip2transaction.aspx",
			type: "ajax",
			paramsAsJson: true,
			$configStrict: false,
			reader: {
				type: "json",
				rootProperty: "rows",
				totalProperty: "totalRows"
			},
			getMethod: function(request) {
				return "POST";
			}
		}
	});
}
