Ext.define("Ext.ux.serppreview.FieldContainer", {
	extend: "Ext.form.FieldContainer",
	alias: "widget.serppreviewfield",

	// THIS SHOULD NOT BE NEEDED
	mixins: {
		field: "Ext.form.field.Field"
	},

	isFormField: false,

	config: {
		htmlContainer: null
	},

	initComponent: function() {

		var i_self = this;

		i_self.isFormField = false;

		i_self.callParent();

		i_self.initField();

		var i_htmlcontainer = Ext.widget("serppreviewcontainer", {
			titleField: i_self._xpdConfig.titleField,
			urlField: i_self._xpdConfig.urlField,
			descriptionField: i_self._xpdConfig.descriptionField,
			keywordsField: i_self._xpdConfig.keywordsField
		});

		i_self.setHtmlContainer(i_htmlcontainer);
		i_self.add(i_htmlcontainer);

		return i_self;
	},

	changeLanguage: function() {

		var i_self = this;
		i_self.getHtmlContainer().changeLanguage();
	},

	getValue: function() {
		return "";
	},

	setValue: function() {
		var test = "";
	},

	isDirty: function() {
		return false
	},

	isValid: function() {
		return true;
	}
});
