Ext.ns('Ext.ux.folderorganizer');
Ext.ux.folderorganizer.Alert = function(cfg) {
	cfg = cfg || {};

	Ext.applyIf(cfg, {
		buttons: Ext.Msg.OK,
		icon: Ext.Msg.WARNING
	});

	Ext.MessageBox.show(cfg);
}
