.icon:before {
	 width: 16px;
	 height: 11px;
	 vertical-align: middle;
}
.icon.trigger:before {
	 vertical-align: initial;
}
.flag:before {
	 content: "" !important;
	 display: inline-block;
}
.flag.af:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-za");
}
.flag.am:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-et");
}
.flag.ar:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ae");
}
.flag.arn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cl");
}
.flag.as:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.ba:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ru");
}
.flag.be:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-by");
}
.flag.bg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bg");
}
.flag.bn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bd");
}
.flag.bo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cn");
}
.flag.br:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fr");
}
.flag.ca:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-es");
}
.flag.co:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fr");
}
.flag.cs:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cz");
}
.flag.cy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb");
}
.flag.da:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-dk");
}
.flag.de:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-de");
}
.flag.dv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mv");
}
.flag.el:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gr");
}
.flag.en:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb");
}
.flag.es:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-es");
}
.flag.et:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ee");
}
.flag.fa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ir");
}
.flag.fi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fi");
}
.flag.fil:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ph");
}
.flag.fo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fo");
}
.flag.fr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fr");
}
.flag.fy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nl");
}
.flag.ga:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ie");
}
.flag.gu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.he:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-il");
}
.flag.hi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.hr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hr");
}
.flag.hsb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-de");
}
.flag.hu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hu");
}
.flag.hy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-am");
}
.flag.id:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-id");
}
.flag.ig:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ng");
}
.flag.ii:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cn");
}
.flag.is:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-is");
}
.flag.it:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-it");
}
.flag.ja:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-jp");
}
.flag.ka:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ge");
}
.flag.kk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kz");
}
.flag.kl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gl");
}
.flag.km:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kh");
}
.flag.kn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.kok:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.ko:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kr");
}
.flag.ky:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kg");
}
.flag.lb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lu");
}
.flag.lo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-la");
}
.flag.lt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lt");
}
.flag.lv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lv");
}
.flag.mi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nz");
}
.flag.mk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mk");
}
.flag.ml:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.mn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mn");
}
.flag.moh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ca");
}
.flag.mr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.ms:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bn");
}
.flag.ms:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-my");
}
.flag.mt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mt");
}
.flag.nb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-no");
}
.flag.ne:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-np");
}
.flag.nl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nl");
}
.flag.nn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-no");
}
.flag.nso:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-za");
}
.flag.oc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fr");
}
.flag.or:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.pa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.pl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pl");
}
.flag.prs:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-af");
}
.flag.ps:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-af");
}
.flag.pt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-br");
}
.flag.pt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pt");
}
.flag.qut:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gt");
}
.flag.quz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bo");
}
.flag.rm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ch");
}
.flag.ro:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ro");
}
.flag.ru:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ru");
}
.flag.rw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-rw");
}
.flag.sah:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ru");
}
.flag.sa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.se:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-se");
}
.flag.si:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lk");
}
.flag.sk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sk");
}
.flag.sl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-si");
}
.flag.sq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-al");
}
.flag.sv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-se");
}
.flag.sw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ke");
}
.flag.syr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sy");
}
.flag.ta:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.te:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.th:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-th");
}
.flag.tk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tm");
}
.flag.tn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-za");
}
.flag.tr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tr");
}
.flag.tt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ru");
}
.flag.ug:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cn");
}
.flag.uk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ua");
}
.flag.ur:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pk");
}
.flag.vi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vn");
}
.flag.wo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sn");
}
.flag.xh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-za");
}
.flag.yo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ng");
}
.flag.zh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tw");
}
.flag.ad:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ad");
}
.flag.ae:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ae");
}
.flag.af:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-af");
}
.flag.ag:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ag");
}
.flag.ai:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ai");
}
.flag.al:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-al");
}
.flag.am:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-am");
}
.flag.ao:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ao");
}
.flag.aq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-aq");
}
.flag.ar:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ar");
}
.flag.as:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-as");
}
.flag.at:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-at");
}
.flag.au:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-au");
}
.flag.aw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-aw");
}
.flag.ax:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ax");
}
.flag.az:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-az");
}
.flag.ba:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ba");
}
.flag.bb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bb");
}
.flag.bd:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bd");
}
.flag.be:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-be");
}
.flag.bf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bf");
}
.flag.bg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bg");
}
.flag.bh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bh");
}
.flag.bi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bi");
}
.flag.bj:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bj");
}
.flag.bl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bl");
}
.flag.bm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bm");
}
.flag.bn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bn");
}
.flag.bo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bo");
}
.flag.bq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bq");
}
.flag.br:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-br");
}
.flag.bs:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bs");
}
.flag.bt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bt");
}
.flag.bv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bv");
}
.flag.bw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bw");
}
.flag.by:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-by");
}
.flag.bz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-bz");
}
.flag.ca:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ca");
}
.flag.cc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cc");
}
.flag.cd:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cd");
}
.flag.cf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cf");
}
.flag.cg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cg");
}
.flag.ch:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ch");
}
.flag.ci:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ci");
}
.flag.ck:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ck");
}
.flag.cl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cl");
}
.flag.cm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cm");
}
.flag.cn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cn");
}
.flag.co:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-co");
}
.flag.cr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cr");
}
.flag.cu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cu");
}
.flag.cv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cv");
}
.flag.cw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cw");
}
.flag.cx:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cx");
}
.flag.cy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cy");
}
.flag.cz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-cz");
}
.flag.de:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-de");
}
.flag.dj:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-dj");
}
.flag.dk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-dk");
}
.flag.dm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-dm");
}
.flag.do:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-do");
}
.flag.dz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-dz");
}
.flag.ec:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ec");
}
.flag.ee:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ee");
}
.flag.eg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-eg");
}
.flag.eh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-eh");
}
.flag.er:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-er");
}
.flag.es-ca:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-es-ca");
}
.flag.es:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-es");
}
.flag.et:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-et");
}
.flag.eu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-eu");
}
.flag.fi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fi");
}
.flag.fj:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fj");
}
.flag.fk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fk");
}
.flag.flags.sh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-flags.sh");
}
.flag.fm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fm");
}
.flag.fo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fo");
}
.flag.fr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-fr");
}
.flag.ga:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ga");
}
.flag.gb-eng:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb-eng");
}
.flag.gb-nir:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb-nir");
}
.flag.gb-sct:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb-sct");
}
.flag.gb-wls:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb-wls");
}
.flag.gb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gb");
}
.flag.gd:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gd");
}
.flag.ge:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ge");
}
.flag.gf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gf");
}
.flag.gg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gg");
}
.flag.gh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gh");
}
.flag.gi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gi");
}
.flag.gl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gl");
}
.flag.gm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gm");
}
.flag.gn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gn");
}
.flag.gp:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gp");
}
.flag.gq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gq");
}
.flag.gr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gr");
}
.flag.gs:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gs");
}
.flag.gt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gt");
}
.flag.gu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gu");
}
.flag.gw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gw");
}
.flag.gy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-gy");
}
.flag.hk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hk");
}
.flag.hm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hm");
}
.flag.hn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hn");
}
.flag.hr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hr");
}
.flag.ht:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ht");
}
.flag.hu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-hu");
}
.flag.id:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-id");
}
.flag.ie:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ie");
}
.flag.il:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-il");
}
.flag.im:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-im");
}
.flag.in:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-in");
}
.flag.io:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-io");
}
.flag.iq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-iq");
}
.flag.ir:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ir");
}
.flag.is:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-is");
}
.flag.it:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-it");
}
.flag.je:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-je");
}
.flag.jm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-jm");
}
.flag.jo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-jo");
}
.flag.jp:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-jp");
}
.flag.ke:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ke");
}
.flag.kg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kg");
}
.flag.kh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kh");
}
.flag.ki:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ki");
}
.flag.km:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-km");
}
.flag.kn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kn");
}
.flag.kp:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kp");
}
.flag.kr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kr");
}
.flag.kw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kw");
}
.flag.ky:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ky");
}
.flag.kz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-kz");
}
.flag.la:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-la");
}
.flag.lb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lb");
}
.flag.lc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lc");
}
.flag.li:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-li");
}
.flag.lk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lk");
}
.flag.lr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lr");
}
.flag.ls:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ls");
}
.flag.lt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lt");
}
.flag.lu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lu");
}
.flag.lv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-lv");
}
.flag.ly:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ly");
}
.flag.ma:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ma");
}
.flag.mc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mc");
}
.flag.md:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-md");
}
.flag.me:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-me");
}
.flag.mf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mf");
}
.flag.mg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mg");
}
.flag.mh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mh");
}
.flag.mk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mk");
}
.flag.ml:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ml");
}
.flag.mm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mm");
}
.flag.mn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mn");
}
.flag.mo:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mo");
}
.flag.mp:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mp");
}
.flag.mq:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mq");
}
.flag.mr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mr");
}
.flag.ms:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ms");
}
.flag.mt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mt");
}
.flag.mu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mu");
}
.flag.mv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mv");
}
.flag.mw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mw");
}
.flag.mx:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mx");
}
.flag.my:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-my");
}
.flag.mz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-mz");
}
.flag.na:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-na");
}
.flag.nc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nc");
}
.flag.ne:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ne");
}
.flag.nf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nf");
}
.flag.ng:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ng");
}
.flag.ni:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ni");
}
.flag.nl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nl");
}
.flag.no:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-no");
}
.flag.np:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-np");
}
.flag.nr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nr");
}
.flag.nu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nu");
}
.flag.nz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-nz");
}
.flag.om:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-om");
}
.flag.pa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pa");
}
.flag.pe:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pe");
}
.flag.pf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pf");
}
.flag.pg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pg");
}
.flag.ph:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ph");
}
.flag.pk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pk");
}
.flag.pl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pl");
}
.flag.pm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pm");
}
.flag.pn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pn");
}
.flag.pr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pr");
}
.flag.ps:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ps");
}
.flag.pt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pt");
}
.flag.pw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-pw");
}
.flag.py:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-py");
}
.flag.qa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-qa");
}
.flag.re:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-re");
}
.flag.ro:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ro");
}
.flag.rs:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-rs");
}
.flag.ru:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ru");
}
.flag.rw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-rw");
}
.flag.sa:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sa");
}
.flag.sb:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sb");
}
.flag.sc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sc");
}
.flag.sd:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sd");
}
.flag.se:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-se");
}
.flag.sg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sg");
}
.flag.sh:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sh");
}
.flag.si:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-si");
}
.flag.sj:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sj");
}
.flag.sk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sk");
}
.flag.sl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sl");
}
.flag.sm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sm");
}
.flag.sn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sn");
}
.flag.so:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-so");
}
.flag.sr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sr");
}
.flag.ss:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ss");
}
.flag.st:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-st");
}
.flag.sv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sv");
}
.flag.sx:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sx");
}
.flag.sy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sy");
}
.flag.sz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-sz");
}
.flag.tc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tc");
}
.flag.td:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-td");
}
.flag.tf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tf");
}
.flag.tg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tg");
}
.flag.th:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-th");
}
.flag.tj:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tj");
}
.flag.tk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tk");
}
.flag.tl:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tl");
}
.flag.tm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tm");
}
.flag.tn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tn");
}
.flag.to:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-to");
}
.flag.tr:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tr");
}
.flag.tt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tt");
}
.flag.tv:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tv");
}
.flag.tw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tw");
}
.flag.tz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-tz");
}
.flag.ua:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ua");
}
.flag.ug:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ug");
}
.flag.um:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-um");
}
.flag.un:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-un");
}
.flag.us:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-us");
}
.flag.uy:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-uy");
}
.flag.uz:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-uz");
}
.flag.va:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-va");
}
.flag.vc:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vc");
}
.flag.ve:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ve");
}
.flag.vg:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vg");
}
.flag.vi:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vi");
}
.flag.vn:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vn");
}
.flag.vu:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-vu");
}
.flag.wf:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-wf");
}
.flag.ws:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ws");
}
.flag.xk:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-xk");
}
.flag.ye:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-ye");
}
.flag.yt:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-yt");
}
.flag.za:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-za");
}
.flag.zm:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-zm");
}
.flag.zw:before {
	 background-image: url("/system/shared/collection/flags/flags.svg#flag-zw");
}
