if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityKeyManagerDataModel")) {
	Ext.define('Portfolio.designer.model.domainModelEntityKeyManagerDataModel', {
		extend: 'Ext.data.Model',
		fields: [
			{ type: 'string', name: 'name' },
			{ type: 'boolean', name: 'primary' },
			{ type: 'auto', name: 'columns' },
			{ type: 'boolean', name: 'drop' },
			{ type: 'string', name: 'was' }
		]
	});
}
