Ext.define("Ext.ux.form.field.ISOTimestampPreSaveField", {
	extend: "Ext.ux.form.field.ISOTimestampField",

	alias: "widget.isotimestamppresavefield",

	initComponent: function () {
		var i_self = this;
		i_self.callParent();
	},

	presave: function() {
		var i_self = this;
		var value = ip2.createIsoTimeStamp();
		i_self.setValue(value);
	}
});
