Ext.define("Ext.ux.navigation.viewmodel.RolePermissionPanel", {
	extend: "Ext.app.ViewModel",
	alias: "viewmodel.rolepermissionpanel",

	data: {
		permissionGrid: null,
		dbGrid: null,
		type: null,
		filter: null,
		record: null,
		key: null
	}
});
