if (!Ext.ClassManager.isCreated("Ext.ux.data.model.EditCode")) {
	Ext.define('Ext.ux.data.model.EditCode', {

		extend: 'Ext.data.Model',

		alias: ['model.editcode'],

		fields: [
			{ name: 'key', type: 'string' },
			{ name: 'comma', type: 'string' },
			{ name: 'decimal', type: 'string' },
			{ name: 'sign', type: 'string' },
			{ name: 'sign', type: 'string' },
			{ name: 'blank', type: 'string' },
			{ name: 'iValue', type: 'string' },
			{ name: 'jValue', type: 'string' },
			{ name: 'zero', type: 'string' }
		]
	});
}

