Ext.define('Ext.ux.stackorganizer.column.MoveUp', {

	extend: 'Ext.ux.themed.ActionColumn',

	alias: ['widget.stackmoveupcolumn'],

	initComponent: function () {

		var i_cmp = this;

		i_cmp.callParent(arguments);

		return i_cmp;
	}
});
