if (!Ext.ClassManager.isCreated("Portfolio.registry.tree.model.Registry")) {

	Ext.define("Portfolio.registry.tree.model.Registry", {

		extend: "Ext.data.Model",

		alternateClassName: [
			"Ext.registry.tree.model.Registry"
		],

		fields: [
			{
				name: "name",
				type: "string"
			},
			{
				name: "key",
				type: "string"
			},
			{
				name: "value",
				type: "auto"
			},
			{
				name: "type",
				type: "string"
			},
			{
				name: "label",
				type: "string"
			},
			{
				name: "leaf",
				type: "boolean"
			},
			{
				name: "iconCls",
				type: "string"
			},
			{
				name: "meta",
				type: "auto"
			},
			{
				name: "plugin",
				type: "auto"
			},
			{
				name: "children",
				type: "auto"
			}
		]
	});
}

