{
	xtype: 'panel'
	,border: false
	,bodyPadding: 10
	,layout: "anchor"
	,defaults:{
		labelAlign:'top'
	},
	onInitialize: function(i_compoent) {
		var i_self = this;
		i_self.add(i_self.createFields());
		i_self.enableBubble("dirtychange");
	},
	createFields: function() {

		var i_self = this;

		var a_fields = [{
				xtype: "textfield",
				anchor: '100%',
				fieldLabel: ip2.i18n({
					da: "Brugernavn",
					en: "Username"
				}),
				name:'USRNAM',
				allowBlank:false
			},
			{
				xtype: "textfield",
				vtype: "email",
				anchor: '100%',
				fieldLabel: ip2.i18n({
					da: "Email",
					en: "Email"
				}),
				name:'USREML'
			},
			{
				fieldLabel: ip2.i18n({
					da: "Sprog",
					en: "Language"
				}),
				name:'USRLNG',
				xtype:'ip2NavigatorLanguageAutoComboBox',
				anchor: '100%'
			},
			{
				fieldLabel: ip2.i18n({
					da: "Profil billede",
					en: "Profile image"
				}),
				xtype: "uploadpicker",
				name: "USRPRO",
				path: "/images/profile",
				anchor: '100%',
				extensions: Ext.ux.file.util.Handler.imageExtension,
				pageSize: 10,
				listeners: {
					afterrender: {
						fn: function(item) {
							var o_record = {
								path : "/system/images/icons/extjs5/32x32/user.png",
								name: "user.png"
							}
							var i_store = item.getStore();
							var i_record = i_store.model.create(o_record);
							i_store.loadRecords([i_record], {addRecords: true});
							i_store.insert(0, i_record);
							i_store.on({
								load: function(store, a_records) {
									i_store.loadRecords([i_record], {addRecords: true});
									i_store.insert(0, i_record);
								}
							});
						}
					}
				}
			}
		];
		return a_fields;
	}
}
