if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityFormDesignerPropertyGridModel")) {

	Ext.define("Portfolio.designer.model.domainModelEntityFormDesignerPropertyGridModel", {
		extend: "Ext.data.Model",
		fields: [
			{
				name: "name",
				type: "auto"
			},
			{
				name: "value",
				type: "auto"
			},
			{
				name: "category",
				type: "auto"
			}
		],
		idProperty: "name"
	});
}
