if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelCardinalityModel")) {
	Ext.define('Portfolio.designer.model.domainModelCardinalityModel', {
		extend: 'Ext.data.Model',
		fields: [
			{ type: 'auto', name: 'key' },
			{ type: 'auto', name: 'value' }
		]
	});
}
