if (!Ext.ClassManager.isCreated("Portfolio.dynapp.model.Relation")) {

	Ext.define("Portfolio.dynapp.model.Relation", {

		extend: "Ext.data.Model",

		alternateClassName: [
			"Ext.dynapp.model.Relation"
		],

		fields: [
			{ type: "string", name: "table" },
			{ type: "string", name: "schema" },
			{ type: "string", name: "join" }
		]
	});
}

