if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityRelationManagerDataModel")) {
	Ext.define("Portfolio.designer.model.domainModelEntityRelationManagerDataModel", {
		extend: "Ext.data.Model",
		fields: [
			{ type: "string", name: "betweenEntity", mapping: "between.entity" },
			{ type: "string", name: "andEntity", mapping: "and.entity" },
			{ type: "auto", name: "betweenOn", mapping: "between.on" },
			{ type: "string", name: "betweenOccur", mapping: "between.occurrence" },
			{ type: "auto", name: "andOn", mapping: "and.on" },
			{ type: "string", name: "andOccur", mapping: "and.occurrence" },
			{ type: "string", name: "cardinality", mapping: "between.occurrence",
				convert: function(value, i_record) {

					var s_betweenOccur = i_record.get("betweenOccur");
					var s_and = i_record.get("andOccur");

					var a_result = [s_betweenOccur, s_and];
					var s_result = "";

					if (s_betweenOccur && s_and) {
						s_result = a_result.join("2");
					}

					return s_result;
				}
			},
			{ type: "auto", name: "columns", mapping: "columns" },
			{ type: "auto", name: "properties", mapping: "properties" },
			{ type: "boolean", name: "drop", mapping: "drop" },
			{ type: "string", name: "betweenEntityWas" },
			{ type: "string", name: "andEntityWas" }
		]
	});
}
