/* /home/teamcity/buildAgent/work/47e72153b7b97580/sencha/ext/modern/theme-neptune/sass/var/slider/Slider.scss:154 */
.x-toggleslider .x-slider {
	padding-top: 7px;
    background-image: none;
}

.x-toggleslider .x-slider-end {
    background-image: none;
}

.x-toggleslider .x-slider-inner {
    background-image: none;
}

.x-toggleslider .x-slider-inner.x-toggle-off {
	background-color: #e1e1e1;
}

.x-toggleslider .x-slider-inner.x-toggle-on {
	background-color: #bbdefb;
}


.x-toggleslider .x-slider-thumb {
    z-index: 1;
}
