
Ext.define('Ext.ux.relationorganizer.Tree', {

	extend: 'Ext.tree.Panel',

	requires: [
		'Ext.data.Store'
	],

	initComponent: function () {

		var i_cmp = this;

		i_cmp.callParent(arguments);

		return i_cmp;
	}
});
