Ext.define('Ext.ux.mailChimp.View', {
	extend: 'Ext.panel.Panel',
	alias: ['widget.mailchimpview'],
	layout: 'border',
	initComponent: function() {
		var me = this;

		me.items = [{
			region: 'north',
			xtype: 'mailchimpsearchpanel'
		},{
			region: 'center',
			xtype: 'mailchimpgrid'
		}];

		me.callParent();
	}
});
