
@import url("https://fonts.googleapis.com/css?family=Open+Sans");
@import url("https://fonts.googleapis.com/css?family=Roboto");
@import url("/system/shared/collection/fonts.css");


@media print {
	html, body {
		zoom: 80%;
	}
	@page {
		size: A4 landscape;
	}

}


.disabled-row .x-grid-cell {
	background-color: #bcbcbc !important;
}​

.login {
	font:normal 13px arial;
}

.login a {
	text-decoration:none;
}

.login .box {
	background-image:url(/portfolio/Themes/Default/login-loginbox.png);
	width: 527px;
	height:327px;
	position: absolute;
	margin: auto;
	left: 0px;
	top: 0px;
	display:none;
}

.login .title {
	color:#ffffff;
	font-size:22px;
}
.login .title .companyname {
	position:absolute;
	top:60px;
	right:132px;
	float:right;
}
.login .title .action {
	position:absolute;
	top:60px;
	left:421px;
	float:left;
}

.login .form {
	position:absolute;
	top:260px;
	left:28px;
}

.login .inputfield, .button {
	display:inline-block;
	float:left;
	padding-left:7px;
}
.login .inputfield input {
	background: url(/portfolio/Themes/default/login-inputfield.png) no-repeat;
	width:183px;
	height:29px;
	color:#333333;
}
.login .inputfield input.error {
	background: url(/portfolio/Themes/default/login-inputfield.png) no-repeat 0 -29px;
}
.login .button {
	background: url(/portfolio/Themes/default/login-btn.png) top;
	width: 66px !important;
	height:29px;
	border:none;
	margin-left:8px;
	cursor:pointer;
}
.login .button:hover {
	background-position: 0 -29px;
}
.login .button .text {
	display:block;
	font-weight:bold;
	color:#333333;
	padding: 6px 0 0 8px;
	text-shadow: -1px -1px 0px rgba(255, 255, 255, 0.75);
	opacity:0.50;
}

.login .textfield {
	border: none;
	-khtml-border-radius: 5px;
	-moz-border-radius: 5px;
	-webkit-border-radius: 5px;
	border-radius: 5px;
	background:none;
	padding-left:5px;
	height: 27px !important;
}
.x-ie .login .textfield {
	padding-top:5px;
}

.login .message {
	position:relative;
	top:330px;
	left:13px;
	color:#D81818;
}
.login .message  a {
	color:#D81818;
}

.login .copyright {
	bottom:22px;
	position: absolute;
	width:100%;
	text-align:center;
}
.login .copyright .logo {
	background-repeat:no-repeat;
	height:30px;
	display: block;
	margin-left: auto;
	margin-right: auto;
	padding-bottom:5px;
}
.login .copyright .text {
	height:12px;
	color:#798593;
}

.header {
	font:normal 13px arial;
	width:100%;
	height:51px;
	border:0px;
}
.header a {
	text-decoration:none;
}
.header .logo {
	position:absolute;
	left:15px;
	top:11px;
	height:30px;
	width:30px;
	background:transparent url(/portfolio/Themes/Default/sml-logo-topbar.png) no-repeat;
}
.header .title {
	position:absolute;
	left:53px;
	top:18px;
	color:white;
	font-weight:bold;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.55;
}
.header .caption {
	position: absolute;
	top:17px;
	width:100%;
	text-align:center;
	color:white;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.90;
}
.header .user {
	position:absolute;
	right:98px;
	top:16px;
	color:white;
	font-weight:bold;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.55;
}
.header-user-image {
	background-position: center center;
	background-size: cover;
	cursor: pointer;
},

.header .menu {
	position:absolute;
	right:10px;
	top:11px;
	color:white;
	text-align:left;
}
.header .menu .button {
	background: url(/portfolio/Themes/default/topbar-btn.png) no-repeat;
	width:66px !important;
	height:29px !important;
	border:none;
	margin-left:8px;
	cursor:pointer;
}
.header .menu .button:hover {
	background: url(/portfolio/Themes/default/topbar-btn.png) no-repeat 0 -29px;
}
.header .menu .button .text {
	display:block;
	font-weight:bold;
	color:#ffffff;
	padding: 6px 0 0 1px;
	text-shadow: -1px -1px 0px rgba(150, 150, 150, 0.9);
	opacity:0.50;
}


#splash-content {
	width: 100%;
	height:100%;
}
.splash {
	width: 100%;
	height:100%;
	font:normal 12px arial;
	color:#333333;
	text-align:center;
}
.splash .box {
	background-color:#ffffff;
	width:500px;
	margin:auto;
	position:relative;
	top:20%;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	-webkit-box-shadow: 2px 2px 5px rgba(50, 50, 50, 0.75);
	-moz-box-shadow:    2px 2px 5px rgba(50, 50, 50, 0.75);
	box-shadow:         2px 2px 5px rgba(50, 50, 50, 0.75);
}
.splash .box .title {
	font-size:18px;
	padding-top:26px;
}
.splash .box .text {
	letter-spacing:1px;
	line-height:22px;
	margin:20px 33px 12px 33px;
}
.splash .box .logo {
	/*background-image:url(/portfolio/Themes/Default/login-logo.png);*/
	background-repeat:no-repeat;
	/*height:31px;
	width:23px;*/
	display: block;
	margin-left: auto;
	margin-right: auto;
	padding-bottom:52px;
}


/************************************
	Ext.ux.plugins.UploadDD
************************************/
.x-ux-plugin-uploadDD  div[_type="dropbox"] {
	height: 100%;
	border: 2px dashed #CCCCCC;
	border-radius: 3px;
	font-size: 14px;
	-webkit-transition:border-color .1s ease-out;
}
.x-ux-plugin-uploadDD.x-ux-plugin-uploadDD-over div[_type="dropbox"] {
	border-color: #00c294;
	-webkit-transition: border-color .1s ease-in;
}

.x-ux-plugin-uploadDD.x-ux-plugin-uploadDD-error div[_type="dropbox"] {
	border-color: #D81818;
}


.x-ux-imagelist tr td {
	vertical-align: middle;
}


/********************************************************
	No component, can be used by all that uses a triggers
*********************************************************/
.x-ux-search-icon {
	background-image: url("/system/shared/collection/icons/16x16/plain/magnifying_glass.png") !important;
}

.x-ux-icon-bg {
	background-repeat: no-repeat !important;
	padding-left: 25px !important;
	background-position: 5px 60% !important;
}

/************************************
	Ext.ux.hsvwheel.Field
************************************/
.x-ux-hsvwheel-field input {
	padding-left: 22px;
}

.x-ux-hsvwheel-field-preview {
	position: relative;
	height: 11px;
	width: 11px;
	top: 50%;
	left: 6px;
	border: 1px solid #000000;
	background-color: transparent;
}
.x-ux-hsvwheel-field-preview.error {
	border-color: #D81818;
	background-color: #FFFFFF !important;
}


/************************************
	Ext.ux.gallery.view.MediaGrid
************************************/
.x-ux-imagegrid tr td {
	vertical-align: middle;
}
.x-grid.x-ux-imagegrid .x-grid-body {
	border-left-width: 0;
	border-right-width: 0;
	border-bottom-width: 0;
}

.x-grid.x-ux-imagegrid .x-docked {
	border-left-width: 0;
	border-right-width: 0;
}


/************************************
	Ext.ux.ip2.folderorganizer
************************************/
table.x-ux-folderorganizer-noresult {
	height: 100%;
	width: 100%;
	text-align: center;
}
table.x-ux-folderorganizer-noresult tr {
	height: 100%;
}
table.x-ux-folderorganizer-noresult tr td {
	vertical-align: middle;
	height: 100%;
}

/************************************
	Filter images
************************************/
.x-ux-filter-check{
	background-image:  url("/system/shared/collection/icons/16x16/plain/check.png") !important;
	background-repeat: no-repeat;
}


.x-ux-big-font {
	font-size: 16px;
	color: black;
	height: 50px;
}

.x-ux-stock td:not(:first-child) {
	color: #199ACA !important;
	background-color: #EDF7FB !important;
}

.x-ux-repair td:not(:first-child) {
	color: #D67302 !important;
	background-color: #FCF4EB !important;
}

.x-ux-check td:not(:first-child) {
	color: #E805FF !important;
	background-color: #FEEBFF !important;
}

.x-ux-invalid td:not(:first-child) {
	color: #D03E35 !important;
	background-color: #FCF0EF !important;
}

.x-ux-tree-container td:not(:first-child) {
	background-color: #cbd0d5 !important;
	border-top: 1px solid #aeb6be;
}

.x-ux-tree-container-last td:not(:first-child) {
	background-color: #cbd0d5 !important;
	border-bottom: 1px solid #aeb6be;
}

.x-ux-tree-container-child td:not(:first-child) {
	background-color: #e0e3e6 !important;
}

.x-ux-tree-container-child-first td:not(:first-child) {
	background-color: #e0e3e6 !important;
	border-top: 1px solid #aeb6be;
}

.x-ux-tree-container-child-last td:not(:first-child) {
	background-color: #e0e3e6 !important;
	border-bottom: 1px solid #aeb6be;
}
.x-ux-icon-selected-item {
	outline: 1px dashed black !important;
	background-color: #e0e3e6 !important;
}

.x-grid-row-summary .x-grid-cell-inner {
	font-weight: bold;
}

.x-fieldset {
	border-top: none !important;
}

.centertabs-panel {
	padding-top: 5px;
}

.bold-item {
	font-weight: bold !important;
}

.x-dot {
	display: inline-block;
	background-image: url("/portfolio/styles/icon/entity/dot.svg");
	background-size: 0.30em;
}

.x-tab-icon-right > .x-tab-inner-default {
    max-width: calc(100% - 10px);
}

.x-tab-text.x-tab-icon-right > .x-tab-icon-el-default {
    width: 10px;
    margin-left: 3px;
}


