Ext.define('Ext.ux.data.store.SnapOnIcon', {

	extend: 'Ext.data.Store',

	model: 'Ext.ux.data.model.SnapOnIcon',

	sortRoot: 'score',

	listeners: {
		'beforeload': {
			fn: function(store, operation) {

				if (!operation.params.keyword.length) {
					operation.params.keyword = ' ';
				}
			}
		}
	}
});
