Ext.define('Ext.ux.data.dmd.Types', {

	singleton: true
},
function () {

	var o_type = Ext.data.Types;
	var o_sort = Ext.data.SortTypes;

	Ext.apply(o_type, {
		CHAR: {
			convert: o_type.STRING.convert,
			sortType: o_type.STRING.sortType,
			type: 'char'
		},
		INTEGER: {
			convert: o_type.INTEGER.convert,
			sortType: o_type.INTEGER.sortType,
			type: 'integer'
		},
		NUM6DATE: {
			convert: function (value) {
				return value;
			},
			sortType: o_sort.asDate,
			type: 'num6date'
		},
		NUM8DATE: {
			convert: function (value) {
				return value;
			},
			sortType: o_sort.asDate,
			type: 'num8date'
		},
		VARCHAR: {
			convert: o_type.STRING.convert,
			sortType: o_type.STRING.sortType,
			type: 'varchar'
		}
	});
});
