Ext.define("Ext.ux.form.field.Combi", {

	extend: "Ext.form.FieldContainer",

	isFormField: false,
	alias: "widget.fieldcombi",
	fields: [],
	layout: {
		type: "hbox",
		align: "stretch"

	},
	defaults: {
		margin: "0 5 0 0"
	},

	initComponent: function () {

		var i_self = this;
		i_self.callParent();
	}
});
