if (!Ext.ClassManager.isCreated("Portfolio.designer.ui.constraints.model.Manager")) {
	Ext.define('Portfolio.designer.ui.constraints.model.Manager', {
		extend: 'Ext.data.Model',
		fields: [
			{ type: 'auto', name: 'name' },
			{ type: 'auto', name: 'properties' }
		]
	});
}
