if (!Ext.ClassManager.isCreated("Portfolio.pd.manager.component.grid.model.Collection")) {

	Ext.define("Portfolio.pd.manager.component.grid.model.Collection", {
		extend: "Ext.data.Model",
        fields: [
            {
                name: "type",
                type: "string",
                defaultValue: "collection"
            },
            {
                name: "extend",
                type: "auto",
                defaultValue: null
            },
            {
                name: "name",
                type: "auto",
                defaultValue: ""
            },
            {
                name: "label",
                type: "auto",
                defaultValue: ""
            },
            {
                name: "items",
                type: "auto",
                defaultValue: null
            },
            {
                name: "apperence",
                type: "auto",
                defaultValue: ""
            },
            {
                name: "rule",
                type: "auto",
                defaultValue: null
            },
            {
                name: "uom",
                type: "auto",
                defaultValue: null
            },
            {
                name: "active",
                type: "boolean",
                defaultValue: true
            }
        ],
        idProperty: "name"
    });
}