.pd-form-field-deactive {
    opacity: 0.5;
}

.pd-grid-row-deactive {
    opacity: 0.5;
}

.pd-value-picker-delete {
    font-size: 16px;
    padding-right: 26px;
}

.pd-value-picker-row-deactive {
    opacity: 0.5;
}