Ext.define("Ext.ux.form.field.i18n.VarcharArea", {

	extend: "Ext.form.field.TextArea",

	alias: ["widget.i18nvarchararea"],

	triggers: {
		"switch": {
			cls: "flatui i18n-field-disable",
			tooltip: ip2.i18n({
				da: "Klik her for at ændre til sprogstyret tekstboks",
				en: "Click here to change to a language controlled textarea"
			}),
			handler: function() {

				i_self = this;
				i_self.fireEvent("i18ntrigger", i_self, true);
			},
			scope: "this"
		}
	},

	initComponent: function () {

		var i_self = this;
		var o_config = i_self.initialConfig;

		i_self.callParent(arguments);

		return i_self;
	},

	setValue: function(value) {
		var i_self = this;
		i_self.callParent(arguments);

		if (i_self.originalValue == "") {
			i_self.resetOriginalValue();
		}
	}
});
