if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityFormDesignerTreeDataModel")) {
	Ext.define("Portfolio.designer.model.domainModelEntityFormDesignerTreeDataModel", {
		extend: "Ext.data.Model",
		fields: [
			{
				name: "properties",
				type: "auto"
			},
			{
				name: "name",
				mapping: "properties.name",
				type: "auto"
			},
			{
				name: "title",
				mapping: "properties.fieldLabel",
				type: "auto"
			}
		],
		proxy: {
			type: "memory",
			reader: {
				type: "json"
			}
		}
	});
}
