
Ext.define('Ext.ux.gallery.view.LinkStackGrid', {
	extend: 'Ext.panel.Panel',
	alias: ['widget.mediagallerylinkstackgrid'],
	initComponent: function () {

		var i_cmp = this;

		i_cmp.callParent();

		return i_cmp;
	}
});
