Ext.define('Ext.ux.form.field.QuickCombo', {
	extend: 'Ext.form.field.ComboBox',
	alias: 'widget.quickcombo',

	/**
	     * @cfg {Array} data
	     * The data to put in the quickcombo
	     */
	data: null,

	/**
	     * @cfg {Array} fields
	     * The fields the store uses for data
	     */
	fields: null,

	initComponent: function () {

		var i_cmp = this;
		var i_store = i_cmp.createStore();

		Ext.apply(i_cmp, {
			store: i_store
		});

		//i_cmp.valueField = i_cmp.intitialConfig.valueField || i_cmp.fields[0];
		//i_cmp.displayField = i_cmp.intitialConfig.displayField || i_cmp.fields[1];

		i_cmp.callParent(arguments);

		return i_cmp;
	},

	createStore: function () {

		var i_cmp = this,
		a_field = i_cmp.fields || [],
		a_data = i_cmp.data || [],
		i_store;

		i_store = Ext.create('Ext.data.Store', {
			fields: a_field,
			data: a_data
		});

		return i_store;
	}
});
