Ext.define("Portfolio.dynapp.viewmodel.ColumnProperties", {

	extend: "Ext.app.ViewModel",

	alias: "viewmodel.dynapp.columnproperties",

	alternateClassName: [
		"Ext.dynapp.viewmodel.ColumnPropertie"
	],

	data: {
		grid: null
	}
});
