if (!Ext.ClassManager.isCreated("Ext.ux.data.model.Locale")) {
	Ext.define("Ext.ux.data.model.Locale", {
		extend: "Ext.data.Model",
		fields: [
			{
				name: "code",
				type: "string"
			},
			{
				name: "language",
				type: "auto"
			},
			{
				name: "country",
				type: "string"
			},
			{
				name: "flag",
				type: "string"
			}
		],
		idProperty: "code"
	});
}
