if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelLanguageManagerDataModel")) {
	Ext.define('Portfolio.designer.model.domainModelLanguageManagerDataModel', {
		extend: 'Ext.data.Model',
		fields: [
			{ type: 'string', name: 'code' }
		]
	});
}
