if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityHierarchyTree")) {
	Ext.define("Portfolio.designer.model.domainModelEntityHierarchyTree", {
		extend: "Ext.data.Model",
		fields: [
			{
				type: "auto",
				name: "text"
			},
			{
				type: "string",
				name: "baseTable"
			},
			{
				type: "boolean",
				name: "allowSync"
			},
			{
				type: "boolean",
				name: "allowUpdate"
			},
			{
				type: "string",
				name: "where"
			}
		]
	});
}
