if (!Ext.ClassManager.isCreated('Ext.ux.file.model.TreeHive')) {
	Ext.define("Ext.ux.file.model.TreeHive", {
		extend: "Ext.data.Model",
		fields: [
			{ name: 'cls', type: "string" },
			{ name: 'ext', type: "string" },
			{ name: 'text', type: "string" },
			{ name: 'path', type: "string" },
			{ name: 'hasChild', type: "boolean" },
			{ name: 'leaf', type: "boolean" },
			{ name: 'isHive', type: "boolean" },
			{ name: 'type', type: "string" },
			{ name: "iconCls", type: "string" },
			{ name: "allowImport", type: "boolean" },
			{ name: "allowAdd", type: "boolean" }
		]
	});
}

