Ext.application({

	extend: "Portfolio.Application",

	name: "Portfolio",

	appFolder: "/portfolio/app",

	uses: [
		"Portfolio.login.view.Html"
	],

	mixins: [
		"Ext.mixin.Mashup"
	],

	requiredScripts: [
	]
});
