Ext.define("Ext.ux.buffer.Proxy", {
	extend: "Ext.data.proxy.Ajax",
	alias: "proxy.ux.buffer",
	paramsAsJson: true,
	type: "ajax",
	reader: {
		type: "json",
		headers: {
			"Accept": "application/json"
		},
		rootProperty: "rows",
		totalProperty: "totalRows"
	},
	actionMethods: {
		read: "POST"
	}
});
