.markdown-preview {
    background-color: white;
    overflow-x: hidden;
    overflow-y: auto;
}

.markdown { padding: 20px; }

.markdown * {
    font-size: 14px;
    color: black;
    font-family: helvetica;
}
.markdown h1 {
    margin: -5px 0 0;
    font-size: 26px;
    color: #779ed3;
    line-height: 33px;
}
.markdown h2 {
    font-size: 21px;
}
.markdown h3 {
    color: #779ed3;
    font-weight: bold;
    font-size: 14px;
}

.markdown h2, .markdown h2, .markdown h3, .markdown h4, .markdown h5, .markdown h6, .markdown p, .markdown ul, .markdown ol {
    margin: 10px 0 0;
}

.markdown ul, .markdown ol { margin-left: 25px; }

.markdown ul, .markdown ul li {
    list-style: disc outside none;
}
.markdown ol, .markdown ol li {
    list-style: decimal outside none;
}
.markdown li {
    line-height: 18px;
}

.markdown dl {
    margin-bottom: 18px;
}
.markdown dt, .markdown dd {
    line-height: 18px;
}
.markdown dt {
    font-weight: bold;
}
.markdown dd {
    margin-left: 9px;
}
.markdown hr {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    border-color: #EEEEEE -moz-use-text-color #FFFFFF;
    border-left: 0 none;
    border-right: 0 none;
    border-style: solid none;
    border-width: 1px 0;
    margin: 18px 0;
}
.markdown strong {
    font-weight: bold;
}
.markdown em {
    font-style: italic;
}

.markdown abbr {
    border-bottom: 1px dotted #DDDDDD;
    cursor: help;
    font-size: 90%;
    text-transform: uppercase;
}
.markdown blockquote {
    border-left: 5px solid #EEEEEE;
    margin: 0 0 18px;
    padding: 0 0 0 15px;
}
.markdown blockquote p {
    font-size: 16px;
    font-weight: 300;
    line-height: 22.5px;
    margin-bottom: 0;
}
.markdown blockquote small {
    color: #999999;
    display: block;
    line-height: 18px;
}
.markdown blockquote small:before {
    content: "— ";
}

.markdown q:before, .markdown q:after, .markdown blockquote:before, .markdown blockquote:after {
    content: "";
}
.markdown address {
    display: block;
    font-style: normal;
    line-height: 18px;
    margin-bottom: 18px;
}
.markdown small {
    font-size: 100%;
}
.markdown cite {
    font-style: normal;
}
.markdown code, .markdown pre {
    border-radius: 3px 3px 3px 3px;
    color: #333333;
    font-family: Menlo,Monaco,"Courier New",monospace;
    font-size: 12px;
    padding: 0 3px 2px;
}
.markdown code {
    background-color: #F7F7F9;
    border: 1px solid #E1E1E8;
    color: #DD1144;
    padding: 3px 4px;
}
.markdown pre {
    background-color: #F5F5F5;
    border: 1px solid rgba(0, 0, 0, 0.15);
    border-radius: 4px 4px 4px 4px;
    display: block;
    font-size: 12px;
    line-height: 18px;
    margin: 0 0 9px;
    padding: 8.5px;
    white-space: pre-wrap;
    word-wrap: break-word;
}
.markdown pre code {
    background-color: transparent;
    border: 0 none;
    color: inherit;
    padding: 0;
}

.markdown form {
    margin: 0 0 18px;
}
.markdown fieldset {
    border: 0 none;
    margin: 0;
    padding: 0;
}
.markdown legend {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    border-color: -moz-use-text-color -moz-use-text-color #EEEEEE;
    border-style: none none solid;
    border-width: 0 0 1px;
    color: #333333;
    display: block;
    font-size: 19.5px;
    line-height: 36px;
    margin-bottom: 27px;
    padding: 0;
    width: 100%;
}
.markdown legend small {
    color: #999999;
    font-size: 13.5px;
}
