.flatui .icon:hover {
	cursor: hand;
}


/* COMMON */
.flatui .icon-add						{ background-image: url(/system/shared/collection/icons/16x16/plain/navigate_plus.png?_dc=1) !important; }

.flatui .icon-arrow-up					{ background-image: url(/system/shared/collection/icons/16x16/plain/arrow_up.png?_dc=1) !important; }

.flatui .icon-arrow-down				{ background-image: url(/system/shared/collection/icons/16x16/plain/arrow_down.png?_dc=1) !important; }

.flatui .icon-arrow-left				{ background-image: url(/system/shared/collection/icons/16x16/plain/arrow_left.png?_dc=1) !important; }

.flatui .icon-arrow-right				{ background-image: url(/system/shared/collection/icons/16x16/plain/arrow_right.png?_dc=1) !important; }

.flatui .icon-box-open					{ background-image: url(/system/shared/collection/icons/16x16/plain/box-open.png?_dc=1) !important; }
.flatui .icon-box-closed				{ background-image: url(/system/shared/collection/icons/16x16/plain/box-closed.png?_dc=1) !important; }

.flatui .icon-close						{ background-image: url(/system/shared/collection/icons/16x16/plain/navigate_cross.png?_dc=1) !important; }

.flatui .icon-copy						{ background-image: url(/system/shared/collection/icons/16x16/plain/copy.png?_dc=1) !important; }
.flatui .icon-data-error 				{ background-image: url(/system/shared/collection/icons/16x16/plain/sign_warning.png?_dc=1) !important; }
.flatui .icon-data-new 					{ background-image: url(/system/shared/collection/icons/16x16/plain/pencil.png?_dc=1) !important; }
.flatui .icon-data-ok 					{ background-image: url(/system/shared/collection/icons/16x16/plain/check.png?_dc=1) !important; }

.flatui .icon-delete					{ background-image: url(/system/shared/collection/icons/16x16/plain/navigate_minus.png?_dc=1) !important; }

.flatui .icon-edit						{ background-image: url(/system/shared/collection/icons/16x16/plain/edit.png?_dc=1) !important; }
.flatui .icon-expand					{ background-image: url(/system/shared/collection/icons/16x16/plain/expand.png?_dc=1) !important; }

.flatui .icon-export					{ background-image: url(/system/shared/collection/icons/16x16/plain/cloud_download.png?_dc=1) !important; }

.flatui .icon-filmstrip					{ background-image: url(/system/shared/collection/icons/16x16/plain/filmstrip.png?_dc=1) !important; }
.flatui .icon-find						{ background-image: url(/system/shared/collection/icons/16x16/plain/find.png?_dc=1) !important; }
.flatui .icon-first						{ background-image: url(/system/shared/collection/icons/16x16/plain/first.png?_dc=1) !important; }
.flatui .icon-floating_window			{ background-image: url(/system/shared/collection/icons/16x16/plain/window.png?_dc=1) !important; }
.flatui .icon-forward					{ background-image: url(/system/shared/collection/icons/16x16/plain/navigate_right2.png?_dc=1) !important; }
.flatui .icon-gear						{ background-image: url(/system/shared/collection/icons/16x16/plain/gearwheel.png?_dc=1) !important; }
.flatui .icon-home						{ background-image: url(/system/shared/collection/icons/16x16/plain/home.png?_dc=1) !important; }
.flatui .icon-image						{ background-image: url(/system/shared/collection/icons/16x16/plain/image.png?_dc=1) !important; }
.flatui .icon-mail						{ background-image: url(/system/shared/collection/icons/16x16/plain/mail.png?_dc=1) !important; }
.flatui .icon-last						{ background-image: url(/system/shared/collection/icons/16x16/plain/last.png?_dc=1) !important; }
.flatui .icon-link						{ background-image: url(/system/shared/collection/icons/16x16/plain/link.png?_dc=1) !important; }
.flatui .icon-next						{ background-image: url(/system/shared/collection/icons/16x16/plain/next.png?_dc=1) !important; }
.flatui .icon-orientation_lanscape		{ background-image: url(/system/shared/collection/icons/16x16/plain/rotate_right.png?_dc=1) !important; }
.flatui .icon-orientation_protrait		{ background-image: url(/system/shared/collection/icons/16x16/plain/rotate_left.png?_dc=1) !important; }
.flatui .icon-page-menu					{ background-image: url(/system/shared/collection/icons/16x16/plain/page-menu.png?_dc=1) !important; }
.flatui .icon-preview					{ background-image: url(/system/shared/collection/icons/16x16/plain/eye.png?_dc=1) !important; }
.flatui .icon-preview_off				{ background-image: url(/system/shared/collection/icons/16x16/plain/eye.png?_dc=1) !important; }
.flatui .icon-previous					{ background-image: url(/system/shared/collection/icons/16x16/plain/previous.png?_dc=1) !important; }
.flatui .icon-print						{ background-image: url(/system/shared/collection/icons/16x16/plain/printer_network.png?_dc=1) !important; }

.flatui .icon-refresh					{ background-image: url(/system/shared/collection/icons/16x16/plain/arrow_circle.png?_dc=1) !important; }

.flatui .icon-resend					{ background-image: url(/system/shared/collection/icons/16x16/plain/mail_forward.png?_dc=1) !important; }

.flatui .icon-reset						{ background-image: url(/system/shared/collection/icons/16x16/plain/recycle.png?_dc=1) !important; }

.flatui .icon-save						{ background-image: url(/system/shared/collection/icons/16x16/plain/floppy_disk.png?_dc=1) !important; }

.flatui .icon-save-close				{ background-image: url(/system/shared/collection/icons/16x16/plain/save_as.png?_dc=1) !important; }

.flatui .icon-site-menu					{ background-image: url(/system/shared/collection/icons/16x16/plain/site-menu.png?_dc=1) !important; }
.flatui .icon-translate					{ background-image: url(/system/shared/collection/icons/16x16/plain/translate.png?_dc=1) !important; }
.flatui .icon-lock						{ background-image: url(/system/shared/collection/icons/16x16/plain/lock.png?_dc=1) !important; }

.flatui .icon-user						{ background-image: url(/system/shared/collection/icons/16x16/plain/user.png?_dc=1) !important; }
.flatui .icon-users						{ background-image: url(/system/shared/collection/icons/16x16/plain/users.png?_dc=1) !important; }
.flatui .icon-user_web					{ background-image: url(/system/shared/collection/icons/16x16/plain/user_web.png?_dc=1) !important; }
.flatui .icon-window					{ background-image: url(/system/shared/collection/icons/16x16/plain/window.png?_dc=1) !important; }
.flatui .icon-windows					{ background-image: url(/system/shared/collection/icons/16x16/plain/windows.png?_dc=1) !important; }

/* PACKING */
.flatui .icon-tick-pack					{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/tick.png?_dc=1) !important; }
.flatui .icon-delete-pack				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/delete.png?_dc=1) !important; }
.flatui .icon-radio-pack				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/radiobtn-pack.png?_dc=1) !important; }
.flatui .icon-reset-pack				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/reset.png?_dc=1) !important; }
.flatui .icon-unpack-pack				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/unpack.png?_dc=1) !important; }
.flatui .icon-add-pack					{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/add.png?_dc=1) !important; }
.flatui .icon-pack-pack					{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/pack.png?_dc=1) !important; }
.flatui .icon-pack-open-cont			{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/open-cont.png?_dc=1) !important; }
.flatui .icon-pack-close-cont			{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/close-cont.png?_dc=1) !important; }
.flatui .icon-pack-warning				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/packingwarning.png?_dc=1) !important; }
.flatui .icon-pack-stock				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/stock.png?_dc=1) !important; }
.flatui .icon-pack-repair				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/repair.png?_dc=1) !important; }
.flatui .icon-pack-check				{ background-image: url(/system/shared/collection/icons/16x16/plain/pack/check.png?_dc=1) !important; }

/* SCANNER */
.flatui .icon-scan-refresh				{ background-image: url(/system/shared/collection/icons/16x16/plain/central/scan-refresh.png?_dc=1) !important; }
.flatui .icon-scan-start				{ background-image: url(/system/shared/collection/icons/16x16/plain/central/scan-start.png?_dc=1) !important; }
.flatui .icon-scan-stop					{ background-image: url(/system/shared/collection/icons/16x16/plain/central/scan-stop.png?_dc=1) !important; }
.flatui .icon-scan-started				{ background-image: url(/system/shared/collection/icons/16x16/plain/central/scan-status-started.png?_dc=1) !important; }
.flatui .icon-scan-stopped				{ background-image: url(/system/shared/collection/icons/16x16/plain/central/scan-status-stopped.png?_dc=1) !important; }
.flatui .icon-ok						{ background-image: url(/system/shared/collection/icons/16x16/plain/ok.png?_dc=1) !important; }

/* I18n */

.flatui .i18n-field-active				{
	background-image: url(/portfolio/styles/icon/flatui/language-i18n.png?_dc=1) !important;
	background-position: -16px center !important;
	background-repeat: no-repeat !important;
	width: 18px;
	border-left: 1px solid #d0d0d0;
}

.flatui .i18n-field-active:before {
	content: ""
}

.flatui .i18n-field-disable				{
	background-image: url(/portfolio/styles/icon/flatui/language-i18n.png?_dc=1) !important;
	background-position: 0px center !important;
	background-repeat: no-repeat !important;
	width: 17px;
	border-left: 1px solid #d0d0d0;
}

.flatui .i18n-field-disable:before {
	content: ""
}



/************************************
	Ext.ux.stack
************************************/
.x-ux-stack .x-tree-panel .x-grid-row .x-grid-cell {
	border-color: #ededed;
	border-style: solid;
	border-width: 1px 0;
	border-top-color: #fafafa;
}

.x-ux-stack .no-icon {
	display: none;
	background-image: url('/portfolio/styles/icon/flatui/blank.png?_dc=1') !important;
}

.x-ux-stack .x-grid-cell-inner {
	position: relative;
	height: 110px !important;
}
.x-ux-stack .x-grid-cell-inner table {
	position: absolute;
	height: 100%;
	margin-top: 3px;
	left: 15px;
}

.x-ux-stack .x-grid-cell-inner table td {
	height: 100%;
}

.x-ux-stack .x-ux-stack-column-action > div,
.x-ux-stack .x-ux-stack-column-checkbox > div {
	text-align: center !important;
	padding-top: 48px !important;
}
.x-ux-stack .x-ux-stack-column-action img {
	cursor: pointer;
}

.x-ux-stack .x-ux-stack-icon {
	width: 80px;
	height: 80px;
}
.x-ux-stack .x-ux-stack-icon.textpage { background-image: url('/portfolio/styles/icon/flatui/stack/textpage.png?_dc=1'); }
.x-ux-stack .x-ux-stack-icon.mediagallery { background-image: url('/portfolio/styles/icon/flatui/stack/mediagallery.png?_dc=1'); }

.x-ux-stack-cell > div {
	height: 18px !important;
	overflow: hidden;
	margin-bottom: 3px;
}

.x-ux-stack-cell .x-ux-stack-label {
	float: left;
	width: 70px;
	margin-right: 5px;
}

.x-ux-stack-cell-toplink .x-ux-stack-value {
	color: #1172a9;
}

.x-ux-stack-table {
	width: 100%;
}

.x-ux-stack .x-ux-stack-icon {
	height: 45px;
	width: 45px;
	position: relative;
}

.x-ux-stack .x-ux-stack-icon div {
	margin: 0;
	padding: 0;
	position: absolute;
}

.x-ux-stack .x-ux-stack-icon .preview {
	height: 45px;
	width: 45px;
	top: 0;
	left: 0;
}

.x-ux-stack .x-ux-stack-icon .background {
	bottom: 0;
	right: 0;
	width: 10px;
	height: 10px;
	border: 1px solid black;
	display: none;
}

.x-ux-stack-icon .textpage { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage.png?_dc=1); }
.x-ux-stack-icon .textpage.design-wr { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage-design-wr.png?_dc=1); }
.x-ux-stack-icon .textpage.design-wl { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage-design-wl.png?_dc=1); }
.x-ux-stack-icon .textpage.design-rr { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage-design-rr.png?_dc=1); }
.x-ux-stack-icon .textpage.design-rl { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage-design-rl.png?_dc=1); }
.x-ux-stack-icon .textpage.design-rc { background: url(/portfolio/styles/icon/flatui/stackorganizer/textpage-design-rc.png?_dc=1); }

.x-ux-stack-icon .stack { background: url(/portfolio/styles/icon/flatui/stackorganizer/stack.png?_dc=1); }

.x-ux-stack-icon .teaserstack { background: url(/portfolio/styles/icon/flatui/stackorganizer/teaserstack.png?_dc=1); }
.x-ux-stack-icon .teaserstack.design-banner { background: url(/portfolio/styles/icon/flatui/stackorganizer/teaserstack-design-banner.png?_dc=1); }
.x-ux-stack-icon .teaserstack.design-shelve { background: url(/portfolio/styles/icon/flatui/stackorganizer/teaserstack-design-shelve.png?_dc=1); }

.x-ux-stack-icon .mediagallery { background: url(/portfolio/styles/icon/flatui/stackorganizer/mediagallery.png?_dc=1); }
.x-ux-stack-icon .mediagallery.design-gallery { background: url(/portfolio/styles/icon/flatui/stackorganizer/mediagallery-design-gallery.png?_dc=1); }
.x-ux-stack-icon .mediagallery.design-hero { background: url(/portfolio/styles/icon/flatui/stackorganizer/mediagallery-design-hero.png?_dc=1); }

.x-ux-stack-icon .teaser { background: url(/portfolio/styles/icon/flatui/stackorganizer/teaser.png?_dc=1); }

.x-ux-stack-icon .branch { background: url(/portfolio/styles/icon/flatui/stackorganizer/branch.png?_dc=1); }

.x-ux-stack-icon .quote { background: url(/portfolio/styles/icon/flatui/stackorganizer/quote.png?_dc=1); }

.x-ux-stack-icon .form { background: url(/portfolio/styles/icon/flatui/stackorganizer/form.png?_dc=1); }

.x-ux-stack-icon .widget { background: url(/portfolio/styles/icon/flatui/stackorganizer/widget.png?_dc=1); }

.x-ux-stack-icon .webcontainer { background: url(/portfolio/styles/icon/flatui/stackorganizer/webcontainer.png?_dc=1); }

/************************************
  FILE MANAGER CLASSES
************************************/

.x-ux-filemanager-images-treenode-public{
	background-image:  url(/system/shared/collection/icons/16x16/plain/server2.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-page{
	background-image:  url(/system/shared/collection/icons/16x16/plain/document_orientation_portrait.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-product{
	background-image:  url(/system/shared/collection/icons/16x16/plain/box_open.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-document{
	background-image:  url(/system/shared/collection/icons/16x16/plain/document_empty.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-folder{
	background-image:  url(/system/shared/collection/icons/16x16/plain/folder.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-contact{
	background-image:  url(/system/shared/collection/icons/16x16/plain/user.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-courier{
	background-image:  url(/system/shared/collection/icons/16x16/plain/delivery_truck.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-manufactor{
	background-image:  url(/system/shared/collection/icons/16x16/plain/factory.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-template{
	background-image:  url(/system/shared/collection/icons/16x16/plain/window_gear.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-treenode-productlabel{
	background-image:  url(/system/shared/collection/icons/16x16/plain/tag.png?_dc=1) !important;
	background-repeat: no-repeat;
}

.x-ux-filemanager-images-treenode-profile{
	background-image:  url(/system/shared/collection/icons/16x16/plain/painting_portrait.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-default-collapsed{
	background-image:  url(/system/shared/collection/icons/16x16/plain/folder.png?_dc=1) !important;
	background-repeat: no-repeat;
}
.x-ux-filemanager-images-upload{
	background-image:  url(/system/shared/collection/icons/16x16/plain/cloud_upload.png?_dc=1) !important;
	background-repeat: no-repeat;
	background-position: center;
}

/***** BREADCRUMB ICONS ****/
.x-breadcrumb-icon-folder-default {
	background-image:  url(/system/shared/collection/icons/16x16/plain/folder.png?_dc=1) !important;
	background-repeat: no-repeat;
}
