Ext.define('Ext.ux.data.dmd.Writer', {
	
	extend: 'Ext.data.writer.Json',
	
	alternateClassName: 'Ext.data.DmdWriter',
	
	alias: 'writer.dmd',

	root: 'rows',

	fields: null,

	nameProperty: 'name'
});
