if (!Ext.ClassManager.isCreated("Portfolio.message.model.Error")) {

	Ext.define("Portfolio.message.model.Error", {
		extend: "Ext.data.Model",
		fields: [
			{
				name: "id"
			},
			{
				name: "description"
			},
			{
				name: "message"
			},
			{
				name: "completeStackTrace"
			},
			{
				name: "errorType"
			},
			{
				name: "isRestException"
			},
			{
				name: "src"
			},
			{
				name: "lineNo"
			},
			{
				name: "date", type: "date"
			},
			{
				name: "urlText"
			},
			{
				name: "url "
			},
			{
				name: "debugType"
			},
			{
				name: "params"
			}
		]
	});
}
