Ext.define('Ext.ux.data.store.EditCode', {

	extend: 'Ext.data.Store',

	model: 'Ext.ux.data.model.EditCode',

	data: [
		{
			editcode: "",
			comma: "Commas Displayed",
			decimal: "Decimal Points Displayed",
			sign: "Sign Displayed When Negative Value",
			blank: "Blank Value of QDECFMT System Value",
			iValu: "I Value of QDECFMT System Value",
			jValue: "J Value of QDECFMT System Value",
			zero: "Leading Zero Suppressed"
		},
		{
			editcode: "1",
			comma: "Yes",
			decimal: "Yes",
			sign: "No sign",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "2",
			comma: "Yes",
			decimal: "Yes",
			sign: "No sign",
			blank: "Blanks",
			iValue: "Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "3",
			comma: "",
			decimal: "Yes",
			sign: "No sign",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "4",
			comma: "",
			decimal: "Yes",
			sign: "No sign",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "A",
			comma: "Yes",
			decimal: "Yes",
			sign: "CR",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "B",
			comma: "Yes",
			decimal: "Yes",
			sign: "CR",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "C",
			comma: "",
			decimal: "Yes",
			sign: "CR",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "D",
			comma: "",
			decimal: "Yes",
			sign: "CR",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "J",
			comma: "Yes",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "K",
			comma: "Yes",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "L",
			comma: "",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "M",
			comma: "",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "N",
			comma: "Yes",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "O",
			comma: "Yes",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "P",
			comma: "",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: ".00 or 0",
			iValue: ",00 or 0",
			jValue: "0,00 or 0",
			zero: "Yes"
		},
		{
			editcode: "Q",
			comma: "",
			decimal: "Yes",
			sign: "-(Minus)",
			blank: "Blanks",
			iValue: ",Blanks",
			jValue: "Blanks",
			zero: "Yes"
		},
		{
			editcode: "W",
			comma: "",
			decimal: "",
			sign: "",
			blank: "",
			iValue: "",
			jValue: "",
			zero: "Yes"
		},
		{
			editcode: "Y",
			comma: "",
			decimal: "",
			sign: "",
			blank: "",
			iValue: "",
			jValue: "",
			zero: "Yes"
		},
		{
			editcode: "Z",
			comma: "",
			decimal: "",
			sign: "",
			blank: "",
			iValue: "",
			jValue: "",
			zero: "Yes"
		}
	]
});
