if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityColumnDataModel")) {
	Ext.define("Portfolio.designer.model.domainModelEntityColumnDataModel", {
		extend: "Ext.data.Model",
		fields: [
			{ type: "auto", name: "name" },
			{ type: "auto", name: "title" }
		]
	});
}
