.yellow .bar {
    fill: rgb(236, 236, 0);
}
.yellow .bar-progress {
    fill: rgb(175, 175, 0);
}


.red .bar {
    fill: rgb(236, 0, 0);
}
.red .bar-progress {
    fill: rgb(175, 0, 0);
}

.green .bar {
    fill: rgb(0, 236, 0);
}
.green .bar-progress {
    fill: rgb(0, 175, 0);
}

.gantt .bar-label {
    fill: black;
}
.gantt {
    pointer-events: none
}