Ext.define("Ext.ux.form.field.Html", {

	extend: "Ext.form.FieldContainer",

	mixins: {
		field: 'Ext.form.field.Field'
	},

	isFormField: false,
	alias: "widget.htmlfield",

	initComponent: function () {

		var i_self = this;
		i_self.callParent();

		i_self.initField();
	},

	setValue: function(value) {

		var i_self = this;
		i_self.update(value);
	}
});
