Ext.define("Ext.ux.data.store.Locale", {

	extend: "Ext.data.Store",

	model: "Ext.ux.data.model.Locale",

	autoLoad: true,

	proxy: {
		type: "memory"
	},

	data: [
		{
			country: "Afrikan Republic",
			language: {
				da: "Afrikaans",
				en: "Afrikaans"
			},
			code: "af",
			flag: "flag_central_african_republic"
		},
		{
			country: "Bulgaria",
			language: {
				da: "Bulgarisk",
				en: "Bulgarian"
			},
			code: "bg",
			flag: "flag_bulgaria"
		},
		{
			country: "Czech Republic",
			language: {
				da: "Czech",
				en: "Czech"
			},
			code: "cs",
			flag: "flag_czech_republic"
		},
		{
			country: "Denmark",
			language: {
				da: "Dansk",
				en: "Danish"
			},
			code: "da",
			flag: "flag_da"
		},
		{
			country: "Germany",
			language: {
				da: "Tysk",
				en: "German"
			},
			code: "de",
			flag: "flag_de"
		},
		{
			country: "Great Britain",
			language: {
				da: "Engelsk",
				en: "English"
			},
			code: "en",
			flag: "flag_en"
		},
		{
			country: "Spain",
			language: {
				da: "Spansk",
				en: "Spanish"
			},
			code: "es",
			flag: "flag_es"
		},
		{
			country: "Estonia",
			language: {
				da: "Estonian",
				en: "Estonian"
			},
			code: "et",
			flag: "flag_estonia"
		},
		{
			country: "Iran",
			language: {
				da: "Persian",
				en: "Persian"
			},
			code: "fa",
			flag: "flag_iran"
		},
		{
			country: "Finland",
			language: {
				da: "Finnish",
				en: "Finnish"
			},
			code: "fi",
			flag: "flag_fi"
		},
		{
			country: "France",
			language: {
				da: "Fransk",
				en: "French"
			},
			code: "fr",
			flag: "flag_france"
		},
		{
			country: "Greece",
			language: {
				da: "Græsk",
				en: "Greek"
			},
			code: "gr",
			flag: "flag_greece"
		},
		{
			country: "Hebrew",
			language: {
				da: "Hebraisk",
				en: "Hebrew"
			},
			code: "he",
			flag: "flag_israel"
		},
		{
			country: "Croatian",
			language: {
				da: "Croatian",
				en: "Croatian"
			},
			code: "hr",
			flag: "flag_croatia"
		},
		{
			country: "Hungary",
			language: {
				da: "Hungarian",
				en: "Hungarian"
			},
			code: "hu",
			flag: "flag_hungary"
		},
		{
			country: "Indonesia",
			language: {
				da: "Indonesian",
				en: "Indonesian"
			},
			code: "id",
			flag: "flag_indonesia"
		},
		{
			country: "Italy",
			language: {
				da: "Italian",
				en: "Italian"
			},
			code: "it",
			flag: "flag_italy"
		},
		{
			country: "Japan",
			language: {
				da: "Japansk",
				en: "Japaness"
			},
			code: "ja",
			flag: "flag_japan"
		},
		{
			country: "Korea",
			language: {
				da: "Korean",
				en: "Korean"
			},
			code: "ko",
			flag: "flag_north_korea"
		},
		{
			country: "Lithuania",
			language: {
				da: "Lithuanian",
				en: "Lithuanian"
			},
			code: "lt",
			flag: "flag_lithuania"
		},
		{
			country: "Latvia",
			language: {
				da: "Latvian",
				en: "Latvian"
			},
			code: "lv",
			flag: "flag_latvia"
		},
		{
			country: "Macedonia (FYROM)",
			language: {
				da: "Macedonian (FYROM)",
				en: "Macedonian (FYROM)"
			},
			code: "mk",
			flag: "flag_macedonia"
		},
		{
			country: "Nederland",
			language: {
				da: "Hollandsk",
				en: "Dutch"
			},
			code: "nl",
			flag: "flag_netherlands_antilles"
		},
		{
			country: "Norway",
			language: {
				da: "Norsk (Bokmal)",
				en: "Norwegian (Bokmal)"
			},
			code: "no_NB",
			flag: "flag_no"
		},
		{
			country: "Poland",
			language: {
				da: "Polsk",
				en: "Polish"
			},
			code: "pl",
			flag: "flag_poland"
		},
		{
			country: "Portugal",
			language: {
				da: "Portuguese",
				en: "Portuguese"
			},
			code: "pt",
			flag: "flag_portugal"
		},
		{
			country: "Romania",
			language: {
				da: "Romanian",
				en: "Romanian"
			},
			code: "ro",
			flag: "flag_romania"
		},
		{
			country: "Russia",
			language: {
				da: "Russian",
				en: "Russian"
			},
			code: "ru",
			flag: "flag_russia"
		},
		{
			country: "Slovakia",
			language: {
				da: "Slovak",
				en: "Slovak"
			},
			code: "sk",
			flag: "flag_slovakia"
		},
		{
			country: "Slovenia",
			language: {
				da: "Slovenian",
				en: "Slovenian"
			},
			code: "sl",
			flag: "flag_slovenia"
		},
		{
			country: "Serbian",
			language: {
				da: "Serbian",
				en: "Serbian"
			},
			code: "sr",
			flag: "flag_serbia"
		},
		{
			country: "Sweden",
			language: {
				da: "Svensk",
				en: "Swedish"
			},
			code: "sv_SE",
			flag: "flag_sv"
		},
		{
			country: "Thailand",
			language: {
				da: "Thai",
				en: "Thai"
			},
			code: "th",
			flag: "flag_thailand"
		},
		{
			country: "Turkey",
			language: {
				da: "Turkish",
				en: "Turkish"
			},
			code: "tr",
			flag: "flag_turkey"
		},
		{
			country: "Ukraine",
			language: {
				da: "Ukrainian",
				en: "Ukrainian"
			},
			code: "ukr",
			flag: "flag_ukraine"
		},
		{
			country: "Vietnam",
			language: {
				da: "vietnamesisk",
				en: "Vietnamese"
			},
			code: "vn",
			flag: "flag_vietnam"
		},
		{
			country: "People's Republic of China",
			language: {
				da: "Chinese (Simplified) Legacy",
				en: "Chinese (Simplified) Legacy"
			},
			code: "zh_CN",
			flag: "flag_china"
		},
		{
			country: "Taiwan",
			language: {
				da: "Chinese (Traditional) Legacy",
				en: "Chinese (Traditional) Legacy"
			},
			code: "zh_TW",
			flag: "flag_china"
		}
	]
});
