if (!Ext.ClassManager.isCreated("Ext.ux.data.model.SnapOnIcon")) {
	Ext.define("Ext.ux.data.model.SnapOnIcon", {

		extend: "Ext.data.Model",

		alias: ["model.snaponicon"],

		fields: [
			{ name: "icon" },
			{ name: "score", type: "int" }
		],

		proxy: {
			type: "ajax",
			url: (ip2.isOffline) ? "ip2-services/ip2DmdProxy.php?action=snapOnIcons" : "/system/snapOnIcon.aspx?list=true",
			reader: {
				root: "rows"
			}
		}
	});
}

