Ext.define("Ext.ux.form.field.i18nHook.Combo", {

	extend: "Ext.ux.form.field.i18n.Combo",

	alias: ["widget.i18nhookcombo"],

	initComponent: function () {

		var i_self = this;

		i_self.callParent(arguments);

		return i_self;
	}
});
