if (!Ext.ClassManager.isCreated("Ext.ux.data.model.WebBios")) {
	Ext.define("Ext.ux.data.model.WebBios", {
		extend: "Ext.data.Model",
		alias: [
			"model.webbios"
		],
		fields: [
			{
				name: "id",
				type: "string"
			},
			{
				name: "path",
				type: "string"
			},
			{
				name: "text",
				type: "string"
			},
			{
				name: "ext",
				type: "string"
			},
			{
				name: "cls",
				type: "string"
			},
			{
				name: "leaf",
				type: "boolean"
			},
			{
				name: "mtime",
				type: "string"
			},
			{
				name: "size",
				type: "int",
				digits: 15
			}
		]
	});
}
