if (!Ext.ClassManager.isCreated("Portfolio.designer.model.domainModelEntityAffiliationManagerDataModel")) {
	Ext.define("Portfolio.designer.model.domainModelEntityAffiliationManagerDataModel", {
		extend: "Ext.data.Model",
		fields: [
			{ type: "string", name: "name" },
			{ type: "string", name: "title" },
			{ type: "string", name: "url" },
			{ type: "boolean", name: "drop" },
			{ type: "auto", name: "position" },
			{ type: "auto", name: "component" }
		]
	});
}
