if (!Ext.ClassManager.isCreated("Portfolio.menu.model.System")) {
	Ext.define("Portfolio.menu.model.System", {
		extend: "Ext.data.Model",

		alternateClassName: [
			"treeMenu"
		],

		fields: [
			{
				name: "id"
			},
			{
				name: "rrn",
				type: "int"
			},
			{
				name: "type"
			},
			{
				name: "system",
				type: "int"
			},
			{
				name: "applicationGroup",
				type: "int"
			},
			{
				name: "application",
				type: "int"
			},
			{
				name: "text"
			},
			{
				name: "url"
			},
			{
				name: "iconCls"
			},
			{
				name: "leaf",
				type: "boolean"
			},
			{
				name: "expanded",
				type: "boolean"
			}
		],
		idProperty: "id"
	});
}
