Ext.define("Portfolio.registry.viewmodel.Registry", {

	extend: "Ext.app.ViewModel",

	alias: "viewmodel.registry",

	alternateClassName: [
		"Ext.registry.viewmodel.Registry"
	],

	data: {
		tree: null,
		record: null
	}
});
