@media print {
	html, body {
		zoom: 80%;
	}
	@page {
		size: A4 landscape;
	}

}

.x-btn-default-small {
	border-color: #1c74d5;
	background-color: #1c74d5;
	color: white;
}

body {
  background:url("/portfolio/Themes/Gray/desktop.jpg") no-repeat center center fixed;
}

/**
	ip2.cass
**/

.ext-ie6 .x-small-editor .x-form-text, .ext-ie7 .x-small-editor .x-form-text {
	margin-top:1px !important; /* ie bogus margin bug */
	margin-bottom:-1px !important;
	height:20px !important; /* ie quirks */
	line-height:16px !important;
}
.ext-ie6 .x-form-text, .ext-ie7 .x-form-text {
		margin:0px; /* ie bogus margin bug */
		height:22px; /* ie quirks */
		line-height:18px;
}

.disabled-row .x-grid-cell {
		background-color: #bcbcbc !important;
}​

/* ccs for check column */
.x-grid-checkheader {
		height: 14px;
		background-image: url("/portfolio/Themes/Default/unchecked.gif");
		background-position: 50% -2px;
		background-repeat: no-repeat;
		background-color: transparent;
}

.x-grid-checkheader-checked {
		background-image: url("/portfolio/Themes/Default/checked.gif");
}

.x-grid-checkheader-editor .x-form-cb-wrap {
		text-align: center;
}

.x-form-item-label {
	color:#000000;
}
.x-form-item-label-suffix .x-panel-body {
	color: #727272 !important;
}

.x-item-disabled {
	opacity: 0.8 !important;
}
.x-boundlist-item.x-item-disabled {
	font-style: italic;
}

.x-boundlist {
	overflow: "auto";
}

.x-splitter {
	border:solid 1px #aeb6be;
}

.ip2-grid-row-XL {
	height: 24px;
}

.ip2-titlefield-over {
	border-top: solid 1px #666666 !important;
	border-right: solid 1px #666666 !important;
}
.ip2-titlefield {
	background: #ffffff url(/system/images/flags/16x16/flag_generic.png) no-repeat 0 3px !important;
	border-top: solid 1px #B5B8C8;
	border-right: solid 1px #B5B8C8;
}

.ip2-i18n-wrap {
	float:left;
	width:17px!important;
	height:22px!important;
	font-size:0;
	padding-top:2px;
	border-left: solid 1px #7eadd9;
	border-bottom: solid 1px #7eadd9;
}
.ip2-i18n-wrap-parent {
	border-top: solid 1px #7eadd9;
}
.ip2-i18n-wrap-child {
	border-top: none;
}
.ip2-i18n-trigger {
	background: #ffffff url(/system/images/flags/16x16/flag_generic.png) no-repeat 0 3px !important;
	border-top: solid 1px #B5B8C8;
	border-right: solid 1px #B5B8C8;
}
.ip2-i18n-trigger-over {
	border-top: solid 1px #666666 !important;
	border-right: solid 1px #666666 !important;
}
.ip2-i18n-field-wrap {
	margin: 0;
	border-top: none;
}

.ip2-designer-form {
	background-image: url(/system/images/icons/16x16/window.png) !important;
}
.ip2-designer-portlet {
	background-image: url(/system/images/icons/16x16/window_colors.png) !important;
}
.ip2-designer-portlet-add {
	background-image: url(/portfolio/Themes/Default/portlet_add.png) !important;
}
.ip2-designer-column {
	background-image: url(/system/images/icons/16x16/column.png) !important;
}
.ip2-designer-column-add {
	background-image: url(/system/images/icons/16x16/column_add.png) !important;
}
.ip2-designer-field {
	background-image: url(/system/images/icons/16x16/data.png) !important;
}
.ip2-designer-relation-field {
	background-image: url(/system/images/icons/16x16/data_connection.png) !important;
}
.ip2-designer-entity {
	background-image: url(/system/images/icons/16x16/table.png) !important;
}
.ip2-designer-parentRelation {
	background-image: url(/system/images/icons/16x16/table_connection.png) !important;
}
.ip2-designer-affiliations {
	background-image: url(/system/images/icons/16x16/component.png) !important;
}
.ip2-designer-widgets {
	background-image: url(/system/images/icons/16x16/gear.png) !important;
}

.ip2-gridLayout-entity {
	background-image: url(/system/shared/ext4/resources/themes/images/default/grid/columns.gif) !important;
}

.ip2-love-trigger-search {
	background-image: url(/system/shared/ext4/resources/themes/images/gray/form/search-trigger.gif) !important;
}
.ip2-love-trigger-clear {
	background-image: url(/system/shared/ext4/resources/themes/images/gray/form/clear-trigger.gif) !important;
}


.xxip2-love-search-item {
	padding: 5px 10px;
}

.ip2-editcode-boundlist-cell {
	width:100px;
	white-space:normal;
	text-align:center;
}
.ip2-editcode-boundlist-firstRow {
	background: none repeat scroll 0 0 #cccccc;
}
.ip2-boundlist-cell {
	white-space:normal;
	text-align:left;
	font-size:12px;
}
.ip2-hierarchy-header {
	font-family: "Helvetica Neue",Helvetica,Arial,"Lucida Grande",sans-serif;
	font-size: 90%;
	padding: 9px;
	text-transform: uppercase;
	color: #999;
}
.ip2-tree-node-disabled .x-grid-cell {
	 -moz-opacity: 0.5;
	 opacity:.5;
	 filter: alpha(opacity=50);
}

.ip2-tree-node-disabled .x-grid-cell-inner {
	cursor:default !important;
}

.ip2-label {
	padding-left: 10px;
}

.ip2-shortcut {
	position:absolute;
	left: 200px;
}

.x-editor .x-form-check-wrap {
	background-color: #FFFFFF;
	text-align: center;
}
.x-editor .x-form-checkbox {
	height: 13px;
	margin: 0 0 5px 10px;
	padding-left: 2px;
}

.ip2-slider {
		width:214px;
		padding-top:6px;
		background-image:url(/portfolio/Themes/Default/ticks.gif);
}
.ip2-slider .x-slider-horz .x-slider-thumb {
		background-image:url(/portfolio/Themes/Default/slider-thumb.png);
}

.ip2-source-editor {
	font-family: courier,monospace !important;;
}

.ip2-message-title {
	margin-left: 4px;
	color: #15428B;
	font-family: tahoma,arial,verdana,sans-serif;
	font-size: 11px;
	font-weight: bold;
}
.ip2-message-title-icon {
	background-repeat:no-repeat;
	width:16px;
	float:left;

	background-image:url(/system/shared/collection/icons/16x16/plain/sign_warning.png);
	background-position: center center !important;
}
.ip2-message-title-icon.log {
	background-image:url(/system/shared/collection/icons/16x16/plain/about.png);
}
.ip2-message-title-icon.js {
	background-image:url(/system/shared/collection/icons/16x16/plain/error.png);
}
.ip2-message-title-icon.ajax {
	background-image:url(/system/shared/collection/icons/16x16/plain/sign_warning.png);
}

.ip2-panel-maximized {
	z-index:10001;
}

.ip2-readOnly {
	color: #aaaaaa;
}

.ip2-cell-invalid, textarea.ip2-cell-invalid {
	background: #fff url(/system/shared/ext4/resources/themes/images/default/grid/invalid_line.gif) repeat-x bottom;
}

.ip2-default {
	font-weight:bold !important;
}

/* icons */
/*
	Browse icons using your browser at:
		/System/images/icons/16x16/16x16.html
*/

.ip2-ux-icon-add {
	background-image:url(/images/icons/ip2/add-sprite.png) !important;
	width: 15px !important;
}
.ip2-ux-icon-edit {
	background-image:url(/images/icons/ip2/edit-sprite.png) !important;
	width: 16px !important;
}
.ip2-ux-icon-delete {
	background-image:url(/images/icons/ip2/delete-sprite.png) !important;
	width: 15px !important;
}
.x-btn-default-ip2 {
		-moz-border-radius: 0px;
		-webkit-border-radius: 0px;
		-o-border-radius: 0px;
		-ms-border-radius: 0px;
		-khtml-border-radius: 0px;
		border-radius: 0px;
		padding: 3px 0px 0px 0px;
		border: none;
}f
.x-btn-default-ip2-focus .x-btn-icon {
		background-position-x: -15px;
}
.x-btn-default-ip2-icon button, .x-btn-default-ip2-icon .x-btn-inner, .x-btn-default-ip2-noicon button, .x-btn-default-ip2-noicon .x-btn-inner {
		height: 16px;
		line-height: 16px;
}
.x-btn-default-ip2-icon button {
		padding: 0;
		width: 15px !important;
		height: 16px;
}
.x-btn-default-ip2-icon .x-btn-icon {
		width: 15px;
		height: 16px;
		top: 0;
		left: 0;
		bottom: 0;
		right: 0;
}

.ip2-gridaction-icon {
		padding: 3px 0px 0px 0px;
		border: none;
		width: 15px;
		height: 16px;
		top: 2px;
		left: 0;
		bottom: 0;
		right: 0;
}
.ip2-ux-icon-edit:hover {
		background-position-x: -16px !important;
}
.ip2-gridaction-icon:hover {
		background-position-x: -15px;
}

.ip2-icon-tab {
	background-position: 0 -2px;
}

.ip2-icon-file {
	background-image:url(/system/images/icons/16x16/document.png) !important;
}
.ip2-icon-new-file {
	background-image:url(/system/images/icons/16x16/document_add.png) !important;
}
.ip2-icon-unsaved-file {
	background-image:url(/system/images/icons/16x16/document_new.png) !important;
}
.ip2-icon-folder {
	background-image:url(/system/images/icons/16x16/folder.png) !important;
}
.ip2-icon-new-folder {
	background-image:url(/system/images/icons/16x16/folder_add.png) !important;
}
.ip2-icon-unsaved-folder {
	background-image:url(/system/images/icons/16x16/folder_new.png) !important;
}

.ip2-icon-up {
	background-image:url(/system/images/icons/16x16/arrow_up_green.png) !important;
}

.ip2-icon-folder-href {
	background-image:url(/system/images/icons/16x16/folder_window.png) !important;
}
.ip2-icon-folder-page {
	background-image:url(/system/images/icons/16x16/folder_document.png) !important;
}
.ip2-icon-view {
	background-image:url(/system/shared/collection/icons/16x16/plain/magnifying_glass.png) !important;
}
.ip2-icon-add {
	background-image:url(/system/shared/collection/icons/16x16/plain/add.png) !important;
}
.ip2-icon-edit {
	background-image:url(/system/shared/collection/icons/16x16/plain/edit.png) !important;
}
.ip2-icon-copy {
	background-image:url(/system/shared/collection/icons/16x16/plain/copy.png) !important;
}
.ip2-icon-documents {
	background-image:url(/system/shared/collection/icons/16x16/plain/document_orientation_portrait.png) !important;
}
.ip2-icon-delete {
	background-image:url(/system/shared/collection/icons/16x16/plain/error.png) !important;
}
.ip2-icon-remove {
	background-image:url(/system/shared/collection/icons/16x16/plain/delete.png) !important;
}
.ip2-icon-deselect {
	background-image:url(/system/images/icons/16x16/row.png) !important;
}
.ip2-icon-property {
	background-image:url(/system/images/icons/16x16/gear.png) !important;
}
.ip2-icon-gear {
	background-image:url(/system/images/icons/16x16/gear.png) !important;
}
.ip2-icon-gear-edit {
	background-image:url(/system/images/icons/16x16/gear_edit.png) !important;
}
.ip2-icon-refresh {
	background-image:url(/system/shared/collection/icons/16x16/plain/refresh.png) !important;
}
.ip2-icon-undo {
	background-image:url(/system/images/icons/16x16/undo.png) !important;
}
.ip2-icon-checks {
	background-image:url(/system/images/icons/16x16/checks.png) !important;
}


.ip2-icon-row {
	background-image:url(/system/images/icons/16x16/row.png) !important;
}
.ip2-icon-row-add {
	background-image:url(/system/images/icons/16x16/row_add.png) !important;
}
.ip2-icon-row-add-after {
	background-image:url(/system/images/icons/16x16/row_add_after.png) !important;
}
.ip2-icon-row-add-before {
	background-image:url(/system/images/icons/16x16/row_add_before.png) !important;
}
.ip2-icon-row-delete {
	background-image:url(/system/images/icons/16x16/row_delete.png) !important;
}
.ip2-icon-row-edit {
	background-image:url(/system/images/icons/16x16/row_edit.png) !important;
}
.ip2-icon-row-preferences {
	background-image:url(/system/images/icons/16x16/row_preferences.png) !important;
}

.ip2-icon-expand {
	background-image:url(/system/shared/collection/icons/16x16/plain/elements_tree.png) !important;
}
.ip2-icon-collapse {
	background-image:url(/system/shared/collection/icons/16x16/plain/text.png) !important;
}
.ip2-icon-save {
	background-image:url(/system/images/icons/16x16/disk_blue.png) !important;
}
.ip2-icon-close {
	background-image:url(/system/images/icons/16x16/door2.png) !important;
}
.ip2-icon-restart {
	background-image:url(/system/images/icons/16x16/nav_refresh_green.png) !important;
}
.ip2-icon-search {
	background-image:url(/system/images/icons/16x16/find.png) !important;
}
.ip2-icon-list {
	background-image:url(/system/images/icons/16x16/text_align_justified.png) !important;
}
.ip2-icon-add-relation {
	background-image:url(/system/images/icons/16x16/link_add.png) !important;
}
.ip2-icon-add-entity {
	background-image:url(/system/images/icons/16x16/text.png) !important;
}
.ip2-icon-add-link {
	background-image:url(/system/images/icons/16x16/link.png) !important;
}
.ip2-icon-add-dmd {
	background-image:url(/system/images/icons/16x16/cube_molecule.png) !important;
}
.ip2-icon-import-entity {
	background-image:url(/system/images/icons/16x16/import1.png) !important;
}
.ip2-icon-new-domain-model {
	background-image:url(/system/images/icons/16x16/data_new.png) !important;
}
.ip2-icon-nav-redo-blue {
	background-image:url(/system/images/icons/16x16/nav_redo_blue.png) !important;
}
.ip2-icon-nav-undo-blue {
	background-image:url(/system/images/icons/16x16/nav_undo_blue.png) !important;
}
.ip2-icon-open-domain-model {
	background-image:url(/system/images/icons/16x16/data_into.png) !important;
}
.ip2-icon-save-domain-model {
	background-image:url(/system/images/icons/16x16/disk_blue.png) !important;
}
.ip2-icon-close-domain-model {
	background-image:url(/system/images/icons/16x16/data_previous.png) !important;
}
.ip2-icon-domain-model-properties {
	background-image:url(/system/images/icons/16x16/data_preferences.png) !important;
}
.ip2-icon-domain-model-newEntity {
	background-image:url(/system/images/icons/16x16/add.png) !important;
}
.ip2-icon-import-table {
	background-image:url(/system/images/icons/16x16/table_new.png) !important;
}
.ip2-icon-basedatatypes {
	background-image:url(/system/images/icons/16x16/dictionary.png) !important;
}

.ip2-icon-branch-find {
	background-image:url(/system/images/icons/16x16/branch_find.png) !important;
}
.ip2-icon-branch-view {
	background-image:url(/system/images/icons/16x16/branch_view.png) !important;
}

.ip2-icon-bullet-delete {
	 background-image:url(/system/images/icons/16x16/bullet_delete.png) !important;
}
.ip2-icon-bullet-picture {
	 background-image:url(/system/images/icons/16x16/bullet_picture.png) !important;
}


.ip2-icon-bullet-ball-green {
	 background-image:url(/system/images/icons/16x16/bullet_ball_green.png) !important;
}
.ip2-icon-bullet-ball-red {
	 background-image:url(/system/images/icons/16x16/bullet_ball_red.png) !important;
}

.ip2-icon-translate {
	background-image:url(/system/images/flags/16x16/flag_generic.png) !important;
}
.ip2-icon-menuOrganizer {
	background-image:url(/system/images/icons/16x16/text_tree.png) !important;
}
.ip2-icon-sourceEditor {
	background-image:url(/system/images/icons/16x16/document_edit.png) !important;
}

.ip2-icon-undo {
	background-image:url(/system/images/icons/16x16/undo.png) !important;
}
.ip2-icon-redo {
	background-image:url(/system/images/icons/16x16/redo.png) !important;
}

.ip2-icon-window-colors {
		background-image:url(/system/images/icons/16x16/window_colors.png) !important;
}

.ip2-icon-window-application {
		background-image:url(/system/images/icons/16x16/window_application.png) !important;
}
.ip2-icon-window-application-edit {
		background-image:url(/system/images/icons/16x16/window_application_edit.png) !important;
}

.ip2-icon-exit {
	background-image:url(/system/images/icons/16x16/exit.png) !important;
}



.ip2-icon-icecube {
	background-image: url(/system/images/icons/16x16/icecube.png) !important;
}

.ip2-icon-data {
	background-image: url(/system/images/icons/16x16/data.png) !important;
}
.ip2-icon-data-add {
	background-image: url(/system/images/icons/16x16/data_add.png) !important;
}
.ip2-icon-data-connection {
	background-image: url(/system/images/icons/16x16/data_connection.png) !important;
}
.ip2-icon-data-certificate {
	background-image: url(/system/images/icons/16x16/data_certificate.png) !important;
}
.ip2-icon-data-copy {
	background-image: url(/system/images/icons/16x16/data_copy.png) !important;
}
.ip2-icon-data-delete {
	background-image: url(/system/images/icons/16x16/data_delete.png) !important;
}
.ip2-icon-data-disk {
	background-image: url(/system/images/icons/16x16/data_disk.png) !important;
}
.ip2-icon-data-down {
	background-image: url(/system/images/icons/16x16/data_down.png) !important;
}
.ip2-icon-data-edit {
	background-image: url(/system/images/icons/16x16/data_edit.png) !important;
}
.ip2-icon-data-error {
	background-image: url(/system/images/icons/16x16/data_error.png) !important;
}
.ip2-icon-data-find {
	background-image: url(/system/images/icons/16x16/data_find.png) !important;
}
.ip2-icon-data-forbidden {
	background-image: url(/system/images/icons/16x16/data_forbidden.png) !important;
}
.ip2-icon-data-gear {
	background-image: url(/system/images/icons/16x16/data_gear.png) !important;
}
.ip2-icon-data-information {
	background-image: url(/system/images/icons/16x16/data_information.png) !important;
}
.ip2-icon-data-into {
	background-image: url(/system/images/icons/16x16/data_into.png) !important;
}
.ip2-icon-data-lock {
	background-image: url(/system/images/icons/16x16/data_lock.png) !important;
}
.ip2-icon-data-new {
	background-image: url(/system/images/icons/16x16/data_new.png) !important;
}
.ip2-icon-data-next {
	background-image: url(/system/images/icons/16x16/data_next.png) !important;
}
.ip2-icon-data-out {
	background-image: url(/system/images/icons/16x16/data_out.png) !important;
}
.ip2-icon-data-ok {
	background-image: url(/system/images/icons/16x16/data_ok.png) !important;
}
.ip2-icon-data-preferences {
	background-image: url(/system/images/icons/16x16/data_preferences.png) !important;
}
.ip2-icon-data-previous {
	background-image: url(/system/images/icons/16x16/data_previous.png) !important;
}
.ip2-icon-data-refresh {
	background-image: url(/system/images/icons/16x16/data_refresh.png) !important;
}
.ip2-icon-data-replace {
	background-image: url(/system/images/icons/16x16/data_replace.png) !important;
}
.ip2-icon-data-scroll {
	background-image: url(/system/images/icons/16x16/data_scroll.png) !important;
}
.ip2-icon-data-table {
	background-image: url(/system/images/icons/16x16/data_table.png) !important;
}
.ip2-icon-data-time {
	background-image: url(/system/images/icons/16x16/data_time.png) !important;
}
.ip2-icon-data-unknown {
	background-image: url(/system/images/icons/16x16/data_unknown.png) !important;
}
.ip2-icon-data-up {
	background-image: url(/system/images/icons/16x16/data_up.png) !important;
}
.ip2-icon-data-view {
	background-image: url(/system/images/icons/16x16/data_view.png) !important;
}
.ip2-icon-data-warning {
	background-image: url(/system/images/icons/16x16/data_warning.png) !important;
}





.ip2-icon-cube-molecule {
	background-image: url(/system/images/icons/16x16/cube_molecule.png) !important;
}
.ip2-icon-cube-molecule-add {
	background-image: url(/system/images/icons/16x16/cube_molecule_add.png) !important;
}
.ip2-icon-cube-molecule-connection {
	background-image: url(/system/images/icons/16x16/cube_molecule_connection.png) !important;
}
.ip2-icon-cube-molecule-certificate {
	background-image: url(/system/images/icons/16x16/cube_molecule_certificate.png) !important;
}
.ip2-icon-cube-molecule-copy {
	background-image: url(/system/images/icons/16x16/cube_molecule_copy.png) !important;
}
.ip2-icon-cube-molecule-delete {
	background-image: url(/system/images/icons/16x16/cube_molecule_delete.png) !important;
}
.ip2-icon-cube-molecule-disk {
	background-image: url(/system/images/icons/16x16/cube_molecule_disk.png) !important;
}
.ip2-icon-cube-molecule-down {
	background-image: url(/system/images/icons/16x16/cube_molecule_down.png) !important;
}
.ip2-icon-cube-molecule-edit {
	background-image: url(/system/images/icons/16x16/cube_molecule_edit.png) !important;
}
.ip2-icon-cube-molecule-error {
	background-image: url(/system/images/icons/16x16/cube_molecule_error.png) !important;
}
.ip2-icon-cube-molecule-find {
	background-image: url(/system/images/icons/16x16/cube_molecule_find.png) !important;
}
.ip2-icon-cube-molecule-forbidden {
	background-image: url(/system/images/icons/16x16/cube_molecule_forbidden.png) !important;
}
.ip2-icon-cube-molecule-gear {
	background-image: url(/system/images/icons/16x16/cube_molecule_gear.png) !important;
}
.ip2-icon-cube-molecule-information {
	background-image: url(/system/images/icons/16x16/cube_molecule_information.png) !important;
}
.ip2-icon-cube-molecule-into {
	background-image: url(/system/images/icons/16x16/cube_molecule_into.png) !important;
}
.ip2-icon-cube-molecule-lock {
	background-image: url(/system/images/icons/16x16/cube_molecule_lock.png) !important;
}
.ip2-icon-cube-molecule-new {
	background-image: url(/system/images/icons/16x16/cube_molecule_new.png) !important;
}
.ip2-icon-cube-molecule-next {
	background-image: url(/system/images/icons/16x16/cube_molecule_next.png) !important;
}
.ip2-icon-cube-molecule-out {
	background-image: url(/system/images/icons/16x16/cube_molecule_out.png) !important;
}
.ip2-icon-cube-molecule-ok {
	background-image: url(/system/images/icons/16x16/cube_molecule_ok.png) !important;
}
.ip2-icon-cube-molecule-preferences {
	background-image: url(/system/images/icons/16x16/cube_molecule_preferences.png) !important;
}
.ip2-icon-cube-molecule-previous {
	background-image: url(/system/images/icons/16x16/cube_molecule_previous.png) !important;
}
.ip2-icon-cube-molecule-refresh {
	background-image: url(/system/images/icons/16x16/cube_molecule_refresh.png) !important;
}
.ip2-icon-cube-molecule-replace {
	background-image: url(/system/images/icons/16x16/cube_molecule_replace.png) !important;
}
.ip2-icon-cube-molecule-scroll {
	background-image: url(/system/images/icons/16x16/cube_molecule_scroll.png) !important;
}
.ip2-icon-cube-molecule-table {
	background-image: url(/system/images/icons/16x16/cube_molecule_table.png) !important;
}
.ip2-icon-cube-molecule-time {
	background-image: url(/system/images/icons/16x16/cube_molecule_time.png) !important;
}
.ip2-icon-cube-molecule-unknown {
	background-image: url(/system/images/icons/16x16/cube_molecule_unknown.png) !important;
}
.ip2-icon-cube-molecule-up {
	background-image: url(/system/images/icons/16x16/cube_molecule_up.png) !important;
}
.ip2-icon-cube-molecule-view {
	background-image: url(/system/images/icons/16x16/cube_molecule_view.png) !important;
}
.ip2-icon-cube-molecule-warning {
	background-image: url(/system/images/icons/16x16/cube_molecule_warning.png) !important;
}

.ip2-icon-link {
	background-image:url(/system/images/icons/16x16/link.png) !important;
}
.ip2-icon-link-add {
	background-image:url(/system/images/icons/16x16/link_add.png) !important;
}
.ip2-icon-link-new {
	background-image:url(/system/images/icons/16x16/link_new.png) !important;
}


.ip2-icon-table {
	background-image:url(/system/images/icons/16x16/table.png) !important;
}
.ip2-icon-table-add {
	background-image:url(/system/images/icons/16x16/table_add.png) !important;
}
.ip2-icon-table-new {
	background-image:url(/system/images/icons/16x16/table_new.png) !important;
}
.ip2-icon-table-selection-column {
	background-image:url(/system/images/icons/16x16/table_selection_column.png) !important;
}
.ip2-icon-table-view {
	background-image:url(/system/images/icons/16x16/table_view.png) !important;
}


.ip2-icon-flag-da, .ip2-icon-flag-denmark {
	background-image: url(/system/images/icons/16x16/flag_denmark.png) !important;
}
.ip2-icon-flag-en, .ip2-icon-flag-great-britain {
	background-image: url(/system/images/icons/16x16/flag_great_britain.png) !important;
}
.ip2-icon-flag-de, .ip2-icon-flag-germany {
	background-image: url(/system/images/icons/16x16/flag_germany.png) !important;
}
.ip2-icon-flag-sv, .ip2-icon-flag-sweden {
	background-image: url(/system/images/icons/16x16/flag_sweden.png) !important;
}
.ip2-icon-flag-no, .ip2-icon-flag-norway {
	background-image: url(/system/images/icons/16x16/flag_norway.png) !important;
}






.ip2-icon-disk-blue {
	background-image:url(/system/images/icons/16x16/disk_blue.png) !important;
}
.ip2-icon-disk-blue-add {
	background-image:url(/system/images/icons/16x16/disk_blue_add.png) !important;
}
.ip2-icon-disk-blue-delete {
	background-image:url(/system/images/icons/16x16/disk_blue_delete.png) !important;
}
.ip2-icon-disk-blue-edit {
	background-image:url(/system/images/icons/16x16/disk_blue_edit.png) !important;
}
.ip2-icon-disk-blue-error {
	background-image:url(/system/images/icons/16x16/disk_blue_error.png) !important;
}
.ip2-icon-disk-blue-information {
	background-image:url(/system/images/icons/16x16/disk_blue_information.png) !important;
}
.ip2-icon-disk-blue-ok {
	background-image:url(/system/images/icons/16x16/disk_blue_ok.png) !important;
}
.ip2-icon-disk-blue-warning {
	background-image:url(/system/images/icons/16x16/disk_blue_warning.png) !important;
}
.ip2-icon-disk-blue-delete {
	background-image:url(/system/images/icons/16x16/disk_blue_delete.png) !important;
}
.ip2-icon-disk-blue-window {
	background-image:url(/system/images/icons/16x16/disk_blue_window.png) !important;
}

.ip2-icon-find-text {
	background-image:url(/system/images/icons/16x16/find_text.png) !important;
}

.ip2-icon-replace {
	background-image:url(/system/images/icons/16x16/replace.png) !important;
}
.ip2-icon-text-code-colored {
	background-image:url(/system/images/icons/16x16/text_code_colored.png) !important;
}

.ip2-icon-font {
	background-image:url(/system/images/icons/16x16/font.png) !important;
}

.ip2-icon-1 {
	background-image:url(/portfolio/Themes/Default/1.png) !important;
}
.ip2-icon-2 {
	background-image:url(/portfolio/Themes/Default/2.png) !important;
}
.ip2-icon-3 {
	background-image:url(/portfolio/Themes/Default/3.png) !important;
}
.ip2-icon-4 {
	background-image:url(/portfolio/Themes/Default/4.png) !important;
}
.ip2-icon-5 {
	background-image:url(/portfolio/Themes/Default/5.png) !important;
}
.entity {
	background-image: url(/system/images/icons/16x16/form.png) !important;
}
.entityType {
	background-image: url(/system/images/icons/16x16/list.png) !important;
}
.ip2-icon-grid {
	background-image:url(/system/images/icons/16x16/list.png) !important;
	background-repeat:no-repeat;
}
.ip2-icon-form {
	background-image:url(/system/images/icons/16x16/form.png) !important;
	background-repeat:no-repeat;
}



.ip2-icon-xpgrid {
	background-image:url(/system/images/icons/16x16/package_edit.png) !important;
	background-repeat:no-repeat;
}
.ip2-icon-entity-uiexplorer {
	background-image:url(/system/images/icons/16x16/branch_view.png) !important;
	background-repeat:no-repeat;
}
.ip2-icon-entity-columnsexplorer {
	background-image:url(/system/images/icons/16x16/column_edit.png) !important;
	background-repeat:no-repeat;
}



.x-panel-inline-icon {
	background-position: 0 -2px;
}
.x-window-header {
	background-position: 0 5px;
}
.ip2-header-tool-image {
	background-image: url(/system/shared/ext4/resources/images/default/panel/tool-sprites.gif);
	width:15px;
	height:15px;
	position:absolute;
	right:8px;
	top:8px;
}
.ip2-header-tool {
	background-position: 0 -210px;
}
.ip2-header-tool-over {
	background-position: -15px -210px;
}
.ip2-header-tool-collapsed {
	background-position: 0 -195px;
}
.ip2-header-tool-collapsed-over {
	background-position: -15px -195px;
}

.ip2-prompt-text {
	font-family:tahoma,​arial,​verdana,​sans-serif;
	font-size:12px;
	font-weight:400;
	color:#000000;
}
.ip2-prompt-header {
	font-size:14px;
	font-weight:600;
	padding:0 0 14px 0;
}
.ip2-prompt-indent {
	padding:0 0 14px 14px;
}

.x-tool-exit {
	background-image: url("/portfolio/Themes/Default/tool-close.png") !important;
	background-position:0 0;
}
.x-tool-over .x-tool-exit {
	background-position:-15px 0;
}
.x-tool-collapsed {
background-position:0 -75px;
}
.x-tool-over .x-tool-collasped {
background-position:-15px -75px;
}

.ip2-hidden-iframe {
	position: absolute;
	top: 0px;
	left: 0px;
	width: 0px;
	height: 0px;
	opacity: 1;
	border:none;
	filter:alpha(opacity=0);
}

.ip2-file-upload {
	position: absolute;
	top: 2px;
	right: 37px;
	width: 20px;
	height: 15px;
	opacity: 1;
	cursor: pointer;
	margin-botom: 4px;
	padding: 0px;
	white-space: nowrap;
	x-style: dotted;
	border-style: none;
	background: url(/portfolio/Themes/Default/upload.png) no-repeat;
}

.ip2-file-download {
	position: absolute;
	top: 2px;
	right: 22px;
	width: 15px;
	height: 15px;
	cursor: pointer;
	display: block;
	white-space: nowrap;
	background: url(/portfolio/Themes/Default/download.gif) no-repeat;
}
.ip2-file-delete {
	position: absolute;
	top: 2px;
	right: 4px;
	width: 15px;
	height: 15px;
	cursor: pointer;
	display: block;
	white-space: nowrap;
	background: url(/portfolio/Themes/Default/delete.gif) no-repeat;
}

.ip2-file-loading {
	background: url(/portfolio/Themes/Default/loading.gif) no-repeat;
}
.ip2-file-done {
	background: url(/portfolio/Themes/Default/done.gif) no-repeat;
}
.ip2-file-error {
	background: url(/portfolio/Themes/Default/error.gif) no-repeat;
}

.ip2-trigger-browse {
	background-image:url(/portfolio/Themes/Default/trigger-browse.png) !important;
}
.ip2-no-tree-icon {
	background: none !important;
	width: 0px !important;
}
.ip2-tree-system {
	background-image: url(/system/shared/collection/icons/16x16/plain/server.png) !important;
}
.ip2-tree-application-group {
	background-image: url(/system/shared/collection/icons/16x16/plain/windows.png) !important;
}
.ip2-tree-application {
	background-image: url(/system/shared/collection/icons/16x16/plain/window_explorer.png) !important;
}
.icon-fav {
	background: url(/portfolio/Themes/Default/icons.png) no-repeat -3px -326px !important;
}

.class-tree .x-grid-cell-inner { position: relative; -webkit-transition: background-color 0.15s linear; }
.class-tree .x-grid-cell-inner a.fav { display: block; width: 15px; height: 15px; position: absolute; right: 7px; top: 0; }
.class-tree .x-grid-cell-inner a.fav.show { background: url(/portfolio/Themes/Default/icons.png) no-repeat -4px -326px; }
.class-tree .x-grid-row-over .x-grid-cell-inner { -webkit-transition: background-color 0.15s linear; }
.class-tree .x-grid-row-over .x-grid-cell-inner a.fav { background: url(/portfolio/Themes/Default/icons.png) no-repeat -4px -353px; }
.class-tree .x-grid-row-over .x-grid-cell-inner a.fav:hover { background: url(/portfolio/Themes/Default/icons.png) no-repeat -4px -326px; }

/*	CSS styles for the drag and select box in designer.uiDesigner
 *
 */

.ip2-designer-markselector {
	position: absolute;
	z-index: 9999;
	border: 1px solid #369;
	background-color: #def;
	opacity: 0.3;
	filter: alpha(opacity=30);
}

/*	CSS styles for selected components in the designer.uiDesigner
 *
 *
 */

.ip2-designer-selected .x-resizable-handle-north, .ip2-designer-selected .x-resizable-handle-south,
.ip2-designer-selected .x-resizable-handle-east, .ip2-designer-selected .x-resizable-handle-west,
.ip2-designer-selected .x-resizable-handle-northeast, .ip2-designer-selected .x-resizable-handle-northwest,
.ip2-designer-selected .x-resizable-handle-southeast, .ip2-designer-selected .x-resizable-handle-southwest {
	background-image:	url(/portfolio/Themes/Default/resize-handle.png);
	background-color: transparent;
	background-position: center center;
	background-repeat: no-repeat;
	position: absolute;
}

.ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-north, .ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-south,
.ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-east, .ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-west,
.ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-northeast, .ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-northwest,
.ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-southeast, .ip2-designer-selected.ip2-designer-selected-master .x-resizable-handle-southwest {
	background-image:	url(/portfolio/Themes/Default/resize-handle-master.png);
}

.ip2-designer-selected .x-resizable-handle-north, .ip2-designer-selected .x-resizable-handle-northeast,
.ip2-designer-selected .x-resizable-handle-northwest {
	height: 11px;
	top: -6px;
}

.ip2-designer-selected .x-resizable-handle-south, .ip2-designer-selected .x-resizable-handle-southeast,
.ip2-designer-selected .x-resizable-handle-southwest {
	height: 11px;
	bottom: -6px;
}

.ip2-designer-selected .x-resizable-handle-west {
	width: 11px;
	left: -6px;
}

.ip2-designer-selected .x-resizable-handle-east {
	width: 11px;
	right: -6px;
}

.ip2-designer-selected .x-resizable-handle-northwest, .ip2-designer-selected .x-resizable-handle-northeast,
.ip2-designer-selected .x-resizable-handle-southwest, .ip2-designer-selected .x-resizable-handle-southeast {
	width: 11px;
}

.ip2-designer-selected .x-resizable-handle-northwest, .ip2-designer-selected .x-resizable-handle-southwest {
	left: -6px;
}

.ip2-designer-selected .x-resizable-handle-northeast, .ip2-designer-selected .x-resizable-handle-southeast {
	right: -6px;
}

/* Styles used by ip2.multiQuestionDialog */
.ip2-question-title {
	font-weight:bold;
	font-size: 120%;
	padding:10px 0 0 24px;
	background: transparent url("/system/images/icons/16x16/arrow_right_green.png") no-repeat 0 10px;
}
.ip2-question-text {
	font-weight:normal;
	white-space:normal;
	padding:5px 0 0 24px;
}
.ip2-question-explanation {
	font-weight:bold;
	font-size: 120%;
	padding:5px 0 5px 5px;
}
.ip2-question-actionText {
	font-weight:normal;
	white-space:normal;
	padding:0 0 5px 5px;
}

/* Styles used by Ext.ux.form.ItemSelector */
.x-form-multiselect-body .x-boundlist .x-mask {
		background: none;
}

.x-form-itemselector-body .x-form-item {
		margin: 0;
}

.x-form-itemselector-top {
		background-image: url(/Admin/Img/ItemSelector/top2.gif);
}
.x-form-itemselector-up {
		background-image: url(/Admin/Img/ItemSelector/up2.gif);
}
.x-form-itemselector-add {
		background-image: url(/Admin/Img/ItemSelector/right2.gif);
}
.x-form-itemselector-remove {
		background-image: url(/Admin/Img/ItemSelector/left2.gif);
}
.x-form-itemselector-down {
		background-image: url(/Admin/Img/ItemSelector/down2.gif);
}
.x-form-itemselector-bottom {
		background-image: url(/Admin/Img/ItemSelector/bottom2.gif);
}
.x-form-accept-trigger {
	background-image: url(syste/shared/ext4/resources/themes/images/grey/form/trigger-accept.png);
}


/*	CSS styles for portal and portlets */

#app-header {
		color: #596F8F;
		font-size: 22px;
		font-weight: 200;
		padding: 8px 15px;
		text-shadow: 0 1px 0 #fff;
}
#app-msg {
		background: #D1DDEF;
		border: 1px solid #ACC3E4;
		padding: 3px 15px;
		font-weight: bold;
		font-size: 13px;
		position: absolute;
		right: 0;
		top: 0;
}
.x-panel-ghost {
		z-index: 1;
}
.x-border-layout-ct {
		background: #DFE8F6;
}
.x-portal-body {
		padding: 0 0 0 8px;
}
.x-portal .x-portal-column {
		/* columns must have vertical padding to avoid losing dimensions when empty */
		padding: 8px 8px 0 0;
}
.x-portal .x-panel-dd-spacer {
		border: 2px dashed #99bbe8;
		background: #f6f6f6;
		border-radius: 4px;
		-moz-border-radius: 4px;
		margin-bottom: 10px;
}
.x-portlet {
		margin-bottom:10px;
		padding: 1px;
}
.x-portlet .x-panel-body {
		background: #fff;
}
.portlet-content {
		padding: 10px;
		font-size: 11px;
}

#app-options .portlet-content {
		padding: 5px;
		font-size: 12px;
}


/* client style */
body {
	-webkit-background-size: cover;
	-moz-background-size: cover;
	-o-background-size: cover;
	background-size: cover;
}

.x-ie body {
	background-position: center center;
}

.login {
	font:normal 13px arial;
}

.login a {
	text-decoration:none;
}

.login .box {
	background-image:url(/portfolio/Themes/Default/login-loginbox.png);
	width: 527px;
	height:327px;
	position: absolute;
	margin: auto;
	left: 0px;
	top: 0px;
	display:none;
}

.login .title {
	color:#ffffff;
	font-size:22px;
}
.login .title .companyname {
	position:absolute;
	top:60px;
	right:132px;
	float:right;
}
.login .title .action {
	position:absolute;
	top:60px;
	left:421px;
	float:left;
}

.login .form {
	position:absolute;
	top:260px;
	left:28px;
}

.login .inputfield, .button {
	display:inline-block;
	float:left;
	padding-left:7px;
}
.login .inputfield input {
	background: url(/portfolio/Themes/default/login-inputfield.png) no-repeat;
	width:183px;
	height:29px;
	color:#333333;
}
.login .inputfield input.error {
	background: url(/portfolio/Themes/default/login-inputfield.png) no-repeat 0 -29px;
}
.login .button {
	background: url(/portfolio/Themes/default/login-btn.png) top;
	width: 66px !important;
	height:29px;
	border:none;
	margin-left:8px;
	cursor:pointer;
}
.login .button:hover {
	background-position: 0 -29px;
}
.login .button .text {
	display:block;
	font-weight:bold;
	color:#333333;
	padding: 6px 0 0 8px;
	text-shadow: -1px -1px 0px rgba(255, 255, 255, 0.75);
	opacity:0.50;
}

.login .textfield {
	border: none;
	-khtml-border-radius: 5px;
	-moz-border-radius: 5px;
	-webkit-border-radius: 5px;
	border-radius: 5px;
	background:none;
	padding-left:5px;
	height: 27px !important;
}
.x-ie .login .textfield {
	padding-top:5px;
}

.login .message {
	position:relative;
	top:330px;
	left:13px;
	color:#cc0000;
}
.login .message  a {
	color:#cc0000;
}

.login .copyright {
	bottom:22px;
	position: absolute;
	width:100%;
	text-align:center;
}
.login .copyright .logo {
	background-repeat:no-repeat;
	height:30px;
	display: block;
	margin-left: auto;
	margin-right: auto;
	padding-bottom:5px;
}
.login .copyright .text {
	height:12px;
	color:#798593;
}

.header {
	font:normal 13px arial;
	width:100%;
	height:51px;
	border:0px;
}
.header a {
	text-decoration:none;
}
.header .logo {
	position:absolute;
	left:15px;
	top:11px;
	height:30px;
	width:30px;
	background:transparent url(/portfolio/Themes/Default/sml-logo-topbar.png) no-repeat;
}
.header .title {
	position:absolute;
	left:53px;
	top:18px;
	color:white;
	font-weight:bold;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.55;
}
.header .caption {
	position: absolute;
	top:17px;
	width:100%;
	text-align:center;
	color:white;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.90;
}
.header .user {
	position:absolute;
	right:98px;
	top:16px;
	color:white;
	font-weight:bold;
	text-shadow: 1px 1px 0px rgba(0, 0, 0, 0.65);
	opacity:0.55;
}
.header-user-image {
	background-position: center center;
	background-size: cover;
	cursor: pointer;
},

.header .menu {
	position:absolute;
	right:10px;
	top:11px;
	color:white;
	text-align:left;
}
.header .menu .button {
	background: url(/portfolio/Themes/default/topbar-btn.png) no-repeat;
	width:66px !important;
	height:29px !important;
	border:none;
	margin-left:8px;
	cursor:pointer;
}
.header .menu .button:hover {
	background: url(/portfolio/Themes/default/topbar-btn.png) no-repeat 0 -29px;
}
.header .menu .button .text {
	display:block;
	font-weight:bold;
	color:#ffffff;
	padding: 6px 0 0 1px;
	text-shadow: -1px -1px 0px rgba(150, 150, 150, 0.9);
	opacity:0.50;
}


#splash-content {
	width: 100%;
	height:100%;
}
.splash {
	width: 100%;
	height:100%;
	font:normal 12px arial;
	color:#333333;
	text-align:center;
}
.splash .box {
	background-color:#ffffff;
	width:500px;
	margin:auto;
	position:relative;
	top:20%;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	-webkit-box-shadow: 2px 2px 5px rgba(50, 50, 50, 0.75);
	-moz-box-shadow:    2px 2px 5px rgba(50, 50, 50, 0.75);
	box-shadow:         2px 2px 5px rgba(50, 50, 50, 0.75);
}
.splash .box .title {
	font-size:18px;
	padding-top:26px;
}
.splash .box .text {
	letter-spacing:1px;
	line-height:22px;
	margin:20px 33px 12px 33px;
}
.splash .box .logo {
	/*background-image:url(/portfolio/Themes/Default/login-logo.png);*/
	background-repeat:no-repeat;
	/*height:31px;
	width:23px;*/
	display: block;
	margin-left: auto;
	margin-right: auto;
	padding-bottom:52px;
}
.help, .help * {
	font-size:12px;
}
.help {
	font-family:"verdana";
	line-height:120%;
	margin: 5 5 0 5;
	height:100%;
	overflow:auto;
}

.ux-layout-center-item {
		margin:0 auto;
		text-align:left;
}
.ux-layout-center .x-panel-body,
body.ux-layout-center {
		text-align:center;
}

.x-grid-cell-inner {
	line-height: 16px;
}

.x-form-cb-label-after {
	color: grey;
}

.x-grid-cell-inner .ip2-icon {
	height: 16px;
	width: 16px;
}

.ip2-overflow-x-hidden {
	overflow-x: hidden;
}

.ip2-fieldContainer-header {
	top: 19px !important;
}

.mousepointer {
	cursor: pointer;
}

.x-ux-treeIcon-bg {
	background-position: 0 center !important;
}

.x-tree-no-icon {
	display: none !important;
}

.webshop-family {
	background-image:url("/system/images/icons/16x16/package.png") !important;
}
.webshop-related {
	background-image:url("/system/images/icons/16x16/paperclip.png") !important;
}
.webshop-parent-node {
	background-image:url("/system/shared/collection/icons/16x16/plain/folder.png") !important;
	background-position: center center !important;
}

.webshop-child-node {
	background-image:url("/system/shared/collection/icons/16x16/plain/window_explorer.png") !important;
	background-position: center center !important;
}

.x-form-arrow-trigger {
	background-image: url("/system/shared/collection/icons/16x16/plain/navigate_down.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-form-trigger-search {
	background-image: url("/system/shared/collection/icons/16x16/plain/magnifying_glass.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-form-trigger-upload {
	background-image: url("/system/shared/collection/icons/16x16/plain/cloud_upload.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-form-trigger-clear {
	background-image: url("/system/shared/collection/icons/16x16/plain/delete.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-form-trigger-add {
	background-image: url("/system/shared/collection/icons/16x16/plain/plus.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-form-trigger-download {
	background-image: url("/system/shared/collection/icons/16x16/plain/document_gear.png") !important;
	background-position: 0 center !important;
	background-repeat: no-repeat !important;
}

.x-ux-preview-trigger {
	background-image: url("/system/shared/collection/icons/16x16/plain/eye.png") !important;
	background-position: center center !important;
	background-repeat: no-repeat !important;
}

/**
xtypes.css
**/

/**********************************
	FIELD READONLY OVERRIDE
**********************************/
.x-form-field[readonly]{
	background-color: #f2f2f2;
	color: #626262;
}

/************************************
	Ext.ux.plugins.UploadDD
************************************/
.x-ux-plugin-uploadDD  div[_type="dropbox"] {
	height: 100%;
	border: 4px dashed #CCCCCC;
	border-radius: 5px;
	font-size: 17px;
	-webkit-transition:border-color .1s ease-out;
}
.x-ux-plugin-uploadDD.x-ux-plugin-uploadDD-over div[_type="dropbox"] {
	border-color: #00c294;
	-webkit-transition: border-color .1s ease-in;
}

.x-ux-plugin-uploadDD.x-ux-plugin-uploadDD-error div[_type="dropbox"] {
	border-color: #ff0000;
}


.x-ux-imagelist tr td {
	vertical-align: middle;
}


/********************************************************
	No component, can be used by all that uses a triggers
*********************************************************/
.x-ux-search-icon {
	background-image: url("styles/icon/flatui/search-icon.png") !important;
}

.x-ux-icon-bg {
	background-repeat: no-repeat !important;
	padding-left: 25px !important;
	background-position: 5px 50% !important;
}

/************************************
	Ext.ux.hsvwheel.Field
************************************/
.x-ux-hsvwheel-field input {
	padding-left: 22px;
}

.x-ux-hsvwheel-field-preview {
	position: relative;
	height: 11px;
	width: 11px;
	top: 50%;
	left: 6px;
	border: 1px solid #000000;
	background-color: transparent;
}
.x-ux-hsvwheel-field-preview.error {
	border-color: red;
	background-color: #FFFFFF !important;
}


/************************************
	Ext.ux.gallery.view.MediaGrid
************************************/
.x-ux-imagegrid tr td {
	vertical-align: middle;
}
.x-grid.x-ux-imagegrid .x-grid-body {
	border-left-width: 0;
	border-right-width: 0;
	border-bottom-width: 0;
}

.x-grid.x-ux-imagegrid .x-docked {
	border-left-width: 0;
	border-right-width: 0;
}


/************************************
	Ext.ux.toolbar.PagingToolbar
************************************/
.x-ux-pagingtoolbar input {
	font-size: 10px !important;
	line-height: 10px !important;
}

/************************************
	Ext.ux.ip2.form.field.UploadPicker
************************************/
.x-ux-field-uploadpicker {}

/************************************
	Ext.ux.ip2.folderorganizer
************************************/
table.x-ux-folderorganizer-noresult {
	height: 100%;
	width: 100%;
	text-align: center;
}
table.x-ux-folderorganizer-noresult tr {
	height: 100%;
}
table.x-ux-folderorganizer-noresult tr td {
	vertical-align: middle;
	height: 100%;
}

/************************************
	Filter images
************************************/
.x-ux-filter-check{
	background-image:  url("styles/icon/flatui/check.png") !important;
	background-repeat: no-repeat;
}

/************************************
	xpdPresenter uiviewer
************************************/
.x-window-xpdpresenter {
	font-size: 13px;
}
.x-window-xpdpresenter-label {
	font-weight: 600;
}
.x-window-xpdpresenter-value {

}

/************************************
	LOADMASK SPECIAL CLASSES
************************************/
.x-mask-msg.x-loadmask-small {
	border: none;
	background-color: transparent;
	padding: 0;
}

.x-mask-msg.x-loadmask-small > div {
	border: none;
	background-color: transparent;
}

/************************************
	OVERRIDE FOR BUTTONS IN TABS
************************************/
.x-tab button {
	overflow-x: hidden !important;
}

.x-ux-form-xpdlanguagepanel-form {
	width: 100% !important;
}
.x-ux-form-xpdlanguagepanel-form > div {
	width: 100% !important;
}
.x-ux-big-font {
	font-size: 16px;
	color: black;
	height: 50px;
}

.x-ux-stock td:not(:first-child) {
	color: #199ACA !important;
	background-color: #EDF7FB !important;
}

.x-ux-repair td:not(:first-child) {
	color: #D67302 !important;
	background-color: #FCF4EB !important;
}

.x-ux-check td:not(:first-child) {
	color: #E805FF !important;
	background-color: #FEEBFF !important;
}

.x-ux-invalid td:not(:first-child) {
	color: #D03E35 !important;
	background-color: #FCF0EF !important;
}

.x-ux-tree-container td:not(:first-child) {
	background-color: #cbd0d5 !important;
	border-top: 1px solid #aeb6be;
}

.x-ux-tree-container-last td:not(:first-child) {
	background-color: #cbd0d5 !important;
	border-bottom: 1px solid #aeb6be;
}

.x-ux-tree-container-child td:not(:first-child) {
	background-color: #e0e3e6 !important;
}

.x-ux-tree-container-child-first td:not(:first-child) {
	background-color: #e0e3e6 !important;
	border-top: 1px solid #aeb6be;
}

.x-ux-tree-container-child-last td:not(:first-child) {
	background-color: #e0e3e6 !important;
	border-bottom: 1px solid #aeb6be;
}
.x-ux-icon-selected-item {
	outline: 1px dashed black !important;
	background-color: #e0e3e6 !important;
}

.x-form-multiselect-body .x-boundlist .x-mask {
    background: none;
}

.x-form-itemselector-body .x-form-item {
    margin: 0;
}

.x-form-itemselector-top {
    background-image: url("styles/icon/flatui/itemselector/top.png");
}
.x-form-itemselector-up {
    background-image: url("styles/icon/flatui/itemselector/up.png");
}
.x-form-itemselector-add {
    background-image: url("styles/icon/flatui/itemselector/right.png");
}
.x-form-itemselector-remove {
    background-image: url("styles/icon/flatui/itemselector/left.png");
}
.x-form-itemselector-down {
    background-image: url("styles/icon/flatui/itemselector/down.png");
}
.x-form-itemselector-bottom {
    background-image: url("styles/icon/flatui/itemselector/bottom.png");
}

.x-grid-row-summary .x-grid-cell-inner {
	font-weight: bold;
}
