

	/*!
	 * Automated Icon Libray v0.1.0 (http://www.ailcss.com)
	 * Copyright 2014 Splynxicon Inc.
	 */
	 	div[class*="icon-flag_"]:before {
			 content: "" !important;
		 }


		.AIL-flags .AIL-icon,
		.AIL-flags .AIL-icon-stack,
		.AIL-flags .AIL-icon-stack > * {
			background-size: 16px;
			background-position: center center !important;
			background-repeat: no-repeat;
		}

			.AIL-flags.AIL-icon-8 .AIL-icon,
			.AIL-flags.AIL-icon-8 .AIL-icon-stack,
			.AIL-flags.AIL-icon-8 .AIL-icon-stack > * {
				background-size: 8px;
			}

			.AIL-flags .AIL-icon-8 .AIL-icon,
			.AIL-flags .AIL-icon-8 .AIL-icon-stack,
			.AIL-flags .AIL-icon-8 .AIL-icon-stack > * {
				background-size: 8px;
			}

			.AIL-flags .AIL-icon.AIL-icon-8,
			.AIL-flags .AIL-icon-stack.AIL-icon-8,
			.AIL-flags .AIL-icon-stack.AIL-icon-8 > * {
				background-size: 8px;
			}

			.AIL-flags.AIL-icon-10 .AIL-icon,
			.AIL-flags.AIL-icon-10 .AIL-icon-stack,
			.AIL-flags.AIL-icon-10 .AIL-icon-stack > * {
				background-size: 10px;
			}

			.AIL-flags .AIL-icon-10 .AIL-icon,
			.AIL-flags .AIL-icon-10 .AIL-icon-stack,
			.AIL-flags .AIL-icon-10 .AIL-icon-stack > * {
				background-size: 10px;
			}

			.AIL-flags .AIL-icon.AIL-icon-10,
			.AIL-flags .AIL-icon-stack.AIL-icon-10,
			.AIL-flags .AIL-icon-stack.AIL-icon-10 > * {
				background-size: 10px;
			}

			.AIL-flags.AIL-icon-12 .AIL-icon,
			.AIL-flags.AIL-icon-12 .AIL-icon-stack,
			.AIL-flags.AIL-icon-12 .AIL-icon-stack > * {
				background-size: 12px;
			}

			.AIL-flags .AIL-icon-12 .AIL-icon,
			.AIL-flags .AIL-icon-12 .AIL-icon-stack,
			.AIL-flags .AIL-icon-12 .AIL-icon-stack > * {
				background-size: 12px;
			}

			.AIL-flags .AIL-icon.AIL-icon-12,
			.AIL-flags .AIL-icon-stack.AIL-icon-12,
			.AIL-flags .AIL-icon-stack.AIL-icon-12 > * {
				background-size: 12px;
			}

			.AIL-flags.AIL-icon-14 .AIL-icon,
			.AIL-flags.AIL-icon-14 .AIL-icon-stack,
			.AIL-flags.AIL-icon-14 .AIL-icon-stack > * {
				background-size: 14px;
			}

			.AIL-flags .AIL-icon-14 .AIL-icon,
			.AIL-flags .AIL-icon-14 .AIL-icon-stack,
			.AIL-flags .AIL-icon-14 .AIL-icon-stack > * {
				background-size: 14px;
			}

			.AIL-flags .AIL-icon.AIL-icon-14,
			.AIL-flags .AIL-icon-stack.AIL-icon-14,
			.AIL-flags .AIL-icon-stack.AIL-icon-14 > * {
				background-size: 14px;
			}

			.AIL-flags.AIL-icon-16 .AIL-icon,
			.AIL-flags.AIL-icon-16 .AIL-icon-stack,
			.AIL-flags.AIL-icon-16 .AIL-icon-stack > * {
				background-size: 16px;
			}

			.AIL-flags .AIL-icon-16 .AIL-icon,
			.AIL-flags .AIL-icon-16 .AIL-icon-stack,
			.AIL-flags .AIL-icon-16 .AIL-icon-stack > * {
				background-size: 16px;
			}

			.AIL-flags .AIL-icon.AIL-icon-16,
			.AIL-flags .AIL-icon-stack.AIL-icon-16,
			.AIL-flags .AIL-icon-stack.AIL-icon-16 > * {
				background-size: 16px;
			}

			.AIL-flags.AIL-icon-18 .AIL-icon,
			.AIL-flags.AIL-icon-18 .AIL-icon-stack,
			.AIL-flags.AIL-icon-18 .AIL-icon-stack > * {
				background-size: 18px;
			}

			.AIL-flags .AIL-icon-18 .AIL-icon,
			.AIL-flags .AIL-icon-18 .AIL-icon-stack,
			.AIL-flags .AIL-icon-18 .AIL-icon-stack > * {
				background-size: 18px;
			}

			.AIL-flags .AIL-icon.AIL-icon-18,
			.AIL-flags .AIL-icon-stack.AIL-icon-18,
			.AIL-flags .AIL-icon-stack.AIL-icon-18 > * {
				background-size: 18px;
			}

			.AIL-flags.AIL-icon-20 .AIL-icon,
			.AIL-flags.AIL-icon-20 .AIL-icon-stack,
			.AIL-flags.AIL-icon-20 .AIL-icon-stack > * {
				background-size: 20px;
			}

			.AIL-flags .AIL-icon-20 .AIL-icon,
			.AIL-flags .AIL-icon-20 .AIL-icon-stack,
			.AIL-flags .AIL-icon-20 .AIL-icon-stack > * {
				background-size: 20px;
			}

			.AIL-flags .AIL-icon.AIL-icon-20,
			.AIL-flags .AIL-icon-stack.AIL-icon-20,
			.AIL-flags .AIL-icon-stack.AIL-icon-20 > * {
				background-size: 20px;
			}

			.AIL-flags.AIL-icon-22 .AIL-icon,
			.AIL-flags.AIL-icon-22 .AIL-icon-stack,
			.AIL-flags.AIL-icon-22 .AIL-icon-stack > * {
				background-size: 22px;
			}

			.AIL-flags .AIL-icon-22 .AIL-icon,
			.AIL-flags .AIL-icon-22 .AIL-icon-stack,
			.AIL-flags .AIL-icon-22 .AIL-icon-stack > * {
				background-size: 22px;
			}

			.AIL-flags .AIL-icon.AIL-icon-22,
			.AIL-flags .AIL-icon-stack.AIL-icon-22,
			.AIL-flags .AIL-icon-stack.AIL-icon-22 > * {
				background-size: 22px;
			}

			.AIL-flags.AIL-icon-24 .AIL-icon,
			.AIL-flags.AIL-icon-24 .AIL-icon-stack,
			.AIL-flags.AIL-icon-24 .AIL-icon-stack > * {
				background-size: 24px;
			}

			.AIL-flags .AIL-icon-24 .AIL-icon,
			.AIL-flags .AIL-icon-24 .AIL-icon-stack,
			.AIL-flags .AIL-icon-24 .AIL-icon-stack > * {
				background-size: 24px;
			}

			.AIL-flags .AIL-icon.AIL-icon-24,
			.AIL-flags .AIL-icon-stack.AIL-icon-24,
			.AIL-flags .AIL-icon-stack.AIL-icon-24 > * {
				background-size: 24px;
			}

			.AIL-flags.AIL-icon-26 .AIL-icon,
			.AIL-flags.AIL-icon-26 .AIL-icon-stack,
			.AIL-flags.AIL-icon-26 .AIL-icon-stack > * {
				background-size: 26px;
			}

			.AIL-flags .AIL-icon-26 .AIL-icon,
			.AIL-flags .AIL-icon-26 .AIL-icon-stack,
			.AIL-flags .AIL-icon-26 .AIL-icon-stack > * {
				background-size: 26px;
			}

			.AIL-flags .AIL-icon.AIL-icon-26,
			.AIL-flags .AIL-icon-stack.AIL-icon-26,
			.AIL-flags .AIL-icon-stack.AIL-icon-26 > * {
				background-size: 26px;
			}

			.AIL-flags.AIL-icon-28 .AIL-icon,
			.AIL-flags.AIL-icon-28 .AIL-icon-stack,
			.AIL-flags.AIL-icon-28 .AIL-icon-stack > * {
				background-size: 28px;
			}

			.AIL-flags .AIL-icon-28 .AIL-icon,
			.AIL-flags .AIL-icon-28 .AIL-icon-stack,
			.AIL-flags .AIL-icon-28 .AIL-icon-stack > * {
				background-size: 28px;
			}

			.AIL-flags .AIL-icon.AIL-icon-28,
			.AIL-flags .AIL-icon-stack.AIL-icon-28,
			.AIL-flags .AIL-icon-stack.AIL-icon-28 > * {
				background-size: 28px;
			}

			.AIL-flags.AIL-icon-30 .AIL-icon,
			.AIL-flags.AIL-icon-30 .AIL-icon-stack,
			.AIL-flags.AIL-icon-30 .AIL-icon-stack > * {
				background-size: 30px;
			}

			.AIL-flags .AIL-icon-30 .AIL-icon,
			.AIL-flags .AIL-icon-30 .AIL-icon-stack,
			.AIL-flags .AIL-icon-30 .AIL-icon-stack > * {
				background-size: 30px;
			}

			.AIL-flags .AIL-icon.AIL-icon-30,
			.AIL-flags .AIL-icon-stack.AIL-icon-30,
			.AIL-flags .AIL-icon-stack.AIL-icon-30 > * {
				background-size: 30px;
			}

			.AIL-flags.AIL-icon-32 .AIL-icon,
			.AIL-flags.AIL-icon-32 .AIL-icon-stack,
			.AIL-flags.AIL-icon-32 .AIL-icon-stack > * {
				background-size: 32px;
			}

			.AIL-flags .AIL-icon-32 .AIL-icon,
			.AIL-flags .AIL-icon-32 .AIL-icon-stack,
			.AIL-flags .AIL-icon-32 .AIL-icon-stack > * {
				background-size: 32px;
			}

			.AIL-flags .AIL-icon.AIL-icon-32,
			.AIL-flags .AIL-icon-stack.AIL-icon-32,
			.AIL-flags .AIL-icon-stack.AIL-icon-32 > * {
				background-size: 32px;
			}

			.AIL-flags.AIL-icon-34 .AIL-icon,
			.AIL-flags.AIL-icon-34 .AIL-icon-stack,
			.AIL-flags.AIL-icon-34 .AIL-icon-stack > * {
				background-size: 34px;
			}

			.AIL-flags .AIL-icon-34 .AIL-icon,
			.AIL-flags .AIL-icon-34 .AIL-icon-stack,
			.AIL-flags .AIL-icon-34 .AIL-icon-stack > * {
				background-size: 34px;
			}

			.AIL-flags .AIL-icon.AIL-icon-34,
			.AIL-flags .AIL-icon-stack.AIL-icon-34,
			.AIL-flags .AIL-icon-stack.AIL-icon-34 > * {
				background-size: 34px;
			}

			.AIL-flags.AIL-icon-36 .AIL-icon,
			.AIL-flags.AIL-icon-36 .AIL-icon-stack,
			.AIL-flags.AIL-icon-36 .AIL-icon-stack > * {
				background-size: 36px;
			}

			.AIL-flags .AIL-icon-36 .AIL-icon,
			.AIL-flags .AIL-icon-36 .AIL-icon-stack,
			.AIL-flags .AIL-icon-36 .AIL-icon-stack > * {
				background-size: 36px;
			}

			.AIL-flags .AIL-icon.AIL-icon-36,
			.AIL-flags .AIL-icon-stack.AIL-icon-36,
			.AIL-flags .AIL-icon-stack.AIL-icon-36 > * {
				background-size: 36px;
			}

			.AIL-flags.AIL-icon-38 .AIL-icon,
			.AIL-flags.AIL-icon-38 .AIL-icon-stack,
			.AIL-flags.AIL-icon-38 .AIL-icon-stack > * {
				background-size: 38px;
			}

			.AIL-flags .AIL-icon-38 .AIL-icon,
			.AIL-flags .AIL-icon-38 .AIL-icon-stack,
			.AIL-flags .AIL-icon-38 .AIL-icon-stack > * {
				background-size: 38px;
			}

			.AIL-flags .AIL-icon.AIL-icon-38,
			.AIL-flags .AIL-icon-stack.AIL-icon-38,
			.AIL-flags .AIL-icon-stack.AIL-icon-38 > * {
				background-size: 38px;
			}

			.AIL-flags.AIL-icon-40 .AIL-icon,
			.AIL-flags.AIL-icon-40 .AIL-icon-stack,
			.AIL-flags.AIL-icon-40 .AIL-icon-stack > * {
				background-size: 40px;
			}

			.AIL-flags .AIL-icon-40 .AIL-icon,
			.AIL-flags .AIL-icon-40 .AIL-icon-stack,
			.AIL-flags .AIL-icon-40 .AIL-icon-stack > * {
				background-size: 40px;
			}

			.AIL-flags .AIL-icon.AIL-icon-40,
			.AIL-flags .AIL-icon-stack.AIL-icon-40,
			.AIL-flags .AIL-icon-stack.AIL-icon-40 > * {
				background-size: 40px;
			}

			.AIL-flags.AIL-icon-42 .AIL-icon,
			.AIL-flags.AIL-icon-42 .AIL-icon-stack,
			.AIL-flags.AIL-icon-42 .AIL-icon-stack > * {
				background-size: 42px;
			}

			.AIL-flags .AIL-icon-42 .AIL-icon,
			.AIL-flags .AIL-icon-42 .AIL-icon-stack,
			.AIL-flags .AIL-icon-42 .AIL-icon-stack > * {
				background-size: 42px;
			}

			.AIL-flags .AIL-icon.AIL-icon-42,
			.AIL-flags .AIL-icon-stack.AIL-icon-42,
			.AIL-flags .AIL-icon-stack.AIL-icon-42 > * {
				background-size: 42px;
			}

			.AIL-flags.AIL-icon-44 .AIL-icon,
			.AIL-flags.AIL-icon-44 .AIL-icon-stack,
			.AIL-flags.AIL-icon-44 .AIL-icon-stack > * {
				background-size: 44px;
			}

			.AIL-flags .AIL-icon-44 .AIL-icon,
			.AIL-flags .AIL-icon-44 .AIL-icon-stack,
			.AIL-flags .AIL-icon-44 .AIL-icon-stack > * {
				background-size: 44px;
			}

			.AIL-flags .AIL-icon.AIL-icon-44,
			.AIL-flags .AIL-icon-stack.AIL-icon-44,
			.AIL-flags .AIL-icon-stack.AIL-icon-44 > * {
				background-size: 44px;
			}

			.AIL-flags.AIL-icon-46 .AIL-icon,
			.AIL-flags.AIL-icon-46 .AIL-icon-stack,
			.AIL-flags.AIL-icon-46 .AIL-icon-stack > * {
				background-size: 46px;
			}

			.AIL-flags .AIL-icon-46 .AIL-icon,
			.AIL-flags .AIL-icon-46 .AIL-icon-stack,
			.AIL-flags .AIL-icon-46 .AIL-icon-stack > * {
				background-size: 46px;
			}

			.AIL-flags .AIL-icon.AIL-icon-46,
			.AIL-flags .AIL-icon-stack.AIL-icon-46,
			.AIL-flags .AIL-icon-stack.AIL-icon-46 > * {
				background-size: 46px;
			}

			.AIL-flags.AIL-icon-48 .AIL-icon,
			.AIL-flags.AIL-icon-48 .AIL-icon-stack,
			.AIL-flags.AIL-icon-48 .AIL-icon-stack > * {
				background-size: 48px;
			}

			.AIL-flags .AIL-icon-48 .AIL-icon,
			.AIL-flags .AIL-icon-48 .AIL-icon-stack,
			.AIL-flags .AIL-icon-48 .AIL-icon-stack > * {
				background-size: 48px;
			}

			.AIL-flags .AIL-icon.AIL-icon-48,
			.AIL-flags .AIL-icon-stack.AIL-icon-48,
			.AIL-flags .AIL-icon-stack.AIL-icon-48 > * {
				background-size: 48px;
			}

			.AIL-flags.AIL-icon-50 .AIL-icon,
			.AIL-flags.AIL-icon-50 .AIL-icon-stack,
			.AIL-flags.AIL-icon-50 .AIL-icon-stack > * {
				background-size: 50px;
			}

			.AIL-flags .AIL-icon-50 .AIL-icon,
			.AIL-flags .AIL-icon-50 .AIL-icon-stack,
			.AIL-flags .AIL-icon-50 .AIL-icon-stack > * {
				background-size: 50px;
			}

			.AIL-flags .AIL-icon.AIL-icon-50,
			.AIL-flags .AIL-icon-stack.AIL-icon-50,
			.AIL-flags .AIL-icon-stack.AIL-icon-50 > * {
				background-size: 50px;
			}

			.AIL-flags.AIL-icon-52 .AIL-icon,
			.AIL-flags.AIL-icon-52 .AIL-icon-stack,
			.AIL-flags.AIL-icon-52 .AIL-icon-stack > * {
				background-size: 52px;
			}

			.AIL-flags .AIL-icon-52 .AIL-icon,
			.AIL-flags .AIL-icon-52 .AIL-icon-stack,
			.AIL-flags .AIL-icon-52 .AIL-icon-stack > * {
				background-size: 52px;
			}

			.AIL-flags .AIL-icon.AIL-icon-52,
			.AIL-flags .AIL-icon-stack.AIL-icon-52,
			.AIL-flags .AIL-icon-stack.AIL-icon-52 > * {
				background-size: 52px;
			}

			.AIL-flags.AIL-icon-54 .AIL-icon,
			.AIL-flags.AIL-icon-54 .AIL-icon-stack,
			.AIL-flags.AIL-icon-54 .AIL-icon-stack > * {
				background-size: 54px;
			}

			.AIL-flags .AIL-icon-54 .AIL-icon,
			.AIL-flags .AIL-icon-54 .AIL-icon-stack,
			.AIL-flags .AIL-icon-54 .AIL-icon-stack > * {
				background-size: 54px;
			}

			.AIL-flags .AIL-icon.AIL-icon-54,
			.AIL-flags .AIL-icon-stack.AIL-icon-54,
			.AIL-flags .AIL-icon-stack.AIL-icon-54 > * {
				background-size: 54px;
			}

			.AIL-flags.AIL-icon-56 .AIL-icon,
			.AIL-flags.AIL-icon-56 .AIL-icon-stack,
			.AIL-flags.AIL-icon-56 .AIL-icon-stack > * {
				background-size: 56px;
			}

			.AIL-flags .AIL-icon-56 .AIL-icon,
			.AIL-flags .AIL-icon-56 .AIL-icon-stack,
			.AIL-flags .AIL-icon-56 .AIL-icon-stack > * {
				background-size: 56px;
			}

			.AIL-flags .AIL-icon.AIL-icon-56,
			.AIL-flags .AIL-icon-stack.AIL-icon-56,
			.AIL-flags .AIL-icon-stack.AIL-icon-56 > * {
				background-size: 56px;
			}

			.AIL-flags.AIL-icon-58 .AIL-icon,
			.AIL-flags.AIL-icon-58 .AIL-icon-stack,
			.AIL-flags.AIL-icon-58 .AIL-icon-stack > * {
				background-size: 58px;
			}

			.AIL-flags .AIL-icon-58 .AIL-icon,
			.AIL-flags .AIL-icon-58 .AIL-icon-stack,
			.AIL-flags .AIL-icon-58 .AIL-icon-stack > * {
				background-size: 58px;
			}

			.AIL-flags .AIL-icon.AIL-icon-58,
			.AIL-flags .AIL-icon-stack.AIL-icon-58,
			.AIL-flags .AIL-icon-stack.AIL-icon-58 > * {
				background-size: 58px;
			}

			.AIL-flags.AIL-icon-60 .AIL-icon,
			.AIL-flags.AIL-icon-60 .AIL-icon-stack,
			.AIL-flags.AIL-icon-60 .AIL-icon-stack > * {
				background-size: 60px;
			}

			.AIL-flags .AIL-icon-60 .AIL-icon,
			.AIL-flags .AIL-icon-60 .AIL-icon-stack,
			.AIL-flags .AIL-icon-60 .AIL-icon-stack > * {
				background-size: 60px;
			}

			.AIL-flags .AIL-icon.AIL-icon-60,
			.AIL-flags .AIL-icon-stack.AIL-icon-60,
			.AIL-flags .AIL-icon-stack.AIL-icon-60 > * {
				background-size: 60px;
			}

			.AIL-flags.AIL-icon-62 .AIL-icon,
			.AIL-flags.AIL-icon-62 .AIL-icon-stack,
			.AIL-flags.AIL-icon-62 .AIL-icon-stack > * {
				background-size: 62px;
			}

			.AIL-flags .AIL-icon-62 .AIL-icon,
			.AIL-flags .AIL-icon-62 .AIL-icon-stack,
			.AIL-flags .AIL-icon-62 .AIL-icon-stack > * {
				background-size: 62px;
			}

			.AIL-flags .AIL-icon.AIL-icon-62,
			.AIL-flags .AIL-icon-stack.AIL-icon-62,
			.AIL-flags .AIL-icon-stack.AIL-icon-62 > * {
				background-size: 62px;
			}

			.AIL-flags.AIL-icon-64 .AIL-icon,
			.AIL-flags.AIL-icon-64 .AIL-icon-stack,
			.AIL-flags.AIL-icon-64 .AIL-icon-stack > * {
				background-size: 64px;
			}

			.AIL-flags .AIL-icon-64 .AIL-icon,
			.AIL-flags .AIL-icon-64 .AIL-icon-stack,
			.AIL-flags .AIL-icon-64 .AIL-icon-stack > * {
				background-size: 64px;
			}

			.AIL-flags .AIL-icon.AIL-icon-64,
			.AIL-flags .AIL-icon-stack.AIL-icon-64,
			.AIL-flags .AIL-icon-stack.AIL-icon-64 > * {
				background-size: 64px;
			}

		.AIL-flags .AIL-icon-stack > *:first-child {
			position: absolute;
		}
		.AIL-flags .AIL-icon-stack > *:last-child {
			position: relative;
			background-position: bottom right;
		}
		.AIL-flags .AIL-icon-stack > *:last-child.top {
			background-position-y: top;
		}
		.AIL-flags .AIL-icon-stack > *:last-child.bottom {
			background-position-y: bottom;
		}
		.AIL-flags .AIL-icon-stack > *:last-child.left {
			background-position-x: left;
		}
		.AIL-flags .AIL-icon-stack > *:last-child.right {
			background-position-x: right;
		}

		.AIL-flags .icon-stack > *:last-child {
			background-size: 8px;
		}

			.AIL-flags .AIL-icon-stack.icon-8  > *:last-child {
				background-size: 4px;
			}

			.AIL-flags .AIL-icon-stack.icon-8  > *.icon-x2:last-child {
				background-size: 6px;
			}

			.AIL-flags .AIL-icon-stack.icon-10  > *:last-child {
				background-size: 5px;
			}

			.AIL-flags .AIL-icon-stack.icon-10  > *.icon-x2:last-child {
				background-size: 7.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-12  > *:last-child {
				background-size: 6px;
			}

			.AIL-flags .AIL-icon-stack.icon-12  > *.icon-x2:last-child {
				background-size: 9px;
			}

			.AIL-flags .AIL-icon-stack.icon-14  > *:last-child {
				background-size: 7px;
			}

			.AIL-flags .AIL-icon-stack.icon-14  > *.icon-x2:last-child {
				background-size: 10.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-16  > *:last-child {
				background-size: 8px;
			}

			.AIL-flags .AIL-icon-stack.icon-16  > *.icon-x2:last-child {
				background-size: 12px;
			}

			.AIL-flags .AIL-icon-stack.icon-18  > *:last-child {
				background-size: 9px;
			}

			.AIL-flags .AIL-icon-stack.icon-18  > *.icon-x2:last-child {
				background-size: 13.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-20  > *:last-child {
				background-size: 10px;
			}

			.AIL-flags .AIL-icon-stack.icon-20  > *.icon-x2:last-child {
				background-size: 15px;
			}

			.AIL-flags .AIL-icon-stack.icon-22  > *:last-child {
				background-size: 11px;
			}

			.AIL-flags .AIL-icon-stack.icon-22  > *.icon-x2:last-child {
				background-size: 16.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-24  > *:last-child {
				background-size: 12px;
			}

			.AIL-flags .AIL-icon-stack.icon-24  > *.icon-x2:last-child {
				background-size: 18px;
			}

			.AIL-flags .AIL-icon-stack.icon-26  > *:last-child {
				background-size: 13px;
			}

			.AIL-flags .AIL-icon-stack.icon-26  > *.icon-x2:last-child {
				background-size: 19.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-28  > *:last-child {
				background-size: 14px;
			}

			.AIL-flags .AIL-icon-stack.icon-28  > *.icon-x2:last-child {
				background-size: 21px;
			}

			.AIL-flags .AIL-icon-stack.icon-30  > *:last-child {
				background-size: 15px;
			}

			.AIL-flags .AIL-icon-stack.icon-30  > *.icon-x2:last-child {
				background-size: 22.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-32  > *:last-child {
				background-size: 16px;
			}

			.AIL-flags .AIL-icon-stack.icon-32  > *.icon-x2:last-child {
				background-size: 24px;
			}

			.AIL-flags .AIL-icon-stack.icon-34  > *:last-child {
				background-size: 17px;
			}

			.AIL-flags .AIL-icon-stack.icon-34  > *.icon-x2:last-child {
				background-size: 25.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-36  > *:last-child {
				background-size: 18px;
			}

			.AIL-flags .AIL-icon-stack.icon-36  > *.icon-x2:last-child {
				background-size: 27px;
			}

			.AIL-flags .AIL-icon-stack.icon-38  > *:last-child {
				background-size: 19px;
			}

			.AIL-flags .AIL-icon-stack.icon-38  > *.icon-x2:last-child {
				background-size: 28.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-40  > *:last-child {
				background-size: 20px;
			}

			.AIL-flags .AIL-icon-stack.icon-40  > *.icon-x2:last-child {
				background-size: 30px;
			}

			.AIL-flags .AIL-icon-stack.icon-42  > *:last-child {
				background-size: 21px;
			}

			.AIL-flags .AIL-icon-stack.icon-42  > *.icon-x2:last-child {
				background-size: 31.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-44  > *:last-child {
				background-size: 22px;
			}

			.AIL-flags .AIL-icon-stack.icon-44  > *.icon-x2:last-child {
				background-size: 33px;
			}

			.AIL-flags .AIL-icon-stack.icon-46  > *:last-child {
				background-size: 23px;
			}

			.AIL-flags .AIL-icon-stack.icon-46  > *.icon-x2:last-child {
				background-size: 34.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-48  > *:last-child {
				background-size: 24px;
			}

			.AIL-flags .AIL-icon-stack.icon-48  > *.icon-x2:last-child {
				background-size: 36px;
			}

			.AIL-flags .AIL-icon-stack.icon-50  > *:last-child {
				background-size: 25px;
			}

			.AIL-flags .AIL-icon-stack.icon-50  > *.icon-x2:last-child {
				background-size: 37.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-52  > *:last-child {
				background-size: 26px;
			}

			.AIL-flags .AIL-icon-stack.icon-52  > *.icon-x2:last-child {
				background-size: 39px;
			}

			.AIL-flags .AIL-icon-stack.icon-54  > *:last-child {
				background-size: 27px;
			}

			.AIL-flags .AIL-icon-stack.icon-54  > *.icon-x2:last-child {
				background-size: 40.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-56  > *:last-child {
				background-size: 28px;
			}

			.AIL-flags .AIL-icon-stack.icon-56  > *.icon-x2:last-child {
				background-size: 42px;
			}

			.AIL-flags .AIL-icon-stack.icon-58  > *:last-child {
				background-size: 29px;
			}

			.AIL-flags .AIL-icon-stack.icon-58  > *.icon-x2:last-child {
				background-size: 43.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-60  > *:last-child {
				background-size: 30px;
			}

			.AIL-flags .AIL-icon-stack.icon-60  > *.icon-x2:last-child {
				background-size: 45px;
			}

			.AIL-flags .AIL-icon-stack.icon-62  > *:last-child {
				background-size: 31px;
			}

			.AIL-flags .AIL-icon-stack.icon-62  > *.icon-x2:last-child {
				background-size: 46.5px;
			}

			.AIL-flags .AIL-icon-stack.icon-64  > *:last-child {
				background-size: 32px;
			}

			.AIL-flags .AIL-icon-stack.icon-64  > *.icon-x2:last-child {
				background-size: 48px;
			}

				.AIL-flags .AIL-icon.icon-flag_slovakia,
				.AIL-flags .AIL-icon-stack .icon-flag_slovakia {
					background-image: url("/portfolio/styles/icon/flags/flag_slovakia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mexico,
				.AIL-flags .AIL-icon-stack .icon-flag_mexico {
					background-image: url("/portfolio/styles/icon/flags/flag_mexico.png")
				}

				.AIL-flags .AIL-icon.icon-flag_turkey,
				.AIL-flags .AIL-icon-stack .icon-flag_turkey {
					background-image: url("/portfolio/styles/icon/flags/flag_turkey.png")
				}

				.AIL-flags .AIL-icon.icon-flag_rainbow,
				.AIL-flags .AIL-icon-stack .icon-flag_rainbow {
					background-image: url("/portfolio/styles/icon/flags/flag_rainbow.png")
				}

				.AIL-flags .AIL-icon.icon-flag_belarus,
				.AIL-flags .AIL-icon-stack .icon-flag_belarus {
					background-image: url("/portfolio/styles/icon/flags/flag_belarus.png")
				}

				.AIL-flags .AIL-icon.icon-flag_togo,
				.AIL-flags .AIL-icon-stack .icon-flag_togo {
					background-image: url("/portfolio/styles/icon/flags/flag_togo.png")
				}

				.AIL-flags .AIL-icon.icon-flag_singapore,
				.AIL-flags .AIL-icon-stack .icon-flag_singapore {
					background-image: url("/portfolio/styles/icon/flags/flag_singapore.png")
				}

				.AIL-flags .AIL-icon.icon-flag_madagascar,
				.AIL-flags .AIL-icon-stack .icon-flag_madagascar {
					background-image: url("/portfolio/styles/icon/flags/flag_madagascar.png")
				}

				.AIL-flags .AIL-icon.icon-flag_qatar,
				.AIL-flags .AIL-icon-stack .icon-flag_qatar {
					background-image: url("/portfolio/styles/icon/flags/flag_qatar.png")
				}

				.AIL-flags .AIL-icon.icon-flag_sao_tome_and_principe,
				.AIL-flags .AIL-icon-stack .icon-flag_sao_tome_and_principe {
					background-image: url("/portfolio/styles/icon/flags/flag_sao_tome_and_principe.png")
				}

				.AIL-flags .AIL-icon.icon-flag_uganda,
				.AIL-flags .AIL-icon-stack .icon-flag_uganda {
					background-image: url("/portfolio/styles/icon/flags/flag_uganda.png")
				}

				.AIL-flags .AIL-icon.icon-flag_yemen,
				.AIL-flags .AIL-icon-stack .icon-flag_yemen {
					background-image: url("/portfolio/styles/icon/flags/flag_yemen.png")
				}

				.AIL-flags .AIL-icon.icon-flag_sudan,
				.AIL-flags .AIL-icon-stack .icon-flag_sudan {
					background-image: url("/portfolio/styles/icon/flags/flag_sudan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_iceland,
				.AIL-flags .AIL-icon-stack .icon-flag_iceland {
					background-image: url("/portfolio/styles/icon/flags/flag_iceland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_namibia,
				.AIL-flags .AIL-icon-stack .icon-flag_namibia {
					background-image: url("/portfolio/styles/icon/flags/flag_namibia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_angola,
				.AIL-flags .AIL-icon-stack .icon-flag_angola {
					background-image: url("/portfolio/styles/icon/flags/flag_angola.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guernsey,
				.AIL-flags .AIL-icon-stack .icon-flag_guernsey {
					background-image: url("/portfolio/styles/icon/flags/flag_guernsey.png")
				}

				.AIL-flags .AIL-icon.icon-flag_wales,
				.AIL-flags .AIL-icon-stack .icon-flag_wales {
					background-image: url("/portfolio/styles/icon/flags/flag_wales.png")
				}

				.AIL-flags .AIL-icon.icon-flag_equatorial_guinea,
				.AIL-flags .AIL-icon-stack .icon-flag_equatorial_guinea {
					background-image: url("/portfolio/styles/icon/flags/flag_equatorial_guinea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_isle_of_man,
				.AIL-flags .AIL-icon-stack .icon-flag_isle_of_man {
					background-image: url("/portfolio/styles/icon/flags/flag_isle_of_man.png")
				}

				.AIL-flags .AIL-icon.icon-flag_iran,
				.AIL-flags .AIL-icon-stack .icon-flag_iran {
					background-image: url("/portfolio/styles/icon/flags/flag_iran.png")
				}

				.AIL-flags .AIL-icon.icon-flag_new_zealand,
				.AIL-flags .AIL-icon-stack .icon-flag_new_zealand {
					background-image: url("/portfolio/styles/icon/flags/flag_new_zealand.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saint_helena,
				.AIL-flags .AIL-icon-stack .icon-flag_saint_helena {
					background-image: url("/portfolio/styles/icon/flags/flag_saint_helena.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bhutan,
				.AIL-flags .AIL-icon-stack .icon-flag_bhutan {
					background-image: url("/portfolio/styles/icon/flags/flag_bhutan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_austria,
				.AIL-flags .AIL-icon-stack .icon-flag_austria {
					background-image: url("/portfolio/styles/icon/flags/flag_austria.png")
				}

				.AIL-flags .AIL-icon.icon-flag_taiwan,
				.AIL-flags .AIL-icon-stack .icon-flag_taiwan {
					background-image: url("/portfolio/styles/icon/flags/flag_taiwan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_south_africa,
				.AIL-flags .AIL-icon-stack .icon-flag_south_africa {
					background-image: url("/portfolio/styles/icon/flags/flag_south_africa.png")
				}

				.AIL-flags .AIL-icon.icon-flag_colombia,
				.AIL-flags .AIL-icon-stack .icon-flag_colombia {
					background-image: url("/portfolio/styles/icon/flags/flag_colombia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guinea_bissau,
				.AIL-flags .AIL-icon-stack .icon-flag_guinea_bissau {
					background-image: url("/portfolio/styles/icon/flags/flag_guinea_bissau.png")
				}

				.AIL-flags .AIL-icon.icon-flag_djibouti,
				.AIL-flags .AIL-icon-stack .icon-flag_djibouti {
					background-image: url("/portfolio/styles/icon/flags/flag_djibouti.png")
				}

				.AIL-flags .AIL-icon.icon-flag_israel,
				.AIL-flags .AIL-icon-stack .icon-flag_israel {
					background-image: url("/portfolio/styles/icon/flags/flag_israel.png")
				}

				.AIL-flags .AIL-icon.icon-flag_anguilla,
				.AIL-flags .AIL-icon-stack .icon-flag_anguilla {
					background-image: url("/portfolio/styles/icon/flags/flag_anguilla.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saudi_arabia,
				.AIL-flags .AIL-icon-stack .icon-flag_saudi_arabia {
					background-image: url("/portfolio/styles/icon/flags/flag_saudi_arabia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_macau,
				.AIL-flags .AIL-icon-stack .icon-flag_macau {
					background-image: url("/portfolio/styles/icon/flags/flag_macau.png")
				}

				.AIL-flags .AIL-icon.icon-flag_uruguay,
				.AIL-flags .AIL-icon-stack .icon-flag_uruguay {
					background-image: url("/portfolio/styles/icon/flags/flag_uruguay.png")
				}

				.AIL-flags .AIL-icon.icon-flag_botswana,
				.AIL-flags .AIL-icon-stack .icon-flag_botswana {
					background-image: url("/portfolio/styles/icon/flags/flag_botswana.png")
				}

				.AIL-flags .AIL-icon.icon-flag_south_korea,
				.AIL-flags .AIL-icon-stack .icon-flag_south_korea {
					background-image: url("/portfolio/styles/icon/flags/flag_south_korea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_italy,
				.AIL-flags .AIL-icon-stack .icon-flag_italy {
					background-image: url("/portfolio/styles/icon/flags/flag_italy.png")
				}

				.AIL-flags .AIL-icon.icon-flag_dominica,
				.AIL-flags .AIL-icon-stack .icon-flag_dominica {
					background-image: url("/portfolio/styles/icon/flags/flag_dominica.png")
				}

				.AIL-flags .AIL-icon.icon-flag_niger,
				.AIL-flags .AIL-icon-stack .icon-flag_niger {
					background-image: url("/portfolio/styles/icon/flags/flag_niger.png")
				}

				.AIL-flags .AIL-icon.icon-flag_hong_kong,
				.AIL-flags .AIL-icon-stack .icon-flag_hong_kong {
					background-image: url("/portfolio/styles/icon/flags/flag_hong_kong.png")
				}

				.AIL-flags .AIL-icon.icon-flag_portugal,
				.AIL-flags .AIL-icon-stack .icon-flag_portugal {
					background-image: url("/portfolio/styles/icon/flags/flag_portugal.png")
				}

				.AIL-flags .AIL-icon.icon-flag_seychelles,
				.AIL-flags .AIL-icon-stack .icon-flag_seychelles {
					background-image: url("/portfolio/styles/icon/flags/flag_seychelles.png")
				}

				.AIL-flags .AIL-icon.icon-flag_nicaragua,
				.AIL-flags .AIL-icon-stack .icon-flag_nicaragua {
					background-image: url("/portfolio/styles/icon/flags/flag_nicaragua.png")
				}

				.AIL-flags .AIL-icon.icon-flag_vatican_city,
				.AIL-flags .AIL-icon-stack .icon-flag_vatican_city {
					background-image: url("/portfolio/styles/icon/flags/flag_vatican_city.png")
				}

				.AIL-flags .AIL-icon.icon-flag_sierra_leone,
				.AIL-flags .AIL-icon-stack .icon-flag_sierra_leone {
					background-image: url("/portfolio/styles/icon/flags/flag_sierra_leone.png")
				}

				.AIL-flags .AIL-icon.icon-flag_indonesia,
				.AIL-flags .AIL-icon-stack .icon-flag_indonesia {
					background-image: url("/portfolio/styles/icon/flags/flag_indonesia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_nepal,
				.AIL-flags .AIL-icon-stack .icon-flag_nepal {
					background-image: url("/portfolio/styles/icon/flags/flag_nepal.png")
				}

				.AIL-flags .AIL-icon.icon-flag_belgium,
				.AIL-flags .AIL-icon-stack .icon-flag_belgium {
					background-image: url("/portfolio/styles/icon/flags/flag_belgium.png")
				}

				.AIL-flags .AIL-icon.icon-flag_syria,
				.AIL-flags .AIL-icon-stack .icon-flag_syria {
					background-image: url("/portfolio/styles/icon/flags/flag_syria.png")
				}

				.AIL-flags .AIL-icon.icon-flag_venezuela,
				.AIL-flags .AIL-icon-stack .icon-flag_venezuela {
					background-image: url("/portfolio/styles/icon/flags/flag_venezuela.png")
				}

				.AIL-flags .AIL-icon.icon-flag_gabon,
				.AIL-flags .AIL-icon-stack .icon-flag_gabon {
					background-image: url("/portfolio/styles/icon/flags/flag_gabon.png")
				}

				.AIL-flags .AIL-icon.icon-flag_belize,
				.AIL-flags .AIL-icon-stack .icon-flag_belize {
					background-image: url("/portfolio/styles/icon/flags/flag_belize.png")
				}

				.AIL-flags .AIL-icon.icon-flag_kiribati,
				.AIL-flags .AIL-icon-stack .icon-flag_kiribati {
					background-image: url("/portfolio/styles/icon/flags/flag_kiribati.png")
				}

				.AIL-flags .AIL-icon.icon-flag_ukraine,
				.AIL-flags .AIL-icon-stack .icon-flag_ukraine {
					background-image: url("/portfolio/styles/icon/flags/flag_ukraine.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guam,
				.AIL-flags .AIL-icon-stack .icon-flag_guam {
					background-image: url("/portfolio/styles/icon/flags/flag_guam.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cayman_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_cayman_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_cayman_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_eu,
				.AIL-flags .AIL-icon-stack .icon-flag_eu {
					background-image: url("/portfolio/styles/icon/flags/flag_eu.png")
				}

				.AIL-flags .AIL-icon.icon-flag_da,
				.AIL-flags .AIL-icon-stack .icon-flag_da {
					background-image: url("/portfolio/styles/icon/flags/flag_da.png")
				}

				.AIL-flags .AIL-icon.icon-flag_united_arab_emirates,
				.AIL-flags .AIL-icon-stack .icon-flag_united_arab_emirates {
					background-image: url("/portfolio/styles/icon/flags/flag_united_arab_emirates.png")
				}

				.AIL-flags .AIL-icon.icon-flag_american_samoa,
				.AIL-flags .AIL-icon-stack .icon-flag_american_samoa {
					background-image: url("/portfolio/styles/icon/flags/flag_american_samoa.png")
				}

				.AIL-flags .AIL-icon.icon-flag_zimbabwe,
				.AIL-flags .AIL-icon-stack .icon-flag_zimbabwe {
					background-image: url("/portfolio/styles/icon/flags/flag_zimbabwe.png")
				}

				.AIL-flags .AIL-icon.icon-flag_jamaica,
				.AIL-flags .AIL-icon-stack .icon-flag_jamaica {
					background-image: url("/portfolio/styles/icon/flags/flag_jamaica.png")
				}

				.AIL-flags .AIL-icon.icon-flag_liberia,
				.AIL-flags .AIL-icon-stack .icon-flag_liberia {
					background-image: url("/portfolio/styles/icon/flags/flag_liberia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_lesotho,
				.AIL-flags .AIL-icon-stack .icon-flag_lesotho {
					background-image: url("/portfolio/styles/icon/flags/flag_lesotho.png")
				}

				.AIL-flags .AIL-icon.icon-flag_eritrea,
				.AIL-flags .AIL-icon-stack .icon-flag_eritrea {
					background-image: url("/portfolio/styles/icon/flags/flag_eritrea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_egypt,
				.AIL-flags .AIL-icon-stack .icon-flag_egypt {
					background-image: url("/portfolio/styles/icon/flags/flag_egypt.png")
				}

				.AIL-flags .AIL-icon.icon-flag_en,
				.AIL-flags .AIL-icon-stack .icon-flag_en {
					background-image: url("/portfolio/styles/icon/flags/flag_en.png")
				}

				.AIL-flags .AIL-icon.icon-flag_senegal,
				.AIL-flags .AIL-icon-stack .icon-flag_senegal {
					background-image: url("/portfolio/styles/icon/flags/flag_senegal.png")
				}

				.AIL-flags .AIL-icon.icon-flag_ireland,
				.AIL-flags .AIL-icon-stack .icon-flag_ireland {
					background-image: url("/portfolio/styles/icon/flags/flag_ireland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_sri_lanka,
				.AIL-flags .AIL-icon-stack .icon-flag_sri_lanka {
					background-image: url("/portfolio/styles/icon/flags/flag_sri_lanka.png")
				}

				.AIL-flags .AIL-icon.icon-flag_peru,
				.AIL-flags .AIL-icon-stack .icon-flag_peru {
					background-image: url("/portfolio/styles/icon/flags/flag_peru.png")
				}

				.AIL-flags .AIL-icon.icon-flag_norfolk_island,
				.AIL-flags .AIL-icon-stack .icon-flag_norfolk_island {
					background-image: url("/portfolio/styles/icon/flags/flag_norfolk_island.png")
				}

				.AIL-flags .AIL-icon.icon-flag_monaco,
				.AIL-flags .AIL-icon-stack .icon-flag_monaco {
					background-image: url("/portfolio/styles/icon/flags/flag_monaco.png")
				}

				.AIL-flags .AIL-icon.icon-flag_central_african_republic,
				.AIL-flags .AIL-icon-stack .icon-flag_central_african_republic {
					background-image: url("/portfolio/styles/icon/flags/flag_central_african_republic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_kyrgyzstan,
				.AIL-flags .AIL-icon-stack .icon-flag_kyrgyzstan {
					background-image: url("/portfolio/styles/icon/flags/flag_kyrgyzstan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_dominican_republic,
				.AIL-flags .AIL-icon-stack .icon-flag_dominican_republic {
					background-image: url("/portfolio/styles/icon/flags/flag_dominican_republic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_pakistan,
				.AIL-flags .AIL-icon-stack .icon-flag_pakistan {
					background-image: url("/portfolio/styles/icon/flags/flag_pakistan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_burundi,
				.AIL-flags .AIL-icon-stack .icon-flag_burundi {
					background-image: url("/portfolio/styles/icon/flags/flag_burundi.png")
				}

				.AIL-flags .AIL-icon.icon-flag_east_timor,
				.AIL-flags .AIL-icon-stack .icon-flag_east_timor {
					background-image: url("/portfolio/styles/icon/flags/flag_east_timor.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cyprus,
				.AIL-flags .AIL-icon-stack .icon-flag_cyprus {
					background-image: url("/portfolio/styles/icon/flags/flag_cyprus.png")
				}

				.AIL-flags .AIL-icon.icon-flag_china,
				.AIL-flags .AIL-icon-stack .icon-flag_china {
					background-image: url("/portfolio/styles/icon/flags/flag_china.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saint_vincent_and_the_grenadines,
				.AIL-flags .AIL-icon-stack .icon-flag_saint_vincent_and_the_grenadines {
					background-image: url("/portfolio/styles/icon/flags/flag_saint_vincent_and_the_grenadines.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cook_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_cook_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_cook_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_martinique,
				.AIL-flags .AIL-icon-stack .icon-flag_martinique {
					background-image: url("/portfolio/styles/icon/flags/flag_martinique.png")
				}

				.AIL-flags .AIL-icon.icon-flag_palau,
				.AIL-flags .AIL-icon-stack .icon-flag_palau {
					background-image: url("/portfolio/styles/icon/flags/flag_palau.png")
				}

				.AIL-flags .AIL-icon.icon-flag_oman,
				.AIL-flags .AIL-icon-stack .icon-flag_oman {
					background-image: url("/portfolio/styles/icon/flags/flag_oman.png")
				}

				.AIL-flags .AIL-icon.icon-flag_wallis_and_futuna,
				.AIL-flags .AIL-icon-stack .icon-flag_wallis_and_futuna {
					background-image: url("/portfolio/styles/icon/flags/flag_wallis_and_futuna.png")
				}

				.AIL-flags .AIL-icon.icon-flag_uzbekistan,
				.AIL-flags .AIL-icon-stack .icon-flag_uzbekistan {
					background-image: url("/portfolio/styles/icon/flags/flag_uzbekistan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_poland,
				.AIL-flags .AIL-icon-stack .icon-flag_poland {
					background-image: url("/portfolio/styles/icon/flags/flag_poland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_es,
				.AIL-flags .AIL-icon-stack .icon-flag_es {
					background-image: url("/portfolio/styles/icon/flags/flag_es.png")
				}

				.AIL-flags .AIL-icon.icon-flag_british_virgin_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_british_virgin_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_british_virgin_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_ecuador,
				.AIL-flags .AIL-icon-stack .icon-flag_ecuador {
					background-image: url("/portfolio/styles/icon/flags/flag_ecuador.png")
				}

				.AIL-flags .AIL-icon.icon-flag_fiji,
				.AIL-flags .AIL-icon-stack .icon-flag_fiji {
					background-image: url("/portfolio/styles/icon/flags/flag_fiji.png")
				}

				.AIL-flags .AIL-icon.icon-flag_usa,
				.AIL-flags .AIL-icon-stack .icon-flag_usa {
					background-image: url("/portfolio/styles/icon/flags/flag_usa.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mali,
				.AIL-flags .AIL-icon-stack .icon-flag_mali {
					background-image: url("/portfolio/styles/icon/flags/flag_mali.png")
				}

				.AIL-flags .AIL-icon.icon-flag_antigua_and_barbuda,
				.AIL-flags .AIL-icon-stack .icon-flag_antigua_and_barbuda {
					background-image: url("/portfolio/styles/icon/flags/flag_antigua_and_barbuda.png")
				}

				.AIL-flags .AIL-icon.icon-flag_generic,
				.AIL-flags .AIL-icon-stack .icon-flag_generic {
					background-image: url("/portfolio/styles/icon/flags/flag_generic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bosnia_and_herzegovina,
				.AIL-flags .AIL-icon-stack .icon-flag_bosnia_and_herzegovina {
					background-image: url("/portfolio/styles/icon/flags/flag_bosnia_and_herzegovina.png")
				}

				.AIL-flags .AIL-icon.icon-flag_marshall_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_marshall_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_marshall_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_northern_mariana_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_northern_mariana_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_northern_mariana_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_netherlands_antilles,
				.AIL-flags .AIL-icon-stack .icon-flag_netherlands_antilles {
					background-image: url("/portfolio/styles/icon/flags/flag_netherlands_antilles.png")
				}

				.AIL-flags .AIL-icon.icon-flag_turkmenistan,
				.AIL-flags .AIL-icon-stack .icon-flag_turkmenistan {
					background-image: url("/portfolio/styles/icon/flags/flag_turkmenistan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_burma,
				.AIL-flags .AIL-icon-stack .icon-flag_burma {
					background-image: url("/portfolio/styles/icon/flags/flag_burma.png")
				}

				.AIL-flags .AIL-icon.icon-flag_british_indian_ocean_territory,
				.AIL-flags .AIL-icon-stack .icon-flag_british_indian_ocean_territory {
					background-image: url("/portfolio/styles/icon/flags/flag_british_indian_ocean_territory.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guinea,
				.AIL-flags .AIL-icon-stack .icon-flag_guinea {
					background-image: url("/portfolio/styles/icon/flags/flag_guinea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_latvia,
				.AIL-flags .AIL-icon-stack .icon-flag_latvia {
					background-image: url("/portfolio/styles/icon/flags/flag_latvia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_algeria,
				.AIL-flags .AIL-icon-stack .icon-flag_algeria {
					background-image: url("/portfolio/styles/icon/flags/flag_algeria.png")
				}

				.AIL-flags .AIL-icon.icon-flag_armenia,
				.AIL-flags .AIL-icon-stack .icon-flag_armenia {
					background-image: url("/portfolio/styles/icon/flags/flag_armenia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tajikistan,
				.AIL-flags .AIL-icon-stack .icon-flag_tajikistan {
					background-image: url("/portfolio/styles/icon/flags/flag_tajikistan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_india,
				.AIL-flags .AIL-icon-stack .icon-flag_india {
					background-image: url("/portfolio/styles/icon/flags/flag_india.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guyana,
				.AIL-flags .AIL-icon-stack .icon-flag_guyana {
					background-image: url("/portfolio/styles/icon/flags/flag_guyana.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saint_pierre_and_miquelon,
				.AIL-flags .AIL-icon-stack .icon-flag_saint_pierre_and_miquelon {
					background-image: url("/portfolio/styles/icon/flags/flag_saint_pierre_and_miquelon.png")
				}

				.AIL-flags .AIL-icon.icon-flag_jordan,
				.AIL-flags .AIL-icon-stack .icon-flag_jordan {
					background-image: url("/portfolio/styles/icon/flags/flag_jordan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_australia,
				.AIL-flags .AIL-icon-stack .icon-flag_australia {
					background-image: url("/portfolio/styles/icon/flags/flag_australia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mozambique,
				.AIL-flags .AIL-icon-stack .icon-flag_mozambique {
					background-image: url("/portfolio/styles/icon/flags/flag_mozambique.png")
				}

				.AIL-flags .AIL-icon.icon-flag_no,
				.AIL-flags .AIL-icon-stack .icon-flag_no {
					background-image: url("/portfolio/styles/icon/flags/flag_no.png")
				}

				.AIL-flags .AIL-icon.icon-flag_morocco,
				.AIL-flags .AIL-icon-stack .icon-flag_morocco {
					background-image: url("/portfolio/styles/icon/flags/flag_morocco.png")
				}

				.AIL-flags .AIL-icon.icon-flag_afghanistan,
				.AIL-flags .AIL-icon-stack .icon-flag_afghanistan {
					background-image: url("/portfolio/styles/icon/flags/flag_afghanistan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_comoros,
				.AIL-flags .AIL-icon-stack .icon-flag_comoros {
					background-image: url("/portfolio/styles/icon/flags/flag_comoros.png")
				}

				.AIL-flags .AIL-icon.icon-flag_libya,
				.AIL-flags .AIL-icon-stack .icon-flag_libya {
					background-image: url("/portfolio/styles/icon/flags/flag_libya.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saint_kitts_and_nevis,
				.AIL-flags .AIL-icon-stack .icon-flag_saint_kitts_and_nevis {
					background-image: url("/portfolio/styles/icon/flags/flag_saint_kitts_and_nevis.png")
				}

				.AIL-flags .AIL-icon.icon-flag_pitcairn_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_pitcairn_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_pitcairn_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_lithuania,
				.AIL-flags .AIL-icon-stack .icon-flag_lithuania {
					background-image: url("/portfolio/styles/icon/flags/flag_lithuania.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bahamas,
				.AIL-flags .AIL-icon-stack .icon-flag_bahamas {
					background-image: url("/portfolio/styles/icon/flags/flag_bahamas.png")
				}

				.AIL-flags .AIL-icon.icon-flag_gambia,
				.AIL-flags .AIL-icon-stack .icon-flag_gambia {
					background-image: url("/portfolio/styles/icon/flags/flag_gambia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_albania,
				.AIL-flags .AIL-icon-stack .icon-flag_albania {
					background-image: url("/portfolio/styles/icon/flags/flag_albania.png")
				}

				.AIL-flags .AIL-icon.icon-flag_south_georgia,
				.AIL-flags .AIL-icon-stack .icon-flag_south_georgia {
					background-image: url("/portfolio/styles/icon/flags/flag_south_georgia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_maldives,
				.AIL-flags .AIL-icon-stack .icon-flag_maldives {
					background-image: url("/portfolio/styles/icon/flags/flag_maldives.png")
				}

				.AIL-flags .AIL-icon.icon-flag_malawi,
				.AIL-flags .AIL-icon-stack .icon-flag_malawi {
					background-image: url("/portfolio/styles/icon/flags/flag_malawi.png")
				}

				.AIL-flags .AIL-icon.icon-flag_greece,
				.AIL-flags .AIL-icon-stack .icon-flag_greece {
					background-image: url("/portfolio/styles/icon/flags/flag_greece.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bahrain,
				.AIL-flags .AIL-icon-stack .icon-flag_bahrain {
					background-image: url("/portfolio/styles/icon/flags/flag_bahrain.png")
				}

				.AIL-flags .AIL-icon.icon-flag_haiti,
				.AIL-flags .AIL-icon-stack .icon-flag_haiti {
					background-image: url("/portfolio/styles/icon/flags/flag_haiti.png")
				}

				.AIL-flags .AIL-icon.icon-flag_falkland_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_falkland_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_falkland_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cote_divoire,
				.AIL-flags .AIL-icon-stack .icon-flag_cote_divoire {
					background-image: url("/portfolio/styles/icon/flags/flag_cote_divoire.png")
				}

				.AIL-flags .AIL-icon.icon-flag_grenada,
				.AIL-flags .AIL-icon-stack .icon-flag_grenada {
					background-image: url("/portfolio/styles/icon/flags/flag_grenada.png")
				}

				.AIL-flags .AIL-icon.icon-flag_jersey,
				.AIL-flags .AIL-icon-stack .icon-flag_jersey {
					background-image: url("/portfolio/styles/icon/flags/flag_jersey.png")
				}

				.AIL-flags .AIL-icon.icon-flag_virgin_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_virgin_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_virgin_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_somalia,
				.AIL-flags .AIL-icon-stack .icon-flag_somalia {
					background-image: url("/portfolio/styles/icon/flags/flag_somalia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tanzania,
				.AIL-flags .AIL-icon-stack .icon-flag_tanzania {
					background-image: url("/portfolio/styles/icon/flags/flag_tanzania.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cameroon,
				.AIL-flags .AIL-icon-stack .icon-flag_cameroon {
					background-image: url("/portfolio/styles/icon/flags/flag_cameroon.png")
				}

				.AIL-flags .AIL-icon.icon-flag_japan,
				.AIL-flags .AIL-icon-stack .icon-flag_japan {
					background-image: url("/portfolio/styles/icon/flags/flag_japan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_micronesia,
				.AIL-flags .AIL-icon-stack .icon-flag_micronesia {
					background-image: url("/portfolio/styles/icon/flags/flag_micronesia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_vanuatu,
				.AIL-flags .AIL-icon-stack .icon-flag_vanuatu {
					background-image: url("/portfolio/styles/icon/flags/flag_vanuatu.png")
				}

				.AIL-flags .AIL-icon.icon-flag_scotland,
				.AIL-flags .AIL-icon-stack .icon-flag_scotland {
					background-image: url("/portfolio/styles/icon/flags/flag_scotland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_samoa,
				.AIL-flags .AIL-icon-stack .icon-flag_samoa {
					background-image: url("/portfolio/styles/icon/flags/flag_samoa.png")
				}

				.AIL-flags .AIL-icon.icon-flag_romania,
				.AIL-flags .AIL-icon-stack .icon-flag_romania {
					background-image: url("/portfolio/styles/icon/flags/flag_romania.png")
				}

				.AIL-flags .AIL-icon.icon-flag_malaysia,
				.AIL-flags .AIL-icon-stack .icon-flag_malaysia {
					background-image: url("/portfolio/styles/icon/flags/flag_malaysia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_gibraltar,
				.AIL-flags .AIL-icon-stack .icon-flag_gibraltar {
					background-image: url("/portfolio/styles/icon/flags/flag_gibraltar.png")
				}

				.AIL-flags .AIL-icon.icon-flag_serbia,
				.AIL-flags .AIL-icon-stack .icon-flag_serbia {
					background-image: url("/portfolio/styles/icon/flags/flag_serbia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_kenya,
				.AIL-flags .AIL-icon-stack .icon-flag_kenya {
					background-image: url("/portfolio/styles/icon/flags/flag_kenya.png")
				}

				.AIL-flags .AIL-icon.icon-flag_rwanda,
				.AIL-flags .AIL-icon-stack .icon-flag_rwanda {
					background-image: url("/portfolio/styles/icon/flags/flag_rwanda.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mauritius,
				.AIL-flags .AIL-icon-stack .icon-flag_mauritius {
					background-image: url("/portfolio/styles/icon/flags/flag_mauritius.png")
				}

				.AIL-flags .AIL-icon.icon-flag_white,
				.AIL-flags .AIL-icon-stack .icon-flag_white {
					background-image: url("/portfolio/styles/icon/flags/flag_white.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bulgaria,
				.AIL-flags .AIL-icon-stack .icon-flag_bulgaria {
					background-image: url("/portfolio/styles/icon/flags/flag_bulgaria.png")
				}

				.AIL-flags .AIL-icon.icon-flag_nauru,
				.AIL-flags .AIL-icon-stack .icon-flag_nauru {
					background-image: url("/portfolio/styles/icon/flags/flag_nauru.png")
				}

				.AIL-flags .AIL-icon.icon-flag_thailand,
				.AIL-flags .AIL-icon-stack .icon-flag_thailand {
					background-image: url("/portfolio/styles/icon/flags/flag_thailand.png")
				}

				.AIL-flags .AIL-icon.icon-flag_turks_and_caicos_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_turks_and_caicos_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_turks_and_caicos_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_solomon_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_solomon_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_solomon_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cape_verde,
				.AIL-flags .AIL-icon-stack .icon-flag_cape_verde {
					background-image: url("/portfolio/styles/icon/flags/flag_cape_verde.png")
				}

				.AIL-flags .AIL-icon.icon-flag_kazakhstan,
				.AIL-flags .AIL-icon-stack .icon-flag_kazakhstan {
					background-image: url("/portfolio/styles/icon/flags/flag_kazakhstan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_azerbaijan,
				.AIL-flags .AIL-icon-stack .icon-flag_azerbaijan {
					background-image: url("/portfolio/styles/icon/flags/flag_azerbaijan.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mongolia,
				.AIL-flags .AIL-icon-stack .icon-flag_mongolia {
					background-image: url("/portfolio/styles/icon/flags/flag_mongolia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_macedonia,
				.AIL-flags .AIL-icon-stack .icon-flag_macedonia {
					background-image: url("/portfolio/styles/icon/flags/flag_macedonia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_swaziland,
				.AIL-flags .AIL-icon-stack .icon-flag_swaziland {
					background-image: url("/portfolio/styles/icon/flags/flag_swaziland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_paraguay,
				.AIL-flags .AIL-icon-stack .icon-flag_paraguay {
					background-image: url("/portfolio/styles/icon/flags/flag_paraguay.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bolivia,
				.AIL-flags .AIL-icon-stack .icon-flag_bolivia {
					background-image: url("/portfolio/styles/icon/flags/flag_bolivia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_croatia,
				.AIL-flags .AIL-icon-stack .icon-flag_croatia {
					background-image: url("/portfolio/styles/icon/flags/flag_croatia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_el_salvador,
				.AIL-flags .AIL-icon-stack .icon-flag_el_salvador {
					background-image: url("/portfolio/styles/icon/flags/flag_el_salvador.png")
				}

				.AIL-flags .AIL-icon.icon-flag_de,
				.AIL-flags .AIL-icon-stack .icon-flag_de {
					background-image: url("/portfolio/styles/icon/flags/flag_de.png")
				}

				.AIL-flags .AIL-icon.icon-flag_switzerland,
				.AIL-flags .AIL-icon-stack .icon-flag_switzerland {
					background-image: url("/portfolio/styles/icon/flags/flag_switzerland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_malta,
				.AIL-flags .AIL-icon-stack .icon-flag_malta {
					background-image: url("/portfolio/styles/icon/flags/flag_malta.png")
				}

				.AIL-flags .AIL-icon.icon-flag_fr,
				.AIL-flags .AIL-icon-stack .icon-flag_fr {
					background-image: url("/portfolio/styles/icon/flags/flag_fr.png")
				}

				.AIL-flags .AIL-icon.icon-flag_french_polynesia,
				.AIL-flags .AIL-icon-stack .icon-flag_french_polynesia {
					background-image: url("/portfolio/styles/icon/flags/flag_french_polynesia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bermuda,
				.AIL-flags .AIL-icon-stack .icon-flag_bermuda {
					background-image: url("/portfolio/styles/icon/flags/flag_bermuda.png")
				}

				.AIL-flags .AIL-icon.icon-flag_kuwait,
				.AIL-flags .AIL-icon-stack .icon-flag_kuwait {
					background-image: url("/portfolio/styles/icon/flags/flag_kuwait.png")
				}

				.AIL-flags .AIL-icon.icon-flag_russia,
				.AIL-flags .AIL-icon-stack .icon-flag_russia {
					background-image: url("/portfolio/styles/icon/flags/flag_russia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cambodia,
				.AIL-flags .AIL-icon-stack .icon-flag_cambodia {
					background-image: url("/portfolio/styles/icon/flags/flag_cambodia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_chile,
				.AIL-flags .AIL-icon-stack .icon-flag_chile {
					background-image: url("/portfolio/styles/icon/flags/flag_chile.png")
				}

				.AIL-flags .AIL-icon.icon-flag_slovenia,
				.AIL-flags .AIL-icon-stack .icon-flag_slovenia {
					background-image: url("/portfolio/styles/icon/flags/flag_slovenia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_czech_republic,
				.AIL-flags .AIL-icon-stack .icon-flag_czech_republic {
					background-image: url("/portfolio/styles/icon/flags/flag_czech_republic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_iraq,
				.AIL-flags .AIL-icon-stack .icon-flag_iraq {
					background-image: url("/portfolio/styles/icon/flags/flag_iraq.png")
				}

				.AIL-flags .AIL-icon.icon-flag_lebanon,
				.AIL-flags .AIL-icon-stack .icon-flag_lebanon {
					background-image: url("/portfolio/styles/icon/flags/flag_lebanon.png")
				}

				.AIL-flags .AIL-icon.icon-flag_barbados,
				.AIL-flags .AIL-icon-stack .icon-flag_barbados {
					background-image: url("/portfolio/styles/icon/flags/flag_barbados.png")
				}

				.AIL-flags .AIL-icon.icon-flag_moldova,
				.AIL-flags .AIL-icon-stack .icon-flag_moldova {
					background-image: url("/portfolio/styles/icon/flags/flag_moldova.png")
				}

				.AIL-flags .AIL-icon.icon-flag_ghana,
				.AIL-flags .AIL-icon-stack .icon-flag_ghana {
					background-image: url("/portfolio/styles/icon/flags/flag_ghana.png")
				}

				.AIL-flags .AIL-icon.icon-flag_pirate,
				.AIL-flags .AIL-icon-stack .icon-flag_pirate {
					background-image: url("/portfolio/styles/icon/flags/flag_pirate.png")
				}

				.AIL-flags .AIL-icon.icon-flag_faeroe_islands,
				.AIL-flags .AIL-icon-stack .icon-flag_faeroe_islands {
					background-image: url("/portfolio/styles/icon/flags/flag_faeroe_islands.png")
				}

				.AIL-flags .AIL-icon.icon-flag_puerto_rico,
				.AIL-flags .AIL-icon-stack .icon-flag_puerto_rico {
					background-image: url("/portfolio/styles/icon/flags/flag_puerto_rico.png")
				}

				.AIL-flags .AIL-icon.icon-flag_ethiopia,
				.AIL-flags .AIL-icon-stack .icon-flag_ethiopia {
					background-image: url("/portfolio/styles/icon/flags/flag_ethiopia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_mauritania,
				.AIL-flags .AIL-icon-stack .icon-flag_mauritania {
					background-image: url("/portfolio/styles/icon/flags/flag_mauritania.png")
				}

				.AIL-flags .AIL-icon.icon-flag_trinidad_and_tobago,
				.AIL-flags .AIL-icon-stack .icon-flag_trinidad_and_tobago {
					background-image: url("/portfolio/styles/icon/flags/flag_trinidad_and_tobago.png")
				}

				.AIL-flags .AIL-icon.icon-flag_nigeria,
				.AIL-flags .AIL-icon-stack .icon-flag_nigeria {
					background-image: url("/portfolio/styles/icon/flags/flag_nigeria.png")
				}

				.AIL-flags .AIL-icon.icon-flag_niue,
				.AIL-flags .AIL-icon-stack .icon-flag_niue {
					background-image: url("/portfolio/styles/icon/flags/flag_niue.png")
				}

				.AIL-flags .AIL-icon.icon-flag_luxembourg,
				.AIL-flags .AIL-icon-stack .icon-flag_luxembourg {
					background-image: url("/portfolio/styles/icon/flags/flag_luxembourg.png")
				}

				.AIL-flags .AIL-icon.icon-flag_france,
				.AIL-flags .AIL-icon-stack .icon-flag_france {
					background-image: url("/portfolio/styles/icon/flags/flag_fr.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tuvalu,
				.AIL-flags .AIL-icon-stack .icon-flag_tuvalu {
					background-image: url("/portfolio/styles/icon/flags/flag_tuvalu.png")
				}

				.AIL-flags .AIL-icon.icon-flag_costa_rica,
				.AIL-flags .AIL-icon-stack .icon-flag_costa_rica {
					background-image: url("/portfolio/styles/icon/flags/flag_costa_rica.png")
				}

				.AIL-flags .AIL-icon.icon-flag_burkina_faso,
				.AIL-flags .AIL-icon-stack .icon-flag_burkina_faso {
					background-image: url("/portfolio/styles/icon/flags/flag_burkina_faso.png")
				}

				.AIL-flags .AIL-icon.icon-flag_england,
				.AIL-flags .AIL-icon-stack .icon-flag_england {
					background-image: url("/portfolio/styles/icon/flags/flag_england.png")
				}

				.AIL-flags .AIL-icon.icon-flag_fi,
				.AIL-flags .AIL-icon-stack .icon-flag_fi {
					background-image: url("/portfolio/styles/icon/flags/flag_finland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_finland,
				.AIL-flags .AIL-icon-stack .icon-flag_finland {
					background-image: url("/portfolio/styles/icon/flags/flag_finland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_checkered,
				.AIL-flags .AIL-icon-stack .icon-flag_checkered {
					background-image: url("/portfolio/styles/icon/flags/flag_checkered.png")
				}

				.AIL-flags .AIL-icon.icon-flag_congo_republic,
				.AIL-flags .AIL-icon-stack .icon-flag_congo_republic {
					background-image: url("/portfolio/styles/icon/flags/flag_congo_republic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tibet,
				.AIL-flags .AIL-icon-stack .icon-flag_tibet {
					background-image: url("/portfolio/styles/icon/flags/flag_tibet.png")
				}

				.AIL-flags .AIL-icon.icon-flag_vietnam,
				.AIL-flags .AIL-icon-stack .icon-flag_vietnam {
					background-image: url("/portfolio/styles/icon/flags/flag_vietnam.png")
				}

				.AIL-flags .AIL-icon.icon-flag_estonia,
				.AIL-flags .AIL-icon-stack .icon-flag_estonia {
					background-image: url("/portfolio/styles/icon/flags/flag_estonia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tunisia,
				.AIL-flags .AIL-icon-stack .icon-flag_tunisia {
					background-image: url("/portfolio/styles/icon/flags/flag_tunisia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_zambia,
				.AIL-flags .AIL-icon-stack .icon-flag_zambia {
					background-image: url("/portfolio/styles/icon/flags/flag_zambia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_tonga,
				.AIL-flags .AIL-icon-stack .icon-flag_tonga {
					background-image: url("/portfolio/styles/icon/flags/flag_tonga.png")
				}

				.AIL-flags .AIL-icon.icon-flag_guatemala,
				.AIL-flags .AIL-icon-stack .icon-flag_guatemala {
					background-image: url("/portfolio/styles/icon/flags/flag_guatemala.png")
				}

				.AIL-flags .AIL-icon.icon-flag_brunei,
				.AIL-flags .AIL-icon-stack .icon-flag_brunei {
					background-image: url("/portfolio/styles/icon/flags/flag_brunei.png")
				}

				.AIL-flags .AIL-icon.icon-flag_brazil,
				.AIL-flags .AIL-icon-stack .icon-flag_brazil {
					background-image: url("/portfolio/styles/icon/flags/flag_brazil.png")
				}

				.AIL-flags .AIL-icon.icon-flag_montenegro,
				.AIL-flags .AIL-icon-stack .icon-flag_montenegro {
					background-image: url("/portfolio/styles/icon/flags/flag_montenegro.png")
				}

				.AIL-flags .AIL-icon.icon-flag_san_marino,
				.AIL-flags .AIL-icon-stack .icon-flag_san_marino {
					background-image: url("/portfolio/styles/icon/flags/flag_san_marino.png")
				}

				.AIL-flags .AIL-icon.icon-flag_panama,
				.AIL-flags .AIL-icon-stack .icon-flag_panama {
					background-image: url("/portfolio/styles/icon/flags/flag_panama.png")
				}

				.AIL-flags .AIL-icon.icon-flag_north_korea,
				.AIL-flags .AIL-icon-stack .icon-flag_north_korea {
					background-image: url("/portfolio/styles/icon/flags/flag_north_korea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_papua_new_guinea,
				.AIL-flags .AIL-icon-stack .icon-flag_papua_new_guinea {
					background-image: url("/portfolio/styles/icon/flags/flag_papua_new_guinea.png")
				}

				.AIL-flags .AIL-icon.icon-flag_honduras,
				.AIL-flags .AIL-icon-stack .icon-flag_honduras {
					background-image: url("/portfolio/styles/icon/flags/flag_honduras.png")
				}

				.AIL-flags .AIL-icon.icon-flag_suriname,
				.AIL-flags .AIL-icon-stack .icon-flag_suriname {
					background-image: url("/portfolio/styles/icon/flags/flag_suriname.png")
				}

				.AIL-flags .AIL-icon.icon-flag_andorra,
				.AIL-flags .AIL-icon-stack .icon-flag_andorra {
					background-image: url("/portfolio/styles/icon/flags/flag_andorra.png")
				}

				.AIL-flags .AIL-icon.icon-flag_nl,
				.AIL-flags .AIL-icon-stack .icon-flag_nl {
					background-image: url("/portfolio/styles/icon/flags/flag_nl.png")
				}

				.AIL-flags .AIL-icon.icon-flag_hungary,
				.AIL-flags .AIL-icon-stack .icon-flag_hungary {
					background-image: url("/portfolio/styles/icon/flags/flag_hungary.png")
				}

				.AIL-flags .AIL-icon.icon-flag_chad,
				.AIL-flags .AIL-icon-stack .icon-flag_chad {
					background-image: url("/portfolio/styles/icon/flags/flag_chad.png")
				}

				.AIL-flags .AIL-icon.icon-flag_georgia,
				.AIL-flags .AIL-icon-stack .icon-flag_georgia {
					background-image: url("/portfolio/styles/icon/flags/flag_georgia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_laos,
				.AIL-flags .AIL-icon-stack .icon-flag_laos {
					background-image: url("/portfolio/styles/icon/flags/flag_laos.png")
				}

				.AIL-flags .AIL-icon.icon-flag_saint_lucia,
				.AIL-flags .AIL-icon-stack .icon-flag_saint_lucia {
					background-image: url("/portfolio/styles/icon/flags/flag_saint_lucia.png")
				}

				.AIL-flags .AIL-icon.icon-flag_cuba,
				.AIL-flags .AIL-icon-stack .icon-flag_cuba {
					background-image: url("/portfolio/styles/icon/flags/flag_cuba.png")
				}

				.AIL-flags .AIL-icon.icon-flag_bangladesh,
				.AIL-flags .AIL-icon-stack .icon-flag_bangladesh {
					background-image: url("/portfolio/styles/icon/flags/flag_bangladesh.png")
				}

				.AIL-flags .AIL-icon.icon-flag_benin,
				.AIL-flags .AIL-icon-stack .icon-flag_benin {
					background-image: url("/portfolio/styles/icon/flags/flag_benin.png")
				}

				.AIL-flags .AIL-icon.icon-flag_argentina,
				.AIL-flags .AIL-icon-stack .icon-flag_argentina {
					background-image: url("/portfolio/styles/icon/flags/flag_argentina.png")
				}

				.AIL-flags .AIL-icon.icon-flag_philippines,
				.AIL-flags .AIL-icon-stack .icon-flag_philippines {
					background-image: url("/portfolio/styles/icon/flags/flag_philippines.png")
				}

				.AIL-flags .AIL-icon.icon-flag_greenland,
				.AIL-flags .AIL-icon-stack .icon-flag_greenland {
					background-image: url("/portfolio/styles/icon/flags/flag_greenland.png")
				}

				.AIL-flags .AIL-icon.icon-flag_congo_democratic_republic,
				.AIL-flags .AIL-icon-stack .icon-flag_congo_democratic_republic {
					background-image: url("/portfolio/styles/icon/flags/flag_congo_democratic_republic.png")
				}

				.AIL-flags .AIL-icon.icon-flag_canada,
				.AIL-flags .AIL-icon-stack .icon-flag_canada {
					background-image: url("/portfolio/styles/icon/flags/flag_canada.png")
				}

				.AIL-flags .AIL-icon.icon-flag_liechtenstein,
				.AIL-flags .AIL-icon-stack .icon-flag_liechtenstein {
					background-image: url("/portfolio/styles/icon/flags/flag_liechtenstein.png")
				}

				.AIL-flags .AIL-icon.icon-flag_montserrat,
				.AIL-flags .AIL-icon-stack .icon-flag_montserrat {
					background-image: url("/portfolio/styles/icon/flags/flag_montserrat.png")
				}

				.AIL-flags .AIL-icon.icon-flag_sv,
				.AIL-flags .AIL-icon-stack .icon-flag_sv {
					background-image: url("/portfolio/styles/icon/flags/flag_sv.png")
				}

				.AIL-flags .AIL-icon.icon-flag_aruba,
				.AIL-flags .AIL-icon-stack .icon-flag_aruba {
					background-image: url("/portfolio/styles/icon/flags/flag_aruba.png")
				}

